package jade.db;

/* This class is an adaptor object for communicating with the database.
   It handles all the messy socket stuff. */

import java.io.*;
import java.net.*;
import java.util.*;

// ====================================================================
// Package-only utility class used by the Accessor object

//** Is this a general Pool for Accessor? **//
//** Consider to use Factory Method pattern **//

final class AccessorPool {

  private String theHost;
  private int thePort;
  private int lastUsed;
  private int numberActive;
  private AceAccessor[] theList;
  AccessorPool(String host, int port, int maxAccessors) {
	theHost = host;
	thePort = port;
	theList = new AceAccessor[maxAccessors];
	lastUsed = 0;
	numberActive = 0;
  }  
  synchronized AceAccessor createOrPick(boolean createCache) throws IOException {
	if (numberActive < theList.length) {
	  AceAccessor newA = new AceAccessor(theHost,thePort,false,createCache);
	  if (newA.good) {
	theList[numberActive] = newA;
	lastUsed = numberActive++;
	return newA;
	  }
	}

	//** Can we get here? **//
	if (numberActive <= 0)
	  return null;

	// if we get here then either we've used up our maximum number of
	// accessors, or something went wrong when allocating a new one.

	//** Is it necessary to find a good AceAccessor by a loop? **//
	
	lastUsed++;
	if (lastUsed >= numberActive)
	  lastUsed = 0;
	AceAccessor oldA = theList[lastUsed];

	// Try (once) to reallocate the file handle if something bad happened
	// to it (like a server timeout).
	if (!oldA.good) {
	  try {
	oldA.rebuildConnection();
	  } catch (IOException e) {
	return null;
	  }
	}
	return oldA;
  }      
}