package jade.db;

/* This class is an adaptor object for communicating with the database.
   It handles all the messy socket stuff. */

import java.io.*;
import java.net.*;
import java.util.*;


public class AceAccessor extends Accessor {
  
  // ============= public state variables =================
  // You can use these two streams to communicate
  // directly with the database.  However you must
  // beware of interfering with other objects that
  // are trying to use the same accessor (in other
  // words, make sure to send a newline promptly!)
  public boolean good = false;
  public boolean echoMode = false ;
  public int totalCount = 0 ;

  // =========== public numeric constants  =============
  static public int OK=200;
  static public int GOODBYE=201;
  static public int DEBUG=301;
  static public int COMMENT=302;
  static public int REDIRECT=303;
  static public int PARAMERROR=401;
  static public int SYNTAXERROR=402;
  static public int UNIMPLEMENTED=403;
  static public int TIMEOUTERROR=501;
  static public int COMMERROR=502;
  static public int MEMERROR=503;
  static public int FILEERROR=504;
  
  // Open this many maximum simultaneous connections to
  // the server.  If connections time out, new ones are
  // automatically reopened.  Use the dup() command to
  // obtain either a new database connection or a reused
  // one.
  static public int MAX_CONN=3;
  static public int CACHE_MIN=1000;
  static public int CACHE_MAX=1500;

  private File fil ;

  // ================== protected instance variables =============
  // =============================================================
  // These are intended to be accessed by AceObjects and other
  // entities that know what they're doing.  It's possible to cause
  // a deadlock if these handles are used incorrectly.
  protected PushBackStream in;
  protected /*PrintStream out; */ PrintWriter out;

  // ================== private instance variables =============
  private String aceHost;
  private int acePort;
  private Socket aceH;
  //private AceCache cache;
  protected AceCache cache; // Changed by wgm
  
  private static boolean debug = false ;
  private static Hashtable pool = null;
  private Class[] mainDisplays = null ;

private AceObject jadeChoice = null ;
//** Is this constructor used to communicate with a file? **//   
protected AceAccessor() { //For subclassing purpose
}


public AceAccessor(File fil) throws IOException {

	 // Create a pair of input/output streams for communicating
	 // with the database.
	 super () ;
	 in = new PushBackStream(new BufferedReader(new FileReader(fil))) ;
	 
        out = null ;
	 cache = new AceCache(CACHE_MIN,CACHE_MAX);

	 // Everything that may go wrong should go wrong , so set our flag to true.
	 echoMode = true ;
	 good = true;
   }         
  /* ==================== CONSTRUCTOR ===================== */   
  // The only constructor requires the host name and the port.
  // This may throw an IOException, so watch out!
   public AceAccessor(String host,int port) throws IOException {
	 this(host,port,false,true);
   }   
   public AceAccessor(String host,int port,boolean debugMode,boolean makeCache) throws IOException {
	 super () ;
	 aceHost = host;
	 acePort = port;
	 InetAddress address = InetAddress.getByName(aceHost);
	 aceH = new Socket(address,acePort);
	 if (aceH == null) return;

	 // Create a pair of input/output streams for communicating
	 // with the database.
	 //     in = new PushBackStream(new DataInputStream(aceH.getInputStream()));
	in = new PushBackStream(new BufferedReader(new InputStreamReader(aceH.getInputStream())));
        out =new PrintWriter(aceH.getOutputStream(),true);

	 // Look for the friendly greeting.  If it's not there,
	 // then shut everything down.
	 String line = read();
	 if (resultCode(line) != OK) {
	   aceH.close();
	   in = null;
	   out = null;
	   throw new IOException(resultMessage(line));
	 }

	 if (makeCache)
	   cache = new AceCache(CACHE_MIN,CACHE_MAX);

	 // Everything should be OK, so set our flag to true.
	 echoMode = debugMode;
	 good = true;
   }   
  /* ================================================= */
  /* ================ cacheStats() =================== */
  /* Print out some cache statistics on standard error */
  /* ================================================= */
  public void cacheStats() {
	cache.dumpStats();
  }  
  /* ================================================= */
  /* =================== count() ===================== */
  /* Given a class and name (which may contain wild cards),
	 look up the objects and return how many are found.
	 The list may later be retrieved with list().  Note that
	 if you call this in a multithreaded environment, the
	 active list may not be the same when you call list() as
	 when you called count(). */
  /* ================================================= */
  public int count(String classe,String name) {
	return countRequest("find " + classe + " " + name);
  }  
  // Call this for requests that result in an object count.  
  // It returns the number of objects found; -1 if an error occurred.
  // If the server has timed out during
  // this period, we try to rebuild the connection (once).
  // If all has failed, we return -1.
  protected synchronized int countRequest(String query) {
	String result;
	int resultCode;
	try {
	  try {
	write(query);
	result = read();
	resultCode = resultCode(result);
	  } catch (IOException e) { // we may time out at this point.  Try again.
		rebuildConnection();
	write(query);
	result = read();
	resultCode = resultCode(result);
	  }

	  if (resultCode != OK) return -1;
	  return foundCount(result);

	} catch (IOException e) {
	  return -1;
	}
  }  
  // Return another handle on the same database.  It may be
  // the same handle or a fresh one.
   public AceAccessor dup() {
	 AceAccessor returnval = this;
	 try {
	   returnval =  AceAccessor.open(aceHost,acePort,false);
	 } catch (IOException e) {
	   returnval = this;
	 }
	 // use the same cache please.
	 returnval.cache = cache;
	 return returnval;
   }   
  /* ================================================= */
  /* =================== fetch () ===================== */
  /* Create a new AceObject based on the class and type. */
  /* Note that this creates an UNFILLED object.         */
  /* ================================================= */
  public AceObject fetch(String classe,String name) {
	return fetch(classe,name,false);
  }  
  /* pass a flag of true if you want to retrieve only cached objects */
  public AceObject fetch(String classe,String name,boolean cacheOnly) {
	AceObject ace=null;

	// If object is in the cache, then no need to do a database lookup.
	// Reserve this object if we're going to do a database access
	// (cacheOnly flag is false).
	boolean reserve = !cacheOnly;
	if (cache != null) {//Added by wgm
          ace = cache.fetch(classe,name,reserve);
	  if (ace != null) return ace;
	  if (cacheOnly) return null;
        }
	ace = low_fetch(classe,name,false);
	if (ace != null)
	  return ace;
	// If we get here then we were unsuccessful in fetching
	// the object.  Unreserve the object if it has been
	// reserved.
	if (reserve) cache.unreserve(classe,name);
	return null;
  }  
  /* Any additional true flag specifies that you want a filled object */
  public AceObject fetch(String classe,String name,boolean cacheOnly,boolean fill) {  
	return low_fetch(classe,name,fill);
  }  
  // This method gets called right before GC.  All it does is send
  // a "quit" command
  protected void finalize() throws IOException {
	write("quit");
  }  
  /* ================================================= */
  /* =================== find () ===================== */
  /* Create an array of AceObject based on a "find"    */
  /*                 style query                       */
  /* ================================================= */
  public AceObject[] find(String classe,String name) {
	return requestObjects("find " + classe + " " + name,0,-1);
  }  
  /* ================================================= */
  /* =================== find () ===================== */
  /* Create an array of AceObject based on a "find"    */
  /* style query. as above but with a size limit       */
  /* ================================================= */
  public AceObject[] find(String classe,String name,int length) {
	return requestObjects("find " + classe + " " + name, 0, length);
  }  
  /* Call this one when there are potentially more objects than
	 will easily fit into an array.  Returns an enumeration
	 that should be treated as follows:
	 Enumeration e = a.find_many("Sequence","*");
	 while (e.hasMoreElements()) {
	   AceObject ace = (AceObject)e.nextElement();
	 }
	 */
  public Enumeration find_many(String classe,String name) {
	return new AceEnumerator("find " + classe + " " + name,this);
  }  
  /* ================================================= */
  /* ================= find_one() =================== */
  /* Return _one_ AceObject based on a "find" style    */
  /* query.  If several objects match, you get the first */
  /* one.  If no objects match, you get NULL           */
  /* ================================================= */
  public AceObject find_one(String classe,String name) {
	AceObject[] result = requestObjects("find " + 
					classe + " " + 
					name, 0, 1);
	if (result == null || result.length <= 0)
	  {
	System.out.println("find_one failed on " + classe + ":" + name) ;
	return null;
	  }
	return result[0];
  }  
  /* ================================================= */
  /* ================= follow () ===================== */
  /* Create an array of unfilled AceObject based on a */
  /* follow command                                    */
  /* ================================================= */
  public AceObject[] follow(String classe,String name,String tag) {
	return follow(classe,name,tag,-1);
  }  
  public synchronized AceObject[] follow(String classe,String name,
					 String tag,int length) {
	int objects = count(classe,name);
	if (objects != 1) return null;
	if(request("follow " + tag) != OK) return null;
	return list(0,length);
  }  
  // For messages that have the format "42 objects found", return
  // the count.  You'll want to make sure the result code is 200 (OK)
  // before calling this.
  protected int foundCount(String a) throws IOException {
	return resultCode(resultMessage(a));
  }  
public Class[] getClassDisplays (String klasse)
  { Vector objects = new Vector(10);
	if (jadeChoice == null)
	  jadeChoice = find_one("Jade","default") ;
	Class dd = null ;

	if (jadeChoice != null)
	  { 
	AceObject next =  jadeChoice.at("Display") ;
	if (next != null)
	  {
	    next =  next.at(klasse) ;
	    if (next != null)
	      {
		next = next.right();
		while (next != null) {
		  dd = name2dd (next.name) ;
		  if (dd != null)
		    objects.addElement(dd) ;
		  next = next.down();
		}
	      }
	  }
	  }
	dd = name2dd ("jade.maps.TreeDisplay") ;
	if (dd != null)
	  objects.addElement(dd) ;
	Class [] result = new Class[objects.size()];
	for (int i = 0; i < objects.size(); i++)
	  result[i] = (Class) objects.elementAt(i);
	return result ;
  }  
public Class[] getMainDisplays ()
  { Class dd = null ;
	if (mainDisplays != null)
	  return mainDisplays ;
	Vector objects = new Vector(10);
	if (jadeChoice == null)
	  jadeChoice = find_one("Jade","default") ;
	  //jadeChoice = find_one("Class","*") ;

	if (jadeChoice != null)
	  { 
	AceObject next =  jadeChoice.at("StandAloneDisplay") ;
	if (next != null)
	  {
	    next = next.right();
	    while (next != null) {	  
	      dd = name2dd (next.name) ;
	      if (dd != null)
		objects.addElement(dd) ;
	      next = next.down();
	    }
	  }
	  }
	mainDisplays = new Class[objects.size()];
	for (int i = 0; i < objects.size(); i++)
	  mainDisplays [i] = (Class) objects.elementAt(i);
	return mainDisplays ;
  }        
  /* ================================================= */
  /* =================== rawData () ===================== */
  /* Given a query accepted by netClient
	 Returns every thing exluding the final lone dot
	 without buffering or processing it in anyway
	 returns null if the query is not recognised or there is
	 some other error */
  /* ================================================= */
  public String getRawData(String query) {
	 if (debug) System.out.println("AceAcessor.getRawData" + query) ;
	 return rawDataRequest(query) ;
  }  
  public Object[][] getTable(String name,String[] params,String ccc)
  {
	String [] parameters ;

	if (params == null) { parameters = new String[0]  ;}
	else parameters = params ;

	if (debug)
	  System.out.println("AceAccessor.getTable " + name + " parameter[0] " + parameters[0]) ;

	AceTable tab  = low_tableQuery(name,parameters);
	AceObject[][] t = tab.toTable();
	char [] classes = ccc.toCharArray() ; 
	int i, j ;
	AceObject x ;
	int ncol = classes.length, nline = t.length ;
	Object[][] u = new Object [nline][ncol] ; 
	if (debug)
	  System.out.println("AceAccessor.getTable found " + nline + " lines") ;

	for (j = 0 ; j < nline; j++)
	  for (i = 0 ; i < ncol && i < t[j].length ; i++)
	{
	  char cc = classes[i] ;
	  x = (AceObject) t[j][i] ;
	  if (x != null)
	  switch (cc)
	    {
	    case 'k':
	      u [j][i] = x ;
	      break ;
	    case 'i':
	      int ii = x.intOf() ;
	      Integer I = new Integer (ii) ;
	      u[j][i] = (Object)I ;
	      break ;
	    case 's':
	      String S = x.name ;
	      u[j][i] = (Object)S ;
	      break ;
	    case 'f':
	      float f = x.floatOf() ;
	      Float F = new Float (f) ;
	      u[j][i] = (Object)F ;
	      break ;
	    default:
	      u[j][i] = null ;
	      break ;
	    }
	}
	return u ;
  }  
public String getTableName (String klasse, String displayName, String tab)
  { 
	if (jadeChoice == null)
	  jadeChoice = find_one("Jade","default") ;

	AceObject next =  jadeChoice.at("Display") ;
	if (next != null)
	  {
	 next =  next.at(klasse) ;
	 if (next != null)
	   {
	     next =  next.at(displayName) ;	
	     if (next != null)
	       {
		 next =  next.at(tab) ;
		 if (next != null)
		   next = next.right();
	       }
	   }
	  }

	if (next != null)
	  return next.name() ;
	return tab ;
  }  
  /* ================================================= */
  /* =================== list () ===================== */
  /*       Return the list of Active Objects           */
  /* ================================================= */
  public AceObject[] list() {
	return list(0,-1);
  }  
  /* ================================================= */
  /* ================= list (cnt) ==================== */
  /* Return the list of Active Objects, from start to finish
   start and finish are 0-based. length of -1 is equivalent to infinity*/
  /* ================================================= */
  protected synchronized AceObject[] list(int start,int length) {
	int cnt = 0;
	Vector objects = new Vector(10);
	AceObject[] result;
	int max = (length >= 0) ? length : Integer.MAX_VALUE;
	String command = "list -b " + start + " -c " + length;

	try {
	  // make the database list the names
	  if (request(command) != OK) return null;

	  while (cnt < max) {
	String nextline = read();
	if ((nextline == null) || (nextline.length() == 0))
	  break;
	AceString a = new AceString(nextline);
	AceObject ao = cache.fetch(a.aceClass(),a.aceID(),true);
	if (ao == null)
	  ao = new AceObject(a.aceClass(),a.aceID(),this);
	objects.addElement(ao);
	cnt++;
	  }

	  result = new AceObject[cnt];
	  for (int i = 0; i < cnt; i++) {
	result[i] = (AceObject)objects.elementAt(i);
	  }

	  sync(); // read to the terminal dot
	} catch (IOException e) {
	  result = null;
	}

	return result;
  }  
  // Low level routine, not for public consumption
  protected synchronized AceObject low_fetch(String classe,String name,boolean fill) {
	int count = count(classe,name);
	if (count < 0) return null;
	if (count > 1) return null;
	AceObject ace = null;

	if (fill) {
	  // If we get here, then we got one and only one object
	  // We can show it now.
	  StringBuffer message = new StringBuffer(20);
	  int result = request("show",message);
	  if (result == OK) {
	ace = new AceObject(this);
	  // The ace object (ought to) stop reading when it hits the blank line.
	  // this leaves a dot to read.
	  sync();		
	  }
	} else 
	  ace = new AceObject(classe,name,this);
	return ace;
  }  
  /* ============================================================== */
  /* Ace object creation.  Done here so that we can lock everyone else */
  /* out until this method completes.                                  */
  /* ============================================================== */
  protected synchronized AceObject low_fill(AceObject a) throws IOException {
	AceObject result = null;
	if (count(a.classe,a.name) == 1) {
	  if (request("show") == OK) {
	String inputLine = in.readLine();
	String[] elements = new AceString(inputLine).split('\t');
	if (AceObject.debug2)
	  System.err.println("AceAccessor: low_fill() of AceObject " + a);
	result = new AceObject(elements,in,1,this);
	sync();
	  }
	}
	return result;
  }  
  // Common code called by tableQuery().  Don't use.
  protected AceTable low_tableQuery(String name,String[] parameters) {
	AceTable ace;
	String ID = AceTable.paramToString(name,parameters); 
	// debug = true ;
	if (debug) System.out.println("low_tableQuery 1") ;

	if ((ace = (AceTable)cache.fetch(AceTable.runTimeClass,ID,true))!=null)
	  {     
	if (debug) System.out.println("low_tableQuery 2") ;
	  return ace;
	  }
	if (debug) System.out.println("low_tableQuery 2bis") ;
	if (name.equals(AceTable.runTimeName))
	  { ace = new AceTable(parameters,this);
	  if (debug) System.out.println("low_tableQuery 3") ;
	  }
	else
	  { ace= new AceTable(name,parameters,this);
		if (debug) System.out.println("low_tableQuery 4") ;
	  }
	if (ace != null)
	  return ace; 
	if (debug) System.out.println("low_tableQuery 5") ;
	cache.unreserve(AceTable.runTimeClass,ID);
	return null;
  }  
/**
 * This method was created in VisualAge.
 * @param args java.lang.String[]
 */
public static void main(String args[]) {
	
	AceAccessor worm = null;
	
	try {
		worm = new AceAccessor("alpha.crbm.cnrs-mop.fr", 20100);
		System.out.println("Connect to the server successfully.");
	}
	catch (IOException e) { System.out.println(e); }
	
	Class[] classes = worm.getMainDisplays();

	for (int i=0; i < classes.length; i++) {
		System.out.println(classes[i]);
	}
}
  // mieg, JD, Display control
// Jade      Display Text Text Text UNIQUE Text // Class, jade-class, jade-table, ace-table

  // data example on the ace server

// Jade default
// Display Chromosome jade.maps.Vmap
// Display  Sequence jade.maps.Vmap map.interval link.interval
// Display Table jade.maps.TableDisplay
// Display Cell lignage.Lignage
// Display Node lignage.Lignage

//** Is this method used to confirm if the class implements displayable interface? **//


private Class name2dd (String s)
  {
	try
	  { 
	  if (debug) System.out.println("Invoker looks for " + s) ;
	  Class c = Class.forName(s) ;
	  if (c == null) return null ;
	  if (debug) System.out.println("Invoker found class " + s) ;
	  Class [] ff = c.getInterfaces() ;
	  int j ;
	  if (ff != null)
	{ 
	  if (debug) System.out.println("Invoker found some interface") ; ;
	  for (j = ff.length - 1 ; j >= 0 &&
		 ! ff[j].getName().equals("jade.graphics.Displayable") ; j--) ;
	  if (j >= 0)
	    { 
	      if (debug) System.out.println("Invoker found Displayable interface") ; ;
	      return c ;
	    }
	}
	  }  
	catch (Exception e) 
	  {   
	if (debug) System.out.println("Invoking exception" + e) ;
	return null ;
	  }
	return null ;
  }    
  // Internally the class maintains a pool of accessors.  If you
  // use the CLASS'S open() method to get an accessor, it will
  // allocate a previously used one if appropriate.  The tradeoff
  // is a little initial startup time for a big improvement in
  // overall responsiveness when fetching lots of small objects
  // from the ace server.  It doesn't hurt to use the constructor,
  // and in fact is guaranteed to give you a dedicated channel that 
  // no one else will use.
  // ==================== pool of accessors ===================
  public synchronized static AceAccessor open(String host, int port) throws IOException {
	return open(host,port,true);
  }  
  public synchronized static AceAccessor open(String host, 
					      int port, boolean cache) throws IOException {
	if (pool == null)
	  pool = new Hashtable();
	AccessorPool a = (AccessorPool)pool.get(host + ":" + port);
	if (a == null)
	  pool.put(host + ":" + port,a = new AccessorPool(host,port,MAX_CONN));
	return a.createOrPick(cache);
  }  
  /* ================================================= */
  /* =================== query () ===================== */
  /* Create an array of AceObject based on any query. */
  /* ================================================= */
  public AceObject[] query(String queryStr) {
	return requestObjects("query " + queryStr,0,-1);
  }  
  /* ================================================= */
  /* =================== query () ===================== */
  /* Create an array of AceObject based on any query. */
  /* The same as above but with a maximum size limit */
  /* ================================================= */
  public AceObject[] query(String queryStr,int length)   {
	return requestObjects("query " + queryStr,0,length);
  }  
  /* Call this one when there are potentially more objects than
	 will easily fit into an array.  Returns an enumeration
	 that should be treated as follows:
	 Enumeration e = a.find_many("Sequence","*");
	 while (e.hasMoreElements()) {
	   AceObject ace = (AceObject)e.nextElement();
	 }
	 */
  public Enumeration query_many(String queryStr) {
	return new AceEnumerator(queryStr,this);
  }  
  // Call this for requests 
  // returns everything excluding the terminal dot
  // returns null on error
  // If the server has timed out during
  // this period, we try to rebuild the connection (once).
  // If all has failed, we return null
  protected synchronized String rawDataRequest(String query) {
	String line, result ;
	int resultCode;
	try {
	  try {
	write(query);
	line = read();
	resultCode = resultCode(line);
	  } catch (IOException e) { // we may time out at this point.  Try again.
	rebuildConnection();
	write(query);
	line = read();
	resultCode = resultCode(line);
	  }
	  if (resultCode != OK) 
	return null ;
	  result = new String () ;
	  while (true)
	{
	  line = read () ;  
	  if (line == null)
	    break ;
	  if(line.equals("."))
	    break ;
	  result = result.concat("\n" + line) ; 
	}
	  return result ;
	} catch (IOException e) {
	  return null ;
	}
  }  
  // This is our low level read routine
  protected String read() throws IOException {
	String result = in.readLine();
	if (echoMode)
	  System.err.println("AceAccessor read< " + result);
	Thread.yield();
	return result;
  }  
  // Flush everything to the dot.  Be careful: this may block if you
  // call it in the wrong sequence!!!
  protected synchronized void readToDot() {
	boolean found = false;
	try {
	  while (!found) {
	String line = read();
	found = line.equals(".");
	  }
	} catch (IOException e) {
	  found = true;
	}
  }  
  // This function tries to rebuild the connection if it dies for
  // any reason.  Try it once.  If it doesn't work give up.
  protected void rebuildConnection () throws IOException {
	 InetAddress address = InetAddress.getByName(aceHost);
	 aceH = new Socket(address,acePort);
	 if (aceH == null) throw new IOException("Couldn't reallocate database handle");
	 //     in = new PushBackStream(new DataInputStream(aceH.getInputStream()));
	 in = new PushBackStream(new BufferedReader(new InputStreamReader(aceH.getInputStream())));
        out = /*new PrintStream(aceH.getOutputStream()); */ new PrintWriter(aceH.getOutputStream(),true);
	 // Look for the friendly greeting.
	 String line = read();
	 if (resultCode(line) != OK) {
	   aceH.close();
	   in = null;
	   out = null;
	   throw new IOException(resultMessage(line));
	 }
	 good = true;
  }  
  // Send a request to the server, returning only the
  // result code.  If the server has timed out during
  // this period, we try to rebuild the connection (once).
  // If all has failed, we return -1.
  protected synchronized int request(String query) {
	String result;
	int resultCode = -1;

	try {

	  try {
	write(query);
	result = read();
	resultCode = resultCode(result);
	  } catch (IOException e) {
	rebuildConnection(); // this may raise another exception, which we don't catch
	write(query);
	result = read();
	resultCode = resultCode(result);
	  }

	  return resultCode;

	} catch (IOException e) {
	  return -1;
	}

  }  
  // Send a request to the server.  Return the result
  // code and message.  If the server has timed out during
  // this period, we try to rebuild the connection (once).
  // If all has failed, we return -1.
  protected synchronized int request(String query,StringBuffer message) 
  {
	String result;
	int resultCode;

	try {

	  try {  // may get a timeout error at this point
	write(query);
	result = read();
	resultCode = resultCode(result);
	  } catch (IOException e) {
	rebuildConnection(); // this may raise another exception, which we don't catch
	write(query);
	result = read();
	resultCode = resultCode(result);
	  }
	  message.setLength(0);
	  message.append(resultMessage(result));
	  return resultCode;

	} catch (IOException e) {
	  return -1;
	}
  }  
  /* ============================================================== */
  /* ================= Protected low level funcs ================== */
  // Create an array of Ace Objects based on a general query.
  protected synchronized AceObject[] requestObjects(String query,int start,int length) {
	int cnt;
	int found = countRequest(query);
	totalCount = found ;
	if (found <= 0) return null;
	return list(start,length);
  }  
  // ============== utility functions ==============

  // Parse the result code section of the server message
  // SIDE EFFECT: if resultcode == TIMEOUTERR, then we
  // set our state to closed, and we raise a TimeOutError
  // exception.
  protected int resultCode(String a) throws IOException {
	int index = a.indexOf(' ');
	if (index < 0) return -1;
	try {
	  int r = Integer.parseInt(a.substring(0,index));
	  if (r == TIMEOUTERROR) {
	good = false;
	throw new IOException("Ace connection timed out.  Rebuild connection.");
	  }
	  return Integer.parseInt(a.substring(0,index));
	} catch (NumberFormatException e) {
	  return -1;
	}
  }  
  // Parse the text section of the server message
  protected String resultMessage(String a) {
	int index = a.indexOf(' ');
	if (index < 0) return "";
	return a.substring(index+1,a.length());
  }  
  /* ================================================= */
  /* ==================== sync() ===================== */
  /* ================================================= */
  // Low level function that should not be called naively.
  // Must incorporate this into the "Ace package"
  // Call this guy to "sync up" the database with java.
  // It may be dangerous to call it more than once (will block).
  protected void sync() {
	readToDot();
  }  
  /* EVERYTHING BELOW HERE IS PROTECTED, DON'T MESS WITH IT */
  /* ************************************************************** */
  /* ************************************************************** */
  /* ************************************************************** */
  /* ************************************************************** */

  /* =================== table () ===================== */
  /* Perform a query with an arbitrary table array      */
  /* ================================================= */
  protected synchronized int table(String[] query) {
	int result = -1;

	try { 
	  result = request("table");

	  if (result != OK)
	return result;

	  // If we get here, we can send the table query out, one
	  // line at a time
	  for (int i = 0; i < query.length; i++)
	write(query[i]);
	  write("."); // this terminates the query
	
	  String input = read();
	  return resultCode(input);
	} catch (IOException e) {
	  return -1;
	}

  }  
  /* ================================================= */
  /* =================== table () ===================== */
  /* Perform a query with a named table and parameters */
  /* ================================================= */
  protected int table(String tName,String[] parameters) {
	int result = -1;
	StringBuffer p = new StringBuffer("ntable");
	p.append (" " + tName);
	for (int i = 0; i < parameters.length;i++)
	  p.append(" " + parameters[i]);
	result = request(p.toString());
	return result;
  }  
  protected synchronized AceObject table_fill(AceTable a) throws IOException {
	if (a.runtime) {
	  if (a.theQuery == null)
	return null;

	  if (table(a.theQuery) == OK) {
	a.initializeFromStream(in,this);
	sync();
	  }

	} else {
	  if (table(a.name,a.params) == OK) {
	a.initializeFromStream(in,this);
	sync();
	  }

	}
	return a.right;
  }  
  /* ================================================= */
  /* ================ tableQuery() =================== */
  /* Perform a table query using the provided runtime  */
  /*                    table                          */
  /* ================================================= */
  public AceTable tableQuery(String[] query) {
	return low_tableQuery(AceTable.runTimeName,query);
  }  
  /* ================================================= */
  /* ================ tableQuery() =================== */
  /* Perform a table query using the provided Ace      */
  /*                 Table object                      */
  /* ================================================= */
  public AceTable tableQuery(AceObject theTable,String[] parameters) {
	return low_tableQuery(theTable.name,parameters);
  }  
  /* ================================================= */
  /* ================ tableQuery() =================== */
  /* Perform a table query using the provided runtime  */
  /*                 named table                       */
  /* ================================================= */


  public AceTable tableQuery(String name,String[] parameters) {
	return low_tableQuery(name,parameters);
  }  
  // Protected function.  Tell the database that this object
  // has been touched and needs updating.
  protected void touch(AceObject a) {
	cache.touch(a);
  }  
  // This is our low level write routine
  protected void write(String text) throws IOException {
	if (echoMode)
	  System.err.println("AceAccessor write> " + text + " out=" + out);
        if (text != null && out != null) /*{ out.write(text.getBytes()); out.println(); }/*/ out.println(text);
	  Thread.yield();
  }  
}