package jade.db;

import java.io.*;
import java.util.*;

// ===================================================================================
// This is a package-only class that returns an enumeration on an Ace Object
// It is used to retrieve Ace objects sequentially when the list is too long
// to return a simple array.  It maintains state on the query used to generate it.
// ===================================================================================

class AceEnumerator implements Enumeration {
  private String theQuery = "";
  private int cacheSize = 0;
  private AceObject[] cache = null;
  private int position = 0,current = 0;
  private boolean more = true;
  private AceAccessor db = null;
  private static int DEFAULTCACHE = 1000;
  private static boolean debugging = false;
  public int totalCount = 0 ;

  // package-only constructors
  AceEnumerator(String query, AceAccessor database) {
	this(query,database,DEFAULTCACHE) ;
  }  
  AceEnumerator(String query, AceAccessor database, int size) {
	theQuery = query;
	db = database; 
	cacheSize = size;
  }  
  // fill the cache from the database
  private boolean fill_cache() {
	if (debugging) System.err.println("Filling cache...");

	String query ;
	query = "query " + theQuery;
	cache = db.requestObjects(query,current,cacheSize);
	totalCount = db.totalCount ;
	if ((cache == null) || (cache.length == 0)) {
	  if (debugging) System.err.println("Query failed");
	  return false;
	}
	else {
	  if (debugging) System.err.println("Cache filled..." + cache.length + " objects");
	  position = 0;
	  current += cache.length;
	  return true;
	}
  }  
  // return true if there are more elements to read
  public boolean hasMoreElements() {
	if (!more) return false;
	if ((cache == null) || (position >= cache.length)) {
	  more = fill_cache();
	}
	return more;
  }  
  // return the next object
  public Object nextElement() {
	if ((cache == null) || (position >= cache.length)) {
	  more = fill_cache();
	}
	if (!more)
	  throw new NoSuchElementException("No more Ace objects");
	return cache[position++];
  }  
}