package jade.db;

/* This class is an adaptor object for communicating with the database.
   It handles all the messy socket stuff. */

//** An implemented doubly linked list.   **//

import java.io.*;
import java.net.*;
import java.util.*;


// =====================================================================
// ============================== AceCache =============================
// =====================================================================
// This package-only class will hang onto AceObjects for a while.
// The deallocation algorithm is very simple right now -- we only hold
// onto a maximum number of objects and begin dropping them in a
// least-recently-used manner when MAX_OBJECTS is exceeded.
// There is one cache object for each database pool.

// Wrapper object for linking AceObjects together.
// DESIGN QUESTION: Would it be better to provide AceObjects themselves
// (or a descendent) with the linked list handles?
class AceNode {
  AceObject object=null;
  AceNode left=null;
  AceNode right=null;
  
  AceNode() { }  
  // Create an Ace node
  AceNode(AceObject a) { object = a; }  
  // Create an Ace node and add it to linked list
  // behind the indicated node (which may be null)
  AceNode(AceObject a,AceNode after) {
	this(a);
	this.insert(after);
  }  
  // Remove this node from a linked list.
  // We don't have to worry about actually deleting it
  // because of garbage collection.
  synchronized void delete () {
	if (left != null)
	  left.right = right;
	if (right != null)
	  right.left = left;
	right = null;
	left = null;
  }  
  // Add existing node to a linked list _after_
  // the indicated object.
  // Indicated node may be null.
  // If we are already in a list, then we
  // extricate ourselves.
  synchronized void insert (AceNode after) {
	// extricate ourselves from any current
	// commitments.
	delete();  
	left = after;
	if (left != null) {
	  right = left.right;
	  left.right = this;
	}
	if (right != null)
	  right.left = this;
  }  
  public String toString() {
	if (object == null)
	  return super.toString();
	return "Node: " + object.toString();
  }  
}