package jade.db;

// Definitions for an AceObject
// #include "shortcuts.jh"

import java.io.*;
import java.util.*;
import java.text.*;
import jade.util.*;

public class AceObject extends Key {
  
  public String classe = null;
 
  // Because the object may not be fully populated
  // at any given time, don't access these pointers
  // directly, but use the right() and down() methods.
  // Sorry.
  protected AceObject right = null;
  protected AceObject down = null;

  protected AceAccessor database = null;
  protected boolean filled = false;
  protected boolean tried2fill = false;
  protected boolean tried2pick = false;
  protected boolean pickedOK = false;

  // At class initialization time, we get information about the
  // locale and store it in df
  static protected DateFormat df = DateFormat.getDateInstance();

  // just some debugging variables.  debug2() turns on
  // some messages.  debug() turns on a LOT of them.
  static protected boolean debug = false;
  static protected boolean debug2 = false;

  /* ************ constructors ************ */
  // All constructors (except for the default null constructor)
  // are protected.  To obtain an Ace object you must request it
  // from the AceAccessor class.  This allows caching to work
  // correctly.
  public AceObject() { super() ;
  }  
  // =================================================================
  //                   PRIVATE
  // =================================================================
  protected AceObject(String[] elements,PushBackStream in,int level,
		    AceAccessor db) {
	database = db;
	tried2fill = true;
	try {
	  parseInputStream(elements,in,level,db);
	  filled = true;
	} catch (IOException e) {
	  filled = false;
	}
  }  
  // create a filled object from a database stream
  protected AceObject(AceAccessor theDatabase) {
	if (initializeFromStream(theDatabase.in,theDatabase)) {
	  database = theDatabase;
	}
	register();
  }  
  // create a filled object from an unspecified input stream
  protected AceObject(DataInputStream in) {
	if (initializeFromStream(new PushBackStream(new BufferedReader(new InputStreamReader(in))),null)) {
	  database = null;
	}
  }  
  // create an unfilled object with just the class and name
  protected AceObject(String theType, String theID, AceAccessor theDatabase) {
	classe = theType;
	name = theID;
	database = theDatabase;
	register();
  }  
  protected static boolean _isClass(String classe) {
	if (classe.equals("float")) return false;
	if (classe.equals("int")) return false;
	if (classe.equals("date")) return false;
	if (classe.equals("tag")) return false;
	if (classe.equals("txt")) return false;
	if (classe.equals("dna")) return false;
	if (classe.equals("peptide")) return false;
	//    ?? don't need this statement because table objects
	//       effectively act like classes ??
	//    if (classe.equals("table")) return false;
	return true;
  }  
  // ========================= accessor() ====================
  // return the AceAccessor associated with this object
  public Accessor accessor() { 
	return database;
  }  
  // Utility used for pretty-printing the tree
  protected int appendToString(StringBuffer out,int position, int level) {
	// Tab over the required amount
	for (int i=position;i<level;i++)
	  out.append('\t');

	// Add the root name
	out.append(name);
	// If there are children to the right, add them
	if (right != null) {
	  level = right.appendToString(out,level,level+1);
	}
	// Add anything below
	if (down !=null) {
	  out.append('\n');
	  level = down.appendToString(out,0,level);
	} else
	  level--;

	return level;
  }  
  // ======================= at () ========================
  // Return the partial Ace tree at the indicated key
  public AceObject at(String key) {
	fill();
	AceObject next = right;
	while (next != null) {
	  if (next.name.equalsIgnoreCase(key)) {
	return next;
	  }
	  next = next.down;
	}
	// if we get here, we couldn't find it
	return null;
  }  
  public Object bestType() {
	if (isClass())
	  return new AceObject(classe,name,database.dup());
	  
	if (classe.equals("float"))
	  return Float.valueOf(name);
	else if (classe.equals("int"))
	  return Integer.valueOf(name);
	else if (classe.equals("date"))
	  try {
	return df.parse(name);
	  } catch (ParseException e) {
	return new Date(0); // the epoch
	  }
	else // everything else, including tags, get returned as strings
	  return name;
  }  
  public String classe() {
	return classe;
  }  
  // ======================= col () ========================
  public Object[] col() {
	fill();
	AceObject next = right;
	Vector objects = new Vector(10);

	while (next != null) {
	  objects.addElement(next.bestType());
	  next = next.down;
	}

	Object[] result = new Object[objects.size()];
	for (int i = 0; i < objects.size(); i++)
	  result[i] = objects.elementAt(i);
	
	return result;
  }  
  // ======================= countKeys () =======================
  // count all the keys in this object
  public int countKeys() {
	fill();
	int result = 0;
	AceObject current = right;
	while (current != null) {
	  result++;
	  current = current.down;
	}
	return result;
  }  
  // If name can't be coerced into a date, returns something
  // completely arbitrary: midnight on January 1, 1970 Pacific Standard Time
  public Date dateOf() {
	Date result;
	try {
	  result = df.parse(name);
	} catch (ParseException e) {
	  result = new Date(0); // default == "the epoch"
	}
	return result;
 } 
  // If name can't be coerced into a double, returns 0.0
  public double doubleOf() {
	double result;
	try {
	  result = Double.valueOf(name).doubleValue();
	} catch (NumberFormatException e) {
	  result = 0.0;
	}
	return result;
  }  
  public AceObject down() {
	fill();
	return down;
  }  
  /* *******         dump()     ****** */   
  // dump out in tab-delimited way
  // nice formatting comes later
  public String dump() {
	fill();
	StringBuffer newString = new StringBuffer(1024);
	newString.append(name);
	if (right != null) {
	  newString.append("\t");
	  right.appendToString(newString,1,1);
	}
	return newString.toString();
  }  
  /* *************** public methods ******************** */   


public boolean equals (Object obj)
  {
	try 
	  {
	AceObject k = (AceObject) obj ;
	return 
	  k.name != null && k.name.equals(name) && 
	  k.classe != null && k.classe.equals(classe) ;
	  }
   catch (Exception e) { return false ; }
  }  
  /* ================== initialization routines =================== */

  // FILL
  // The sequence of events is that the object's fill() method is called.
  // It calls its low_fill() method, which by default calls the ace
  // accessor's low_fill() method.  This in turn generates a call back
  // to our stream constructor below.  If something fails, we set our
  // tried2fill flag so that we don't try again.
  // Subclasses should override the low_fill() method rather than 
  // the fill() method.
  protected synchronized void fill() {

	if (tried2fill) return; 

	// consistency checking
	if ((right != null) ||
	(down != null) ||
	(database == null) ||
	(name == null) || 
	(name.length() == 0) ||
	(classe == null) || 
	(classe.length() == 0) ||
	!isClass()) {
	  return;
	}

	// There may be a different object in the cache that
	// has the same name as us. This can happen when we
	// haven't been used for a while and got unfilled & flushed, then
	// the object was reloaded from the database.
	// We can recover from this snarl by stealing our 
	// doppelganger's subtree.
	//    AceObject doppelganger = database.fetch(classe,name,true);
	//    if ((doppelganger != null) && (doppelganger.filled)) {
	//      right = doppelganger.right;  // steal our doppelganger's tree (!)
	//      return;
	//    }

	// tell the database that we're still in play.
	register();

	if (debug2) System.err.println("Filling " + this);
	try {
	  right = low_fill();
	} catch (IOException e) {
	  tried2fill = true;  // so we don't try again
	  return;
	}

  }  
  public boolean filled() {
	return filled;
  }  
  public Object[] flatten() {
	return row();
  }  
  // If name can't be coerced into a float, returns 0.0
  public float floatOf() {
	float result;
	try {
	  result = Float.valueOf(name).floatValue();
	} catch (NumberFormatException e) {
	  result = (float)0.0;
	}
	return result;
  }  
  // ==================== follow () =====================
  // Return the AceObjects located at the indicated tag
  // using get() semantics.
  public AceObject[] follow(String key) {
	//return follow(key,Integer.MAX_VALUE);
        return follow(key, -1); //Changed by wgm
  }  
  public  AceObject[] follow(String key,int maxObjects) {
	AceObject found = get(key);
	if (debug2) {
	  System.err.println("found " + found);
	  System.err.println("found.right() " + found.right());
	}
	if (database == null) return null;
       
	return database.follow(classe,name,key,maxObjects);
  }  
  public AceObject followOne(String key) {
	AceObject[] o = follow(key,1);
	if (o == null) return null;
	return o[0];
  }  
  // ====================== get () ========================
  // Like at(), but does a complete traversal of the tree looking
  // for the indicated key (which must be of type "tag")

  public AceObject get(String key) {
	fill();
	AceObject next = right;
	// Search down, then across in a recursive manner
	while (next != null) {
	  if (next.classe.equalsIgnoreCase("tag") && (next.name.equalsIgnoreCase(key)))
	return next;
	  else {
	AceObject found = null;
	if ((found = next.get(key)) != null)
	  return found;
	next = next.down;
	  }
	}
	return null;
  }  
public Class[] getDisplays ()
  {
	return database.getClassDisplays (classe) ;
  }  
public Object [][] getTable (String displayName, String tableName, 
			     String[] params, String ccc)
  {
	return database.getTable 
	  (database.getTableName(classe,displayName, tableName), params, ccc) ;
  }  
  protected boolean initializeFromStream(PushBackStream in,AceAccessor db) {
	if (debug2)
	  System.err.println(this + " AceObject.initializeFromStream()" );
	try {
	  String[] elements = new AceString(in.readLine()).split('\t');
	  if (elements.length <= 0)
	return false;

	  AceString aS = new AceString(elements[0]);
	  name = aS.aceID();
	  classe = aS.aceClass();

	  right = new AceObject(elements,in,1,db);
	  return true;

	} catch (IOException e) {
	  System.err.println(e);      
	}
	return false;
  }  
  // If name can't be coerced into an integer, returns 0.
  public int intOf() {
	int result;
	try {
	  result = Integer.parseInt(name);
	} catch (NumberFormatException e) {
	  result = 0;
	}
	return result;
  }  
  // ========================= isClass() ======================
  // Return true if we are a class (potentially pickable)
  public boolean isClass() {
	return _isClass(this.classe);
  }  
  // ========================= isTag() ========================
  // Return true if we are a tag (non pickable)
  public boolean isTag() {
	if (classe.equals("tag")) return true ;
	return false ;
  }  
  // ========================== keys () =======================
  // return all the keys at this level
  public AceObject[] keys () {
	int length = countKeys();
	AceObject[] result = new AceObject[length];
	AceObject current = right;
	for (int i=0;i<length;i++) {
	  result[i] = current;
	  current = current.down;
	}
	return result;
  }  
  // The low-level fill() is probably the one that subclasses
  // should override.  See AceTable() for an example.
 protected AceObject low_fill() throws IOException {
   return database.low_fill(this);
 } 
  // ======================= node () ========================
  // Return the single node as the appropriate type (string, tag or date).
  // Ace classes are returned as unfilled identifiers.  Note that you
  // will have to downcast the object in order to use specialized
  // methods with it.  This is a regrettable feature of java.
  public Object node(String key) {
	fill();
	AceObject tree = get(key);
	if (tree == null) return null;
	return tree.bestType();
  }  
  protected void parseInputStream(String[] elements,PushBackStream in,
				int level,AceAccessor db) throws IOException
  {
	// We need to call yield() frequently in order to yield time to other threads
	// in a non-timesliced environment.
	Thread.yield();

	AceString aS = new AceString(elements[level]);
	name = aS.aceID();
	classe = aS.aceClass();
	  
	if (debug)
	  System.err.println("\tClass=" + classe + " Name=" + name + " Level=" + level);

	if (elements.length > level+1) {
	  if (debug) System.err.println(name + ": Adding an object to the right");
	  right = new AceObject(elements,in,level+1,db);
	}
	
	// fetch the next set of elements
	String input = in.readLine();
	if (input==null) {
	  return;
	}
	
	int newLevel = new AceString(input).countTabs();
	if (newLevel == level) {
	  
	  if (debug)
	System.err.println (name + ": Adding an object to the bottom");
	  
	  down = new AceObject(new AceString(input).split('\t'),in,level,db);
	  
	} else if (newLevel < level)
	  in.pushBack(input);
	
  }  
  // ==================== pick () =====================
  // Follow a class reference.  If you pick a root object
  // it returns itself.  If it isn't a class it will return
  // null.  It remembers your attempt to pick and sets the
  // status code as a side effect.
  public AceObject pick () {
	if (status() <= DEAD) return null;
	tried2pick = true;

	AceObject result = database.dup().fetch(classe,name);
	pickedOK = result != null;
	return result;
  }  
  public  AceObject pick (String key) {
	AceObject subtree = get(key);
	if (subtree == null) return null;
	return subtree.right().pick();
  }  
  // Register an ace object with the accessor so that
  // it is (potentially) cached:
  protected void register() {
	if (debug2) System.err.println("Registering " + this +" == "
					   + registration_id()  );
	if (_isClass(classe) && (database != null))
	  database.touch(this);
  }  
  // Our cache registration name.  It's our name() by default, but
  // can be overridden (e.g. by table object)
  protected String registration_id() {
	return name();
  }  
  public AceObject right() {
	fill();
	return right;
  }  
  // ======================= row () ========================
  // Flatten out part of the tree into an array of Object.
  // Useful for retrieving parts of trees that logically belong together.
  public Object[] row() {
	fill();
	AceObject next = right;
	Vector objects = new Vector(10);

	while (next != null) {
	  objects.addElement(next.bestType());
	  next = next.right;
	}

	Object[] result = new Object[objects.size()];
	for (int i = 0; i < objects.size(); i++)
	  result[i] = objects.elementAt(i);
	
	return result;
  }  
   // Return the status code for this object.  LIVE objects are known to be
  // pickable (retrievable classes) and are usually to be found in the
  // object cache.  POTENTIAL objects are classes that may or may not
  // be pickable.  DEAD objects are those that do not have a tree in the
  // database.  INERT objects are floats, integers, dates, and the like.

  public int status() {
	if (!isClass()) return INERT;
	if (tried2pick) {
	  return pickedOK ? LIVE : DEAD;
	} else { // check cache for an object
	  AceObject peer = database.fetch(classe,name,true);
	  if ((peer != null) && (peer != this)) {
	tried2pick = pickedOK = true;
	return LIVE;
	  }
	}
	return POTENTIAL;
  }  
  // ==================== casting functions =============
  // These functions will attempt to cast the object to the requested
  // type.  Items which can't be correctly cast will be coerced into
  // something reasonable: strings will become numeric "0", and
  // dates will become January 1, 1970.
  public String stringOf() {
	return name;
  }  
  /* *******         toString()     ****** */   
  public String toString() {
	return name + " (" + classe + ")";
  }  
  // =================================================================
  //                   PROTECTED
  // =================================================================

  /* ************ constructors ************ */

  // Detach the storage for this object, allowing Java
  // object collection to take place.  This is a test of
  // Java's ability to detect when a cluster of objects
  // is unreachable.  I suppose it can.
  protected synchronized void unfill () {
	if (debug2) System.err.println("Unfilling " + this);
	right = null;
	down = null;
	filled = false;
	tried2fill = false;
  }  
}