package jade.db;


import java.io.*;

public class AceTable extends AceObject {

  // this holds the query to send to the database
  protected String[] theQuery;
  // this keeps track of the geometry of the table
  // for shortcuts
  protected int rows,columns;
  // This keeps track of whether we're a formally-named
  // table or a dynamically created one at runtime.
  protected boolean runtime = false;
  // This keeps track of our runtime parameters
  protected String[] params = {};

  // Our table-specific name and class
  protected static String runTimeName = "Runtime table";
  protected static String runTimeClass = "Table";
  // should not create these objects directly.
  // they must come out of the database
  public AceTable() {
	theQuery = null;
	database = null;
	runtime = true;
	name = runTimeName;
	classe = runTimeClass;
  }  
  // create an unfilled object from a query
 protected AceTable(String[] query,AceAccessor theDatabase) {
	theQuery = query;
	database = theDatabase;
	runtime = true;
	name = runTimeName;
	classe = runTimeClass;
	register();
  }  
  // create a filled object from a database stream
  protected AceTable(AceAccessor theDatabase) {
	super(theDatabase);
	runtime = true;
  }  
  // Create table from a AceObject Table.  Although
  // we _could_ pull out the actual query from the 
  // table, all we really do is scarf the database and
  // table name and allow the database to do the query
  // for us.
  protected AceTable(AceObject tableObject,String[] parameters) {
	database = tableObject.database;
	if (tableObject.classe.equals("Table")) {
	  name = tableObject.name;
	  params = parameters;
	} else {
	  System.err.println("Attempt to pass a non-Table object to the AceTable constructor.  " +
	   "Object's class was " + tableObject.classe);
	}
	classe = runTimeClass;
  }  
  // create a filled object from an input stream
  protected AceTable(DataInputStream in) {
	super(in);
	runtime = true;
  }  
  // create table from a formally-named table in the database
  // using the provided parameters, if any
  protected AceTable(String ID,String[] parameters,AceAccessor theDatabase) {
	database = theDatabase;
	params = parameters;
	name = ID;
	classe = runTimeClass;
  }  
  // Create table from formally-named table but without
  // any parameters
  public AceTable(String ID,AceAccessor theDatabase) {
	database = theDatabase;
	name = ID;
	classe = runTimeClass;
  }  
 // create an unfilled object with just the class and name
  protected AceTable(String theType,String theID,AceAccessor theDatabase) {
	classe = theType;
	name = theID;
	database = theDatabase;
	register();
  }  
  public int colCount() {
	fill();
	return this.columns;
  }  
  // stream initialization is extremely easy in this case
  // because of the predictable columnar nature of the
  // input data.
  protected synchronized boolean initializeFromStream(PushBackStream in, AceAccessor db) {

	AceObject   rowhead = null;
	AceObject   previous = null;
	rows = 0;

	while (true) {
	  String input;
	  try {
	input = in.readLine();
        input = input.trim(); // Added by wgm to remove the empty head from the table
       
      	  } catch (IOException e) {
	input = null;
	  }

	  if (input == null)
	break;

	  if (input.length() == 0) // end of input
	break;

	  String[] elements = new AceString(input).split('\t');
	  if (elements.length <= 0)
	break;

	  // remember this for later reference
	  columns = elements.length;
          rows++;

	  AceObject[] thisRow = new AceObject[elements.length];

	  // first create the linked list of objects
	  previous = null;
	  for (int i = elements.length-1; i >= 0; i--) {
          AceString aS = new AceString(elements[i]);
	thisRow[i] = new AceObject();
	thisRow[i].classe = aS.aceClass();
	thisRow[i].name = aS.aceID();
	thisRow[i].database = db;
	thisRow[i].tried2fill = true;
	thisRow[i].filled = true;
	thisRow[i].right = previous; // link it up on the right
	previous = thisRow[i];   // remember the current object
	  }
 
	  if (rowhead == null) {
	this.right = thisRow[0];
	if (name.equals(""))
	    name = runTimeName;
	if (classe.equals(""))
	  classe = runTimeClass;
	  } else {
	rowhead.down = thisRow[0];
	  }

	  // remember the current row
	  rowhead = thisRow[0];
	}
	return true;
  }  
  protected AceObject low_fill () throws IOException {
	return database.table_fill(this);
  }  
  protected static String paramToString(String name,String[] parameters) {
	StringBuffer concat = new StringBuffer(name + ":");
	for (int i = 0; i < parameters.length; i++) {
	  concat.append(parameters[i]);
	  if (i < parameters.length - 1)
	concat.append(",");
	}
	return concat.toString();
  }  
  protected String registration_id() {
	if (params != null) // params.length > 0)
	  { // System.err.println("Registration_id: " + paramToString(name(),params)) ;
	return paramToString(name(),params);
	  }
	else
	  return paramToString(name(),theQuery);
  }  
  // return rows and column counts
  public int rowCount() {
	fill();
	return this.rows;
  }  
  // return the result as a nice table
  public AceObject[][] toTable () {
	fill();
	AceObject[][] result = new AceObject[rows][columns];
	// traverse our tree
	AceObject rowHeader = right;
	int r = 0;
	while (rowHeader != null) {
	  int c = 0;
	  AceObject current = rowHeader;
	  while (current != null) {
	result[r][c] = current;
	current = current.right;
	c++;
	  }
	  rowHeader = rowHeader.down;
	  r++;
	}
	return result;
  }  
}