/*
 * JSAceClient.java
 *
 * Created on June 13, 2000, 6:58 PM
 */

package jadeSocket;

/**
 *
 * @author  wgm
 * @version
 */

import java.io.*;
import java.net.*;

public class JSAceClient {

    private Socket s = null;
    private DataInputStream dis = null;
    private PrintStream ps = null;
    private String random = null; //32 bytes MD5 random string from the server
    private boolean first = true; // A flag for get random
    private int clientID = 0; // ID of this client
    private int port; //port of the server
    private String host; //host name of the database server
    private String msgtype; //message type. This variable should not be a global one?
    private String message; //string obtained from the server

    private boolean debug = false; //Control the debug mode.
    private boolean debug1 = false;

    /** Creates new JSAceClient */
    public JSAceClient(String host,int port) {
        this.host = host;
        this.port = port;
        //initialize();
    }

    public JSAceClient(String host, int port, boolean debug1) {
        this(host, port);
        this.debug1 = debug1;
    }

    /** Treat msgtype as a property
     */
    private void setMsgType(String mst) {
        this.msgtype = mst;
    }

    public String getMsgType() {
        return msgtype;
    }

    /** message from the server is another property
     */
    private void setMessage (String msg) {
        this.message = msg;
    }

    public String getMessage () {
        return message;
    }


    private void setRandom (String rs) {
        this.random = rs;
    }
    /**Return the random number to other class in the same package.
     */
    protected String getRandom() {
        return random;
    }


    /** The method is used to initialize the socket and DataInputStream and outputstream. */

    private void initialize() {
        if (s!= null) {
            if (debug) System.out.println("Same socket");
            return ;
        }
        try {
            s = new Socket (host, port);
            if (debug) System.out.println("Host connected...");
            ps = new PrintStream(s.getOutputStream(), true); //enable autoflush
            //ps = new DataOutputStream(s.getOutputStream());
            dis = new DataInputStream(s.getInputStream());
        }
    catch (IOException e) { System.out.println(e); }
    }

    /** Use this method to start contacting with the server */
    //public synchronized void startCommunication() {
    public void startCommunication() {
        try {
            talk(HELLO); //Say "Hello" to the server
            //if (msgtype.equalsIgnoreCase(ACESERV_MSGREQ)) {  //For local testing server
            if (msgtype.equalsIgnoreCase(ACESERV_MSGOK)) { //Based on the reaction of the server, we decide what to do.
                if (!verify()) {
                    System.out.println("You cannot be authorized. Sorry!");
                    quit();  //Verify the identity of the client.
                }
                //verify();
                while (true) { //If we can be verified by the server, we can start to talk.
                    System.out.print("AceDB_Socket> ");
                    LineNumberReader query = new LineNumberReader(new InputStreamReader(System.in));
                    String queryS = query.readLine();
                    if (queryS.length() == 4) queryS = queryS + " ";
                    if (debug) System.out.println(queryS);
                    talk(queryS);
                    if (queryS.trim().equalsIgnoreCase("quit")) quit();
                    if (msgtype.equalsIgnoreCase(ACESERV_MSGKILL)) {
                        System.out.println("Connection is closed by the server. Try again later");
                        quit();
                    }
                }
            }
            else {
                System.out.println("Server cannot talk...");
                quit();
            }
        }
        catch (IOException e) {
            System.out.println(e);
            quit();
        }
    }

    /** Talk to server and get information from the server */
    public synchronized void talk(String message) throws IOException {
        if (debug) System.out.println("JSAceClient.talk(): " + message);
        initialize(); 
        send(message);
        parseMessage();
        //if (msgtype.equalsIgnoreCase(ACESERV_MSGENCORE)) talk(ENCORE);
    }

    /** This method is used to verify the identity of the client based on user ID and pwd
     *  using the MD5Test class for MD5 encripting.
     */
    /**One overloaded method to verify users by system.in
     */
    public boolean verify() throws IOException {
        String user1 = null;
        String pwd1 = null;

        try {
            LineNumberReader input = new LineNumberReader(new BufferedReader(new InputStreamReader(System.in)));
            System.out.print("Plase input user name: ");
            user1 = input.readLine();
            System.out.println("User: " + user1);
            System.out.print("Please input password (It seems that there is not a simple way to turn the echo off in java): ");
            pwd1 = input.readLine();
            System.out.println("Password: " + pwd1);

        }
    catch (IOException e) { System.out.println("verify: " + e); }
        return verify(user1, pwd1);
    }

    /** One overloaded method to verify users by some other ways
     */
    public boolean verify(String user, String pwd) throws IOException {

        String hash = getHash(user, pwd); //It looks more natural to send all these arguments to this method?
        StringBuffer msgB = new StringBuffer();
        msgB.append(user);
        msgB.append(" ");
        msgB.append(hash);
        String msg = msgB.toString();
        if (debug) System.out.println("The sending message: " + msg);
        talk(msg);

        return msgtype.equalsIgnoreCase(ACESERV_MSGOK);
    }

    protected synchronized void send(String msg) {
        int length = msg.length();
        if (debug) System.out.println("ClientID: " + clientID);
        String request = (msg.equalsIgnoreCase("encore")) ? ACESERV_MSGENCORE : ACESERV_MSGREQ;
        byte[] message = encodeMessage(MAGIC, length, VERSION, clientID, 0, request, msg);
        if (debug) System.out.println(new String(message));
        try {
            ps.write(message, 0, message.length);
            //ps.println(); //Only used in local testing.
            ps.flush();
        }
    catch (Exception e) { System.out.println("Send method: " + e); quit(); }
    }

    /** Calculate the hash of user and pwd with random
     */
    private String getHash(String user,String pwd) {
        MD5Test md = new MD5Test();
    String[] message1 = {user, pwd};
        String hash1 = md.getMD5(message1);
        if (debug) System.out.println(hash1);
    String[] message2 = {hash1, random};
        String hash2 = md.getMD5(message2);
        if (debug) System.out.println(hash2);
        return hash2;
    }

    /** Quit program gracely */
    private void quit() {
        close();
        System.out.println("Closing the connection to the server. Bye");
        System.exit(0);
    }

    public static void main(String[] args) {
        if (args.length != 2) {
            System.out.println("Usage: java host port");
            System.exit(0);
        }
        int port = Integer.parseInt(args[1]);
        String host = args[0];
        JSAceClient test = new JSAceClient(host, port, true);
        test.startCommunication();
    }

    /** This method is used to parse the returned message from the database server */
    protected synchronized void parseMessage() throws IOException {

        byte[] msg = null;
        int total = 0;

        //try {
        String result;
        int in;
        if (debug) System.out.println("Start parsing...");
        in = dis.readInt();
        if (debug) System.out.println("Magic: 0x" + Integer.toHexString(in));
        int length = dis.readInt();
        if (debug) System.out.println("Length: " + length);
        in = dis.readInt();
        if (debug) System.out.println("Version: " + in);
        clientID = dis.readInt();
        if (debug) System.out.println("ClientID: " + clientID);
        in = dis.readInt();
        if (debug) System.out.println("MaxBytes: " + in);
        byte[] msgtypeB = new byte[30];
        int msgtypeLength = dis.read(msgtypeB, 0, 30);
        msgtype = new String(msgtypeB).trim();
        if (debug){
            System.out.print("MsgType: " + msgtype);
            System.out.println(" ---- length: " + msgtypeLength);
        }
        setMsgType(msgtype);

        //msg = new byte[length];
        msg = new byte[length-1]; // Thanks to Thomas Down and Matthew Pocock.
        total = 0;                 // I have to refer their code in AceSocket class
        dis.readFully(msg);       // in org.acedb.socket.
        /*                         // What a trick in Java? Who knows!!!!
        for (int i = 0; i < length; i++) {
        msg[i] = dis.readByte();
        total ++;
        }
         */
        dis.skipBytes(1);         // Remove null from the DataInputStream.
        //}
        //catch (IOException e) {System.out.println("Exception in parseMessage: " + e); }

        String ss = new String(msg).trim();
        if ((ss.indexOf("you are an unknown client")) > 0) {
            throw new UnknownUserException();
        }
        if (first)  {   // Get MD5 random string and put it into random
            setRandom(ss);
            first = false;
        }
        else setMessage(ss);
        if (debug1) {
            System.out.println("Message: " + ss);
        }
        if (debug)  System.out.println("Message length: " + total);

        //close();
    }

    private void close() {
        try {
            dis.close();
            dis = null;
            ps.close();
            ps = null;
            s.close();
            s = null;
        }
    catch (IOException e) { System.out.println(e); }
    }

    /** This method is used to encode the header and message to byte array.
     *  The long arguments are header and the message itself of the client
     */
    protected synchronized byte[] encodeMessage(int magic,int length,int version,int clientID,int maxBytes,String request,String message) {
        byte[] rtn = null;

        try {
            ByteArrayOutputStream bao = new ByteArrayOutputStream();
            DataOutputStream dos = new DataOutputStream(bao);
            dos.writeInt(magic);      //Write the magic number
            dos.writeInt(length+1);   //Write the length of the message + null
            dos.writeInt(version);    //Write the version number
            dos.writeInt(clientID);   //Write the client number----the important number for this client
            dos.writeInt(maxBytes);   //Write the maxbytes
            dos.writeBytes(request);  //Write the msgtype as bytes

            //Padding the msgtype to 30 bytes
            byte pad = (byte) '\u0000';
            while (bao.size() < HEAD_LENGTH)
            dos.writeByte(pad);

            //Write the message as bytes
            dos.writeBytes(message);

            //Adding one byte null
            byte nu[] = new byte[1];
            nu[0] = (byte) '\u0000';
            dos.write(nu, 0, 1);

            //Transform all these information into a byte array
            rtn = bao.toByteArray();

            //Close these two streams
            bao.close();
            dos.close();
        }
    catch (IOException e) { System.out.println(e); } //Hopefully, everything is OK.

        return rtn;
    }

    /** Static variables */
    public final static int MAGIC = 0x12345678;
    public final static int VERSION = 1;
    public final static String ACESERV_MSGREQ = new String("ACESERV_MSGREQ");
    public final static String ACESERV_MSGOK = new String("ACESERV_MSGOK");
    public final static String ACESERV_MSGDATA = new String("ACESERV_MSGDATA");
    public final static String ACESERV_MSGENCORE = new String("ACESERV_MSGENCORE");
    public final static String ACESERV_MSGKILL = new String("ACESERV_MSGKILL");
    public final static String ACESERV_MSGFAIL = new String("ACESERV_MSGFAIL");
    public final static String HELLO = new String("bonjour");
    public final static String ENCORE = new String("encore");

    /** final number used in the objects of this class */
    private final int HEAD_LENGTH = 50;
}