/** This class is used to connect the jade api
 */

package jadeSocket;

import java.io.*;
import java.net.*;

import jade.db.*;

public class JSAceServer {
	
	private ServerSocket ss = null; // Listening the socket from the jade
	private String host = null; // The server host name
	private int port; // The port number of the server
	private final int SERVER_PORT = 40000;
	
	private boolean debug = false; // for debug mode
						
	public static void main (String args[]) {
          if (args.length != 2) {
            System.out.println("Usage: java JSAceServer host port");
            System.exit(0);
          }
          
          String host = args[0];
          int port = Integer.parseInt(args[1]);
          new JSAceServer(host, port);
        }
        
        
        public JSAceServer (String host, int port) {
		this.host = host;
		this.port = port;
		try { 
			ss = new ServerSocket(SERVER_PORT);
                        if (debug) System.out.println("JSAceServer is listening...");
			while (true) {
				Socket s = ss.accept(); 
				Thread t = new JadeThread(s);
				t.start();
				if (debug) System.out.println("Get Connected from jade client.");
			}
		}
		catch (IOException e) { System.out.println("JSAceServer Constructor: " + e); }
	}
	
	/** The inner class for multithreading communication between jade and server
	 */
	class JadeThread extends Thread {
		
		private Socket s = null;
		private LineNumberReader lnr = null;
		private PrintWriter pw = null;
		private JSControl control = null;
		
		JadeThread (Socket s) {
			this.s = s;
			try { 
				lnr = new LineNumberReader(new BufferedReader(new InputStreamReader(s.getInputStream())));
				pw = new PrintWriter(new OutputStreamWriter(s.getOutputStream()));
			}
			catch (IOException e) { System.out.println(e); }
			
			control = new JSControl(host, port);
                        control.initializeRequest("guest", "guest");
                        send(AceAccessor.OK + " hello");
		}
		
		private void send(String s) {
                  char[] sb = s.toCharArray();
                  int length = sb.length;
                  pw.println(length);
                  if (debug) System.out.println("Length in JSAceServer: " + length);
                  pw.write(sb, 0, length);
                  //pw.println();
                  pw.flush();
                }
                
                private String digest(String s) {
                  String rtnString = null;
                  StringBuffer buffer = new StringBuffer();
                  LineNumberReader lnr = new LineNumberReader(new CharArrayReader(s.toCharArray()));
                  String line = null;
                  int pos;
                  int pos1;
                  try {
                    while ((line = lnr.readLine()) != null) {
                      
                      if ((pos = line.indexOf("//")) != -1) {
                        if ((pos1 = line.indexOf("Active Object")) != -1) {
                          String numberS = line.substring(pos+2, pos1).trim();
                          //int number = Integer.parseInt(numberS);
                          buffer.insert(0, numberS + " ");
                        }
                        line = line.substring(0, pos);
                        //line = line + "\r\n";
                      }
                      line = line + "\r\n";
                      buffer.append(line);
                    }
                  }
                  catch (IOException e) { System.out.println(e); }
                 
                  return buffer.toString();
                }
                
                public void run () {
			
			while (true) {
				try {
					String query = lnr.readLine();
					if (debug) System.out.println(query);
					//if (query == null) break;
                                        if (query.length() == 4) query = query + " ";
                                        if ((query.indexOf("list") != -1) || (query.indexOf("show") != -1))
                                          query = query + " -j";
                                        control.request(query);
					String result = control.getMessage();
					result = digest(result);
                                        result = AceAccessor.OK + " " + result;
                                        send(result);
                                        send(".");
					if (query.endsWith("quit")) break;
				}
				catch (IOException e) { System.out.println(e);  break; }
           }
           try {
			   lnr.close();
               s.close();
               if (debug) System.out.println("Client closed.");
           }
           catch (IOException e) { System.out.println("Thread: " + e); }
      }
	}
}
