package jadex.swing;

/**
 * This type was created in VisualAge.
 */
import java.util.*;
import jade.db.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;


/** This model class stores all object information in a specified class to be displayed
 * in the ClasslistDisplayer.
*/

public class ClassListModel {
	private String name;  //Used to hold the class name.
	private java.beans.PropertyChangeSupport propertyChange;
	private AceAccessor accessor = null;
	private AceAccessorFacade facade = null;
	private JLabel startingLabel = null;
	private Vector letterList = new Vector();
	private JPanel fieldCurrentPanel = new JPanel();
	private Thread queryThread = new Thread(); //Thread used to query the objects.
	private Vector list = null;
/**
 * Default constructor.
 */
public ClassListModel() {
	super();
}
/**
 * An overloaded construtor.
 * @param accessor the AceAccessor for accessing the AceDB.
 * @param label the selected JLabel from ClassDisplayer.
 * @param classModel the model storing all counting information for this class.
 */
public ClassListModel(AceAccessor accessor, javax.swing.JLabel label, ClassDisplayerModel classModel) {
	super();
	this.accessor = accessor;
	this.facade = new AceAccessorFacade(accessor);
	this.startingLabel = label;
	setClassName(classModel.getClassName());
	setLetterList(classModel.getLabelAll(), classModel.getLabels());
}
/**
 * The addPropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void addPropertyChangeListener(java.beans.PropertyChangeListener listener) {
	getPropertyChange().addPropertyChangeListener(listener);
}
/**
 * Fetch the objects with leading letter as letter from start to start + legnth -1.
 * @param letter The leading letter of the object names
 * @param start the start position of the object in the total list of the class
 * @param length the total number of the objects to be fetched.
 */

public void fetchObjects(String letter, int start, int length) {
	String pattern = null;
	if (letter.equalsIgnoreCase("Total")) pattern = "*";
	else pattern = letter;
	
	final String query = "find " + getClassName() + " " + pattern;
	final int localStart = start;
	final int localLength = length;
	
	queryThread = new Thread(new Runnable() {   //All database query stuff is done here
		public void run() {
			WelcomePane information = new WelcomePane();
			information.setMessage("Downloading data from AceDB...");
			setCurrentPanel(information);
			Thread.yield();
			AceObject[] objects = null;
			try {
				objects = facade.getAceObjects(query, localStart, localLength);
				information.setMessage("Downloading finished.");
				information.repaint();
				Thread.yield();
				list = new Vector();
				for (int i = 0; i < objects.length; i++) 
					list.addElement(objects[i]);
				setList(list);
			}
			catch (Throwable t) {
				information.setMessage("Downloading failed. Try again.");
				return;
			}
		}
	});
	queryThread.start();
}
/**
 * The firePropertyChange method was generated to support the propertyChange field.
 */
public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
	getPropertyChange().firePropertyChange(propertyName, oldValue, newValue);
}
/**
 * Get the AceAccessor.
 * @return jade.db.AceAccessor
 */
public AceAccessor getAceAccessor() {
	return accessor;
}
public String getClassName() {return this.name; }
/**
 * Gets the currentPanel property (javax.swing.JPanel) value.
 * @return The currentPanel property value.
 * @see #setCurrentPanel
 */
public JPanel getCurrentPanel() {
	return fieldCurrentPanel;
}
public Vector getLetterList() { return letterList; }
/**
 * Accessor for the propertyChange field.
 */
protected java.beans.PropertyChangeSupport getPropertyChange() {
	if (propertyChange == null) {
		propertyChange = new java.beans.PropertyChangeSupport(this);
	};
	return propertyChange;
}
public JLabel getStartingLabel() { return startingLabel; }
/**
 * The removePropertyChangeListener method was generated to support the propertyChange field.
 */
public synchronized void removePropertyChangeListener(java.beans.PropertyChangeListener listener) {
	getPropertyChange().removePropertyChangeListener(listener);
}
/**
 * Set AceAccessor for accessing the AceDB.
 * @param model jadex.swing.AceModel
 */
public void setAceAccessor(AceAccessor model) {
	if (accessor == null)
		accessor = model;
}
public void setClassName(String name) {this.name = name; }
/**
 * Sets the currentPanel property (javax.swing.JPanel) value.
 * @param currentPanel The new value for the property.
 * @see #getCurrentPanel
 */
public void setCurrentPanel(JPanel currentPanel) {
	JPanel oldValue = fieldCurrentPanel;
	fieldCurrentPanel = currentPanel;
	firePropertyChange("currentPanel", oldValue, currentPanel);
}
public void setLetterList(JLabel[] letter) { 
	for (int i = 0; i < letter.length; i++) {
		if (letter[i].getToolTipText() != null)
			letterList.addElement(letter[i].getToolTipText());
	}
}
public void setLetterList(JLabel labelAll, JLabel[] letter) { 
	letterList.addElement(labelAll.getToolTipText()); //Always put the total entries on the top
	for (int i = 0; i < letter.length; i++) {
		if (letter[i].getToolTipText() != null)
			letterList.addElement(letter[i].getToolTipText());
	}
}
public void setLetterList(Vector letter) { 
	this.letterList = letter;
}
/**
 * Displaying all objects in list in the listPanel.
 * @param list java.util.Vector
 */
 
private void setList(Vector list) {
	JPanel listPanel = new JPanel();
	int total = list.size();
	int row = (int) (total/10.0 + 0.9);
	java.awt.GridLayout grid = new java.awt.GridLayout(row, 10, 3, 1);
	listPanel.setLayout(grid);

	//final Hashtable openFrames = new Hashtable();
	for (int i = 0; i < total; i++) {
		AceObject obj = (AceObject) list.elementAt(i);
		JLabel label = new ObjectLabel(obj);
		label.setForeground(Color.blue);
		listPanel.add(label);
	}

	setCurrentPanel(listPanel);
}
public void setStartingLabel(JLabel label) { this.startingLabel = label; }
}