package jadex.swing;

/**
 * This type was created in VisualAge.
 */
import jade.db.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;

import jadex.swing.classView.*;

/** This class is used to control the display for individual AceObject.
 */

public class ObjectViewController {

    private AceObject localObj = null; // Holder for the local AceObject
    private Properties viewProp = null;
    //private boolean propLoaded = false; // Check if view.prop is loaded.

    /**
     * Default constructor.
     */
    public ObjectViewController() {
        super();
    }
    /**
     * A overloaded construcotr.
     * @param obj the AceObject to be displayed.
     */
    public ObjectViewController(AceObject obj) {
        this();
        setAceObject(obj);
    }

    /** Get the AceObject.
     * @return the AceObject displayed.
    */
    public AceObject getAceObject() {
        return localObj;
    }
    /** Based on the types of AceObject, privide different views of AceObjects. 
     * The default view is JTree. JTextArea is used to show the longtext AceObject.
     * @return A JFrame showing the information on the AceObject.
    */

    public JFrame getObjectView() {

        JFrame jc = null;
        Class c = null;
        String viewName = checkProperties();
        
        if ((viewName != null) && (viewName.length() != 0)) {
            try {
                c = Class.forName(viewName);
                ClassView view = (ClassView) c.newInstance();
                view.setObject(localObj);
                return view.getViewFrame();
            }
            catch (Exception e) {
                System.out.println("getObjectView() in ObjectViewController: " + e);
                //It is guaranted that all aceobjects have treeviews.
            }
        }
        jc = new TreeClassView(localObj).getViewFrame(); ;
        return jc;
    }

    /** Check the display view setting in the property file.
     * @return the view class name to be used for the dispaying the AceObject.
    */
    private String checkProperties() {

        String rtn = null;

        viewProp = new Properties();
        String name = "jadex" + File.separator + "view.prop";
        try {
            FileInputStream fis = new FileInputStream(new File(name));
            viewProp.load(fis);
        }
    catch (Exception e) { return null; }

    // Both Geneticmap and SequenceMap both are Map in AceDB. However, they should be 
    // distinguished.
    String cls = checkForMap(localObj);
        rtn = viewProp.getProperty(cls, null);
        return rtn;
    }

    /** A helper to distinguish two kinds of Map: GeneticMap and SequenceMap
    */
    private String checkForMap(AceObject obj) {
        // Not a map, do as usually.
        if (!(obj.classe.equalsIgnoreCase("Map"))) return obj.classe;
        else {
            if ((obj.name.indexOf("Sequence")) > -1) return "Sequence_map";
            else return "Genetic_map";
        }
    }
 
    /** Set the Aceobject to be displayed.
    */
    public void setAceObject (AceObject obj) {
        try {
            AceAccessor accessor = (AceAccessor) obj.accessor();
            localObj = accessor.fetch(obj.classe(), obj.name());
        }
        catch (Exception e) {
            System.out.println("Error in fetching the AceObject");
        }
    }
}