package jadex.swing;

/**
 * This type was created in VisualAge.
 * A singleton to index the open frames,
 */

import java.util.*;
import javax.swing.*;
import jade.db.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;

/** A singleton to control the individual AceObject displaying.
*/

public class OpenFrames {
    private Hashtable index = null;
    private static OpenFrames instance = null;
    private Applet applet = null;
    /**
     * OpenFrames constructor comment.
     */
    protected OpenFrames() {
        super();
        index = new Hashtable();
    }
    
    /** Register a displaying AceObject.
     * @param title the title of the JFrame as the key for the HashTable.
     * @param f the JFrame displaying the AceObject.
    */
    public void addFrame(String title, JFrame f) {
        index.put(title, f);
    }
    
    /** Get the JFrame with the title.
     * @param title the title as the key for the HashTable searching.
     * @return the JFrame displaying an AceObject.
    */
    public JFrame getFrame(String title) {
        if (index.containsKey(title)) {
            return (JFrame) index.get(title);
        }
    else { return null; }
    }
    
    /** This is the only method to get the instance of this class.
     * @return the sole instance of this class.
    */
    public static OpenFrames getInstance () {
    if (instance == null) { return instance = new OpenFrames(); }
    else { return instance; }
    }
    
    /** Remove the JFrame with title title.
    */
    public void removeFrame(String title) {
        index.remove(title);
    }

    /** Get the applet. If this class is used in an applet, the default applet will returned.
    */
    public Applet getApplet() {
        return applet;
    }

    /** Set the applet. For applet use.
    */
    public void setApplet(Applet a) { this.applet = a; }

    /** Clients should call this method to display the individual AceObject. 
    *   In an applicatin, a new JFrame will be popup. In an applet, use <pre>
    * show(String url, String name) </pre>.
     * @param o the AceObject to be displayed.
    * @see #show(String, String).
    */
    public void show(AceObject o) {
        final String title = o.classe() + ": " + o.name();
        JFrame frame = null;
        frame = getFrame(title);
        if (frame == null) {
            ObjectViewController ovc = new ObjectViewController(o);
            frame = ovc.getObjectView();
            frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    JFrame f = (JFrame) e.getSource();
                    removeFrame(title);
                    f.setVisible(false);
                    f.dispose();
                }
            });
            frame.validate();
            addFrame(title, frame);
        }
        else {
            frame.setState(Frame.NORMAL);
            frame.setVisible(true);
            frame.requestFocus();
            frame.validate();
        }
    }

    /** A overloaded method for displaying AceObjects in Applets.
    * @param url the URL for fetch the information for the AceObject. It is from
     * <a href="http://www.wormbase.org">the Wormbase</a> usually.
     * @param name the name of the AceObject to be displayed.
    */
    public boolean show(String url, String name) {
        if (applet == null) return false;

        try {
            URL u = new URL(url);
            getApplet().getAppletContext().showDocument(u, name);
            return true;
        }
        catch (Exception e) {
            System.err.println(e);
            return false;
        }
    }
}