/*
 * AbstractObjectView.java
 *
 * Created on July 6, 2000, 11:30 AM
 */

package jadex.swing.classView;

import javax.swing.*;
import jade.db.*;
import java.awt.event.*;

/**
 * This is the super class for all ClassView. Each specific classView, e.g., MapView,
 * SequenceView, or SequenceMapView, should be subclassed it for automatically loading.
 * @author  wgm
 * @version
 */
public abstract class ClassView {

    /** Set the AceObject to be displayed by this ClassView.
     * @param object the AceObject to be displayed.
     */
    public abstract void setObject(AceObject object);

    /** Get the displayed AceObject.
    * @return the AceObject displayed.
    */
    public abstract AceObject getObject();

    /** Get the JComponent used to display the AceObject.
    * @return the JComponent displaying this AceObject.
    */
    public abstract JComponent getView();

    /** Set a JComponent for displaying the AceObject.
    * @param comp the JComponent to be used for displaying this AceObject.
    */
    public abstract void setView(JComponent comp);

    /** Get a self-showable JFrame displaying the AceObject.
    * @return The JFrame displaying the AceObject.
    */
    public JFrame getViewFrame() {
        JFrame frame= new JFrame(getObject().classe() + ": " + getObject().name());
        frame.setSize(400, 300);
        frame.getContentPane().add(getView(), "Center");
        frame.setVisible(true);
        return frame;
    }
}