/*
 * MapViewContainer.java: A JFrame as a container for both genetic and sequence map views
 *
 * Created on October 22, 2000, 3:24 PM
 */

package jadex.swing.classView;

import jadex.swing.map.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A JFrame as a container for both genetic and sequence map views. However, present version of 
* {@link MapView MapView} has not used this class. 
 * @author  wgm
 * @version 1.0
 */
public class MapViewContainer extends JFrame {
    private LocusPanel marker = null; // markers for locus
    private JPanel markerPanel = null; // holding marker panel
    private MapCartoon locator = null; // for zooming.
    private JPanel locatorPanel = null; // holding locator.
    private JPanel markerAndLocatorPanel = null; // holding marker and locator panels
    private RangeControlPanel rangeControl = null; // For zooming
    private JPanel controlPanel = null;// The left controal panel holding marke, locator and rangecontrol

    private MapScale1 leftScale, rightScale; // The mapscales
    private JPanel scalePane1, scalePane2; // The panels holding mapscales
    private JPanel container = null; // holding interval, locus, clone or other displays
    private MapViewContainerMenuBar menubar = null; // Menubar for this container
    
    private MapControl control = null; // Central control for all control widgets
    /** Default constructor */
    public MapViewContainer() {
        initialize();
    }
    
    /** Second construcotr with title as the argument */
    public MapViewContainer(String title) {
        super.setTitle(title);
        initialize();
    }
    
    /** GUI initialization */
    private void initialize() {
        //The parent panel holding all these panels
        JPanel pane = new JPanel();
        //pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
        pane.setLayout(new BorderLayout());
        pane.setBorder(BorderFactory.createRaisedBevelBorder());
       
        controlPanel = new JPanel();
        controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
        controlPanel.setBorder(BorderFactory.createRaisedBevelBorder());
        
        markerAndLocatorPanel = new JPanel();
        markerAndLocatorPanel.setLayout(new GridLayout(1, 2));
        markerAndLocatorPanel.setPreferredSize(new Dimension(90, 300));
        controlPanel.add(markerAndLocatorPanel);
        // Left mapscale panel
        scalePane1 = new JPanel();
        scalePane1.setLayout(new BorderLayout());
        scalePane1.add(getLabel("Scale"), "North");
        scalePane1.add(getLabel("Scale"), "South");
        // Right mapscale panel
        scalePane2 = new JPanel();
        scalePane2.setLayout(new BorderLayout());
        scalePane2.add(getLabel("Scale"), "North");
        scalePane2.add(getLabel("Scale"), "South");
        // Conainer for clonedisplay, intervaldisplay or locusdisplay
        
        container = new JPanel();
        container.setLayout(new BoxLayout(container, BoxLayout.X_AXIS));
        container.setBorder(BorderFactory.createEtchedBorder());
        container.setPreferredSize(new Dimension(600, 500));
        
        JPanel p1 = new JPanel();
        p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
        p1.add(controlPanel);
        p1.add(scalePane1);
        
        pane.add(p1, BorderLayout.WEST);
        scalePane1.setVisible(false);
        pane.add(container, BorderLayout.CENTER);
        pane.add(scalePane2, BorderLayout.EAST);
        scalePane2.setVisible(false);
         
        getContentPane().add(new JScrollPane(pane), "Center");
        setTitle("Map View: ");
    }
    
    /** Add a JPanel for displaying.
    * @param name the panel label on both north and south ends.
    * @param p1 the panel to be added.
    */
    public void addPanel(String name, JPanel p1) {
        JPanel p = new JPanel();
        p.setLayout(new BorderLayout());
        p.setBorder(BorderFactory.createEtchedBorder());
        p.add(getLabel(name), "North");
        p.add(getLabel(name), "South");
        p1.setPreferredSize(new Dimension(1000, 400));
        p.add(new JScrollPane(p1), "Center");
        container.add(p);
        if (menubar != null) {
            menubar.addViewMenuItem(name, p);
            menubar.showViewMenu(true);
        }
        container.invalidate();
        validate();
        repaint();
    }
    
    /** Remove all displayed panels.
    */
    public void removePanels() {
        container.removeAll();
        container.invalidate();
        showControlPanel(false);
        setRightMapScale(false);
        setLeftMapScale(false);
        if (menubar != null) {
            menubar.removeViewItems();
            menubar.showViewMenu(false);
        }
        setTitle("Map View: ");
        validate();
        repaint();
    }
    
    /** Create a JMenubar for this container.
    * @param menubar the JMenubar will be used in this MapContainer.
    */
    public void setMapMenuBar(JMenuBar menubar) {
        try {
            this.menubar = (MapViewContainerMenuBar) menubar;
            this.menubar.addViewMenuItem("Left Scale", scalePane1);
            this.menubar.addViewMenuItem("Right Scale", scalePane2);
            setJMenuBar(menubar);
        }
        catch(ClassCastException e) {
            System.err.println("MapViewContainer.setMapMenuBar(): " + e);
        }
    }

    /** Create a default JMenuBar with MapMapMenubar.*/
    public void setMapMenuBar() {
        MapViewContainerMenuBar menubar = new MapViewContainerMenuBar(this);
        menubar.setFileMenu();
        menubar.setViewMenu();
        menubar.setHelpMenu();
        setMapMenuBar(menubar);
    }
    
    /** A helper method to create a JLabel with text label.
    * @param lable the text to be used in the label.
    * @return the formatted JLabel with label as its text.
    */
    private JLabel getLabel(String label) {
        JLabel n1 = new JLabel(label);
        n1.setBorder(BorderFactory.createEtchedBorder());
        n1.setForeground(Color.black);
        n1.setHorizontalAlignment(JLabel.CENTER);
        n1.setFont(new Font("Dialog", 0, 10));
        return n1;
    }
    /** create a marker panel from locus.
    * @param p the LocusPanel used as the markerpanel.
    */
    public void setMarker(LocusPanel p) {
        //Adding the marker
        markerPanel = new JPanel();
        markerPanel.setLayout(new BorderLayout());
        markerPanel.add(getLabel("Marker"), "North");
        markerPanel.add(getLabel("Marker"), "South");
        marker.setPreferredSize(new Dimension(50, 500));
        marker.setBorder(BorderFactory.createEtchedBorder());
        markerPanel.add(p, "Center");
        markerAndLocatorPanel.add(markerPanel);
    }
    
    /** create a mapcartoon.
    * @param show the mapcartoon should be displayed if true. Otherwise, no cartoon displayed.
    */
    public void setMapCartoon(boolean show) {
        if (show) {
            if (locator == null) {// First time call this method
                locator = new MapCartoon();
                locatorPanel = new JPanel();
                locatorPanel.setLayout(new BorderLayout());
                locatorPanel.add(getLabel("Locator"), "North");
                locatorPanel.add(getLabel("Locator"), "South");
                //locator.setPreferredSize(new Dimension(40, 500));
                locator.setBorder(BorderFactory.createEtchedBorder());
                //locator.setMapControl(control);
                locatorPanel.add(locator, "Center");
                markerAndLocatorPanel.add(locatorPanel);
                markerAndLocatorPanel.invalidate();
                validate();
            }
            else locatorPanel.setVisible(true);
        }
        else {
            if (locatorPanel != null) locatorPanel.setVisible(false);
        }
    }
    
    /** create a range control panel.
    * @param show the RangeControl should be displayed if show is true. Otherwise, not displayed.
    */
    public void setRangeControl(boolean show) {
        if (show) {
            if (rangeControl == null) { // Create a range control panel first
                rangeControl = new RangeControlPanel();
                rangeControl.setBorder(BorderFactory.createEtchedBorder());
                rangeControl.setPreferredSize(new Dimension(90, 120));
                controlPanel.add(rangeControl);
                controlPanel.invalidate();
                validate();
             }
            rangeControl.setVisible(true);
        }
        else if (rangeControl != null) rangeControl.setVisible(false);
    }
    
    /** create the left mapscale.
    * @param show show LeftMapScale if true. 
    */
    public void setLeftMapScale(boolean show) {
        if (show) {
            if (leftScale == null) {
                leftScale = new MapScale1();
                leftScale.setPreferredSize(new Dimension(50, 500));
                leftScale.setBorder(BorderFactory.createEtchedBorder());
                scalePane1.add(leftScale, "Center");
                scalePane1.setVisible(true);
            }
            else scalePane1.setVisible(true);
        }
        else if (scalePane1 != null) scalePane1.setVisible(false);
    } 
    
    /** create the right mapscale.
    * @param show show rightmalscale if show is true.
    */
    public void setRightMapScale(boolean show) {
        if (show) {
            if (rightScale == null) {
                rightScale = new MapScale1();
                rightScale.setPreferredSize(new Dimension(50, 500));
                rightScale.setBorder(BorderFactory.createEtchedBorder());
                scalePane2.add(rightScale, "Center");
                scalePane2.setVisible(true);
            }
            else scalePane2.setVisible(true);
        }
        else if (scalePane2 != null) scalePane2.setVisible(false);
    }
    
    /** Control if both MapCartoon and RangeControl should be displayed.
    * @param show true, not showing; false, showing.
    */
    public void showControlPanel(boolean show) {
        setMapCartoon(show);
        setRangeControl(show);
        markerAndLocatorPanel.setVisible(show);
    }
    
    /** Set MapControl to coordinate all zooming activities. 
    * @param c the mapcontrol to be used.
    */
    public void setMapControl(MapControl c) {
        control = c;
        locator.setMap(control.getMap());
        locator.setMapControl(control);
        rangeControl.setMapControl(control);
        leftScale.setRange(control.getMap());
        rightScale.setRange(control.getMap());
        
        control.addPropertyChangeListener(locator);
        control.addPropertyChangeListener(leftScale);
        control.addPropertyChangeListener(rightScale);
    }
    
    /** Get the MapControl used in this MapContainer.
    * @return the MapControl used in this MapContainer.
    */
    public MapControl getMapControl() {
        return control;
    }
    
    /** A test method.
    */
    public static void main(String[] args) {
        MapViewContainer container = new MapViewContainer("A Test");
        container.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) { System.exit(0); }
        });
        container.setMapCartoon(true);
        container.setRangeControl(true);
        container.setLeftMapScale(true);
        container.setRightMapScale(true);
        container.setMapMenuBar();
        container.setSize(800, 600);
        container.setVisible(true);
    }
}