/*
 * MapViewContainerMenuBar.java
 *
 * Created on July 11, 2000, 2:13 PM
 */

package jadex.swing.classView;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import jadex.swing.*;

/**
 * This is a JMenubar for MapViewContainer.
 * @author  wgm
 * @version 1.0
 */
public class MapViewContainerMenuBar extends javax.swing.JMenuBar {

    private MapViewContainer container = null; // MapViewContainer this MenuBar will be attached
    private JMenu viewMenu = null; // view
    private JMenu items = null; // View Items
    private Map itemMap = null; // Holding all items
    private JMenu helpMenu = null; // help menu
    private JMenu fileMenu = null; // File menu
    private JMenuItem open = null; // This file menuitem should be kept
    private Font dFont = new Font("Dialog", 0, 11); // default font
    
    /** Default constructor */
    public MapViewContainerMenuBar() {
    }

    /** An overloaded constructor.
    * @param container the MapViewContainer this Menubar used for.
    */
    public MapViewContainerMenuBar(MapViewContainer container) {
        this.container = container;
    }
   
    /** Setup the file Menu.
    */
    public void setFileMenu() {
        fileMenu = new JMenu("File");
        fileMenu.setFont(dFont);
        open = new JMenuItem("Open");
        open.setFont(dFont);
        //open.addActionListener(new OpenFileAction(f, getMapView()));
        final JMenuItem close = new JMenuItem("close");
        close.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (container != null) {
                    container.removePanels();
                    close.setEnabled(false);
                    showHelpMenu(false);
                }
            }
        });
        close.setFont(dFont);
        close.setEnabled(false);
        open.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                close.setEnabled(true);
                showHelpMenu(true);
            }
        });
        JMenuItem quit = new JMenuItem("Quit");
        quit.setFont(dFont);
        quit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (container != null) {
                    container.setVisible(false);
                    container.dispose();
                }
                System.exit(0);
            }
        });
        
        fileMenu.add(open);
        fileMenu.add(new JSeparator());
        fileMenu.add(close);
        fileMenu.add(new JSeparator());
        fileMenu.add(quit);
        this.add(fileMenu);
    }

    /** Applet File menu: quit only */
    public void setFileMenuForApplet() {
        fileMenu = new JMenu("File");
        fileMenu.setFont(dFont);
        JMenuItem quit = new JMenuItem("Quit");
        quit.setFont(dFont);
        quit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
    }
    
    /** Control if the File menu should be displayed.
    * @param show, true for show.
    */
    public void showFileMenu(boolean show) {
        if (show) {
            if (fileMenu == null) setFileMenu();
            else fileMenu.setVisible(true);
        }
        else fileMenu.setVisible(false);
    }
    
    /** Set the actionListener for the file open MenuItem.
    * @param l the ActionListener for the File Opem MenuItem.
    */
    public void setFileOpenAction(ActionListener l) {
        if (open != null) open.addActionListener(l);
    }
    
    /** Set up the View Menu.
    */
    public void setViewMenu() {
        viewMenu = new JMenu("View");
        viewMenu.setFont(dFont);
        items = new JMenu("Items");
        items.setFont(dFont);
        viewMenu.add(items);
        this.add(viewMenu);
        itemMap = new HashMap();
    }
    
    /** Add controllable View Items to the View Menu Item.
    * @param label the text label that will be displayed on the View Item menuItem.
    * @param l the JPanel controlled by this View Item.
    */
    public void addViewMenuItem(String label, final JPanel p) {
        final JCheckBox item = new JCheckBox(label);
        item.setFont(dFont);
        item.setSelected(true);
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (item.getSelectedObjects() != null) p.setVisible(true);
                else p.setVisible(false);
            }
        });
        items.add(item);
        itemMap.put(label, item);
    }
    
    /** Remove the view Item from the View MenuItem.
    * @param label the text label used as a key for the displayed JPanel.
    */
    public void removeViewItem(String label) {
        JCheckBox jcb = (JCheckBox) itemMap.get(label);
        if (jcb != null) items.remove(jcb);
    }
    
    /** Remove all menuitems except left and right mapscales*/
    public void removeViewItems() {
        int count = items.getItemCount();
        // Remove all other checkbox
        if (count > 2) {
            for (int i = 2; i < count; i++) {
                items.remove(i);
            }
        }
    }
    
    /** Control if the View Menu should be displayed or not.
    * @param show true for showing.
    */
    public void showViewMenu(boolean show) {
        if (show) {
            if (viewMenu == null) setViewMenu();
            else viewMenu.setVisible(true);
        }
        else if (viewMenu != null) viewMenu.setVisible(false);
    }
    
    /** Set up the help menu.
    */
    public void setHelpMenu() {
        helpMenu = new JMenu("Help");
        helpMenu.setFont(dFont);

        JMenuItem help = new JMenuItem("Help");
        help.setFont(dFont);
        /*
        help.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFrame f = new JFrame("Help");
                f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                f.setSize(400, 300);
                HelpPane h = new HelpPane(isApplet);
                h.setBorder(BorderFactory.createEtchedBorder());
                f.getContentPane().add(new JScrollPane(h), "Center");
                f.setVisible(true);
            }
        });
        */
        helpMenu.add(help);
        this.add(helpMenu);
    }
    
    /** Control if the Help Menu should be displayed or not.
    * @param show true for showing.
    */
    public void showHelpMenu(boolean show) {
        if (show) {
            if (helpMenu == null) {
                setHelpMenu();
            }
            else helpMenu.setVisible(true);
        }
        else helpMenu.setVisible(false);
    }
}