/*
 * SequenceMapView.java: This is the sequence map view, which should be implemented as a Java Bean
 *
 * Created on October 21, 2000, 9:19 PM
 */

package jadex.swing.classView;

import javax.swing.*;
import jade.db.*;
import jadex.swing.*;
import jadex.swing.sequence.*;
import jadex.swing.map.*;
import java.awt.event.*;
import java.applet.Applet;
/**
 * This class is used to display sequence_map. For gegetic_map display, please use
 * {@link MapView MapView} class.
 * @author  wgm
 * @version 1.0
 */

/** All object views should be a subclss of classView */
public class SequenceMapView extends ClassView {
    private MapViewContainer container = null; // container holding all GUI
    private MapViewContainerMenuBar menubar = null; // The menubar for container
    private String name = null; // The name of the chromosome shown in this object
    private final String PREFIX = "sequence_map_"; // A file prefix to fetch the data from the files
    // A flag to indicate if this mapview is in an applet
    private boolean isApplet = false;
    private AceObject obj = null; // The displayed AceObject.
    private JComponent comp = null; // The JComponent for displaying the AceObject.

    private CloneControl cloneControl = null; // Only for searching purpose

    /** Default constructor */
    public SequenceMapView() {
        initialize(isApplet);
    }

    /** An overloaded constructor with name as the Argument.
     * @param name the name of the Sequence_map to be displayed, e.g., Sequence-I.
     */
    public SequenceMapView(String name) {
        initialize(isApplet);
        setName(name);
    }

    /** An overloaded constructor.
     * @param name the name of the Sequence_Map to be displayed, e.g., Sequence_I.
     * @param a the Applet this Sequence_map is in.
     */
    public SequenceMapView(String name, Applet a) {
        OpenFrames.getInstance().setApplet(a);
        isApplet = true;
        initialize(isApplet);
        setName(name);
    }

    /** Initialize for applet */
    private void initialize(boolean isApplet) {
        container = new MapViewContainer();
        container.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                container.setVisible(false);
                container.dispose();
                //try { System.exit(0); }
                //catch(Exception e1) { System.err.println(e1); }
            }
        });

        menubar = new MapViewContainerMenuBar(container);

        if (isApplet) {
            menubar.setFileMenuForApplet();
        }
        else {
            menubar.setFileMenu();
        }
        menubar.setViewMenu();
        menubar.setHelpMenu();
        menubar.setFileOpenAction(new CloneOpenFileAction(container, this));

        container.setMapMenuBar(menubar);
        container.setSize(800, 600);
        container.setVisible(true);
        container.showControlPanel(false);
        menubar.showViewMenu(false);
        menubar.showHelpMenu(false);
    }

    /** Set the AceObject to be displayed.
     * @param object the AceObject to be displayed.
     * @deprecated please use {@link setName(String) setName} method.
     */
    /** aceobject is implemented as a property of this class */
    public void setObject(AceObject object){
        this.obj = object;
        setName(obj.name());
    }

    /** Set the name of the Sequence_Map to be displayed.
     * @param name the name fo the Sequence_map to be displayed.
     */
    /** Implement name as a property of this class.
     * Temporarily, I will dump all sequence_map informaion into six static files
     * to increase the performance
     **/
    public void setName(String name) {
        this.name = name;
        // A lot of stuff should take place here.
        //String file = PREFIX + name;
        //CloneFetcher fetcher = new FileCloneFetcher(file);
        CloneFetcher fetcher = new AceDBCloneFetcher((JSAceAccessor)obj.accessor(), name);
        fetcher.fetchClone(); // Fetch all information on sequence_map
        MapRange r = fetcher.getMapScale(); // get the total scale of sequence_map
        CloneDisplay display = new CloneDisplay();
        CloneControl control = new CloneControl(display);
        setCloneControl(control);
        display.setRange(r);
        control.setCloneFetcher(fetcher);
        control.setMapRange(r);
        display.setCloneControl(control);
        MapControl mapControl = new MapControl();
        mapControl.setMap(r);
        mapControl.addPropertyChangeListener(control);
        control.setMapControl(mapControl);
        container.setTitle(container.getTitle() + fetcher.getMapName());
        container.showControlPanel(true);
        container.setLeftMapScale(true);
        container.setRightMapScale(true);
        container.setMapControl(mapControl);
        container.addPanel(new String("Clone Display"), display);
        // If used in AceClient browser, turn off FileMenu.
        if (obj != null) menubar.showFileMenu(false);
        menubar.showHelpMenu(true);
    }

    /** Get the name of the Sequence_map displayed in the SequenceMapView.
     * @return the name of the Sequence_map displayed.
     */
    public String getName() {
        return this.name;
    }

    /** Get the JComponent displaying the AceObject.
     * @return the JComponent displaying the AceObject.
     */
public JComponent getView() { return this.comp; }

    /** Set the JComponent displaying the AceObject.
     * @param comp the JComponent displaying the AceObject.
     */
    public void setView(JComponent comp) {
        this.comp = comp;
    }

    /** Get the AceObject displayed.
     * @return the AceObject displayed.
     * @deprecated the AceObject will be null.
     */
public AceObject getObject() { return obj; }

    /** Set the CloneControl for this SequenceMapView.
     * @param c the cloneContorl used to coordinate all zooming activities.
     */
public void setCloneControl(CloneControl c) { this.cloneControl = c; }

    /** Get the CloneControl used in this SequenceMapView.
     * @return the CloneControl used in this SequenceMapView.
     */
public CloneControl getCloneControl() { return this.cloneControl; }

    /** Search a clone displayed in this SequenceMapView.
    * @param cls the class of the searched AceObject, i.e., clone.
    * @param patter the seached key.
     */
    public void search(String cls, String pattern) {
        if (cls.equalsIgnoreCase("clone") && (cloneControl != null)) {
            cloneControl.doSearch(pattern);
        }
    }

    /** Get the JFrame holding this SequenceMapView.
    * @return the JFrame displaying the SequenceMap.
     */
    public JFrame getViewFrame() {
        return container;
    }
    
    /** A test method.
    */
    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("Usage java jadex.swing.classView.SequenceMapView Sequence-I (or ...)");
            return;
        }
        SequenceMapView view = new SequenceMapView(args[0]);
        // view.setName("sequence_I");
        //view.search("Clone", "C18B3");
    }
}