/*
 * TreeClassView.java
 *
 * Created on July 19, 2000, 8:49 PM
 */

package jadex.swing.classView;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import jade.db.*;
import jadex.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

/**
 * This is a general usable TreeView of the AceObjects.
 * @author  wgm
 * @version 1.0
 */
public class TreeClassView extends ClassView {

    private AceObject localObj = null; // The AceObject to be displayed.
    private JComponent tree = null; // JTree holding displaying the AceObject.
    private boolean inLine = false; // A flag to control if some contents should be in the same line.
    
    /** The TreeCellRenderer used to show the labels of the AceObjedts */
    class AceObjectTreeRenderer implements TreeCellRenderer {
   
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            JLabel label = null;
            //JPanel panel = null;
            if (value instanceof DefaultMutableTreeNode) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
                Object obj = node.getUserObject();
                label = (JLabel) obj;
                //panel = (JPanel) obj;
            }
            // To show the colors of the contructed class nodes
            if (hasFocus) {
                if (label instanceof ObjectLabel)
                label.setForeground(MouseSenser.IN_FOREGROUND_COLOR);
                //panel.setBackground(Color.lightGray);
            }
            //else panel.setBackground(Color.white);
            
            else {
                if (label instanceof ObjectLabel)
                label.setForeground(MouseSenser.OUT_FOREGROUND_COLOR);
            }

            if (label instanceof ObjectLabel) label.setToolTipText("Click to show detailed information");
            
            return label;
            //return panel;
        }
    };

    /** Default constructor */
    public TreeClassView() {
    }

    /** An overloaded constructor.
    * @param o the AceObject to be displayed.
    */
    public TreeClassView(AceObject o) {
        setObject(o);
    }

    /** Set the AceObject to be displayed.
    * @param object the AceObject to be displayed.
    */
    public void setObject(AceObject object) {
        localObj = object;
        tree = getJTreeView();
        setView(tree);
    }

    /** Get the displayed AceObject.
    * @return the AceObject displayed.
    */
    public AceObject getObject() {
        return localObj;
    }

    /** Get the JComponent displaying this AceObject.
    * @return the JComponent displaying this AceObject.
    */
    public JComponent getView() {
        return tree;
    }

    /** Set the displaying JComponent.
    * @param comp the JComponent displaying the AceObject.
    */
    public void setView(JComponent comp) {
        tree = comp;
    }
    
    /** Get a JLabel for the AceObjects in the displayed AceObject.
    */
    private JLabel getObjLabel(AceObject obj, DefaultMutableTreeNode node) {
        inLine = false;
        JLabel label;
        if (obj.isClass()) label = new ObjectLabel(obj);
        else if ((obj.classe().equalsIgnoreCase("int")) || (obj.classe().equalsIgnoreCase("float")) 
            || (obj.classe().equalsIgnoreCase("date"))) {
            label = (JLabel) node.getUserObject();
            if (label instanceof ObjectLabel) {
                inLine = false;
                label = new JLabel(obj.name());
            }
            else {
                inLine = true;
                String text = label.getText();
                text += "   " + obj.name();
                label.setText(text);
            }
        }
        else label = new JLabel(obj.name());

        return label;
    }
    
    /** The method to parse the filled AceObject based on depth-first algorithm. */

    private void appendToTree(AceObject obj, DefaultMutableTreeNode root) {

        DefaultMutableTreeNode node = new DefaultMutableTreeNode(getObjLabel(obj, root));
        if (!inLine) root.add(node);

        if (obj.right() != null) {
            // If float or int or date, add to the same node
            if (!inLine) appendToTree(obj.right(), node);
            // If all other classes, add to the parent node
            else appendToTree(obj.right(), root);
        }
        
        if (obj.down() != null) {
            appendToTree(obj.down(), root);
        }
    }
    
    /** Get the actual JTree displaying the AceObject.
    * @return the JComponent actually displaying the AceObject.
    */
    /** This method is too heavy, and should be refactored. */
    protected JComponent getJTreeView() {

        final JTree tree;

        //DefaultMutableTreeNode root = new DefaultMutableTreeNode(getObjLabel(localObj));
        // It is not necessary to label root.
        DefaultMutableTreeNode root = new DefaultMutableTreeNode(new JLabel(localObj.name()));
        
        if (localObj.right() != null)
        appendToTree(localObj.right(), root);
        //appendToTree(localObj, root);

        tree = new JTree(root);
        tree.setCellRenderer(new AceObjectTreeRenderer());
        tree.setRowHeight(0);
        tree.setEnabled(true);

        //This inner class is used to mediate the mouseevent handling on the JTree. Actually,
        //these behaviours should be handled by the ObjectLabel.
        class MouseActivator extends MouseAdapter implements MouseMotionListener {

        public void mouseDragged(MouseEvent e) {}

            //Used to change the color of the ObjectLabel
            public void mouseMoved(MouseEvent e) {
                localProcessMouseEvent(e);
            }

            public void mousePressed(MouseEvent e) {
                localProcessMouseEvent(e);
            }

            private void localProcessMouseEvent(MouseEvent e) {
                if (e.getSource() == tree) {
                    try {
                        TreePath path = tree.getPathForLocation(e.getX(), e.getY());
                        //To request the focus
                        tree.setSelectionPath(path);
                        Object item = path.getLastPathComponent();
                        if (item instanceof DefaultMutableTreeNode) {
                            DefaultMutableTreeNode node = (DefaultMutableTreeNode) item;
                            Object obj = node.getUserObject();
                            //if (obj instanceof JLabel) {
                              //  JLabel label = (JLabel) obj;
                                if (obj instanceof ObjectLabel) {
                                    ObjectLabel olabel = (ObjectLabel) obj;
                                    olabel.processMouseEvent(e);
                                }
                           // }
                        }
                    }
                catch (Exception ecp) { return; }
                }
            }
        };

        tree.addMouseListener(new MouseActivator());
        tree.addMouseMotionListener(new MouseActivator());
        
        TreeExpansionListener treeExpansion = new TreeExpansionListener() {
            public void treeExpanded(TreeExpansionEvent e) {
                TreePath path = e.getPath();
                int currentRow = tree.getRowForPath(path);
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
                java.util.Enumeration enum = node.children();
                DefaultMutableTreeNode tmpNode = null;
                TreePath tmpPath = null;
                while (enum.hasMoreElements()) {
                    tmpNode = (DefaultMutableTreeNode) enum.nextElement();
                    tmpPath = path.pathByAddingChild(tmpNode);
                    tree.expandPath(tmpPath);
                }
            }
            
            public void treeCollapsed(TreeExpansionEvent e) {
            }
        };
        
        tree.addTreeExpansionListener(treeExpansion);

        return new JScrollPane(tree);
    }
}