/** The popupmenu used in LocusDisplay
 */

package jadex.swing.map;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** This is a JPopupMenu used in LocusDisplay.
 * @author wgm
 * @version 1.0
 */

public class DisplayPopup extends JPopupMenu {
    private DisplayControl control; // control for controling MapPointDisplay
    private MapPointDisplay display; // I hope that all control activies of display can be controlled by control
    // But it seems that it is not possible.

    private JMenuItem more, less, reset; // These munues will be reset
    private JRadioButton columnLayout, smartLayout; // To control the switch between two namelayouts.
    private int count = 0; // for controling the enability of showNumber menues
    private String cls = null; // The class name this popup is used for

    public static final String COLUMN_LAYOUT = "columnLayout";
    public static final String SMART_LAYOUT = "smartLayout";

    /** Constructor by using MapPointDisplay and DisplayControl.
    * @param display the MapPointDisplay this popup will be added to.
    * @param control the DisplayControl this popup interacts with.
    */
    public DisplayPopup(MapPointDisplay display, DisplayControl control) {
        this.display = display;
        this.control = control;
        initialize();
    }

    /** Another constructor.
    * @param display the MapPointDisplay this popup will be added to.
    * @param control the DisplayControl this popup will interacts with.
    * @param cls the Class name this popup is used for.
    */
    public DisplayPopup(MapPointDisplay display, DisplayControl control, String cls) {
        this.display = display;
        this.control = control;
        this.cls = cls;
        initialize();
    }
    
    /** Set the class name this popup used for.
    * @param name the class name.
    */
    public void setClassName(String name) {
        this.cls = name;
    }

    /** Initilize all Menus. */
    private void initialize() {
        setLayoutMenu();
        setLineMenu();
        setShowNumberMenu();
        setZoomMenu();
        setSearchMenu();
    }

    // Set up layout popup menu
    private void setLayoutMenu() {
        JMenu layoutMenu = new JMenu("Set layout");
        columnLayout = new JRadioButton("Column Layout");
        columnLayout.setSelected(true);
        smartLayout = new JRadioButton("Smart Layout");
        ButtonGroup bg = new ButtonGroup();
        bg.add(columnLayout);
        bg.add(smartLayout);

        layoutMenu.setFont(new Font("Dialog", 0, 11));
        columnLayout.setFont(new Font("Dialog", 0, 11));
        smartLayout.setFont(new Font("Dialog", 0, 11));

        ActionListener l1 = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                display.setNameLayout(COLUMN_LAYOUT);
            }
        };

        ActionListener l2 = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                display.setNameLayout(SMART_LAYOUT);
            }
        };

        columnLayout.addActionListener(l1);
        smartLayout.addActionListener(l2);
        layoutMenu.add(columnLayout);
        layoutMenu.add(smartLayout);
        add(layoutMenu);
        add(new JSeparator());
    }

    // Set up line popup menu
    private void setLineMenu() {
        final JMenuItem line = new JMenuItem("Hide Lines");
        line.setFont(new Font("Dialog", 0, 11));
        line.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean cShown = display.isConnectShown();
            if (cShown) { display.setConnectShown(false); line.setText("Show Lines"); }
            else { display.setConnectShown(true); line.setText("Hide Lines"); }
            }
        });
        add(line);
        add(new JSeparator());
    }

    // Set up showing locus number menu
    private void setShowNumberMenu() {
        more = new JMenuItem("Show more...");
        less = new JMenuItem("Show less...");
        reset = new JMenuItem("Reset");

        more.setFont(new Font("Dialog", 0, 11));
        more.setActionCommand("more");
        less.setFont(new Font("Dialog", 0, 11));
        less.setActionCommand("less");
        reset.setFont(new Font("Dialog", 0, 11));
        reset.setActionCommand("reset");

        ActionListener a = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String command = e.getActionCommand();
                if (command.equalsIgnoreCase("more")) {
                    control.showMore();
                    count ++;
                }
                else if (command.equalsIgnoreCase("less")) {
                    control.showLess();
                    count --;
                }
                else if (command.equalsIgnoreCase("reset")) {
                    control.showReset();
                    count = 0;
                }
                checkShowMenu();
            }
        };

        more.addActionListener(a);
        less.addActionListener(a);
        less.setEnabled(false);
        reset.addActionListener(a);
        reset.setEnabled(false);

        add(more);
        add(less);
        add(reset);
        add(new JSeparator());
    }

    /** Checking the enability of show menuitems. */
    public void checkShowMenu() {

        int totalShow = control.getTotalShowNumber();
        int showNumber = control.getShowNumber();
        int number_per_column = control.NUMBER_PER_COLUMN;
        //System.out.println("CheckShowMenu popup: " + totalShow + "  " + showNumber);

        if (showNumber < totalShow) {
            if (showNumber > number_per_column) {
                less.setEnabled(true);
                more.setEnabled(true);
                if (count == 0) reset.setEnabled(false);
                else reset.setEnabled(true);
            }
            else if (showNumber == number_per_column) {
                less.setEnabled(false);
                more.setEnabled(true);
                reset.setEnabled(false);
            }
            else if (showNumber < number_per_column) {
                less.setEnabled(false);
                more.setEnabled(true);
                reset.setEnabled(true);
            }
        }
        else if (totalShow > number_per_column) {
            more.setEnabled(false);
            less.setEnabled(true);
            reset.setEnabled(true);
        }
        else if (totalShow <= number_per_column) {
            more.setEnabled(false);
            less.setEnabled(false);
            reset.setEnabled(false);
        }

        if (showNumber < 10) { // Automatically set the layout as smartlayout
            display.setNameLayout(SMART_LAYOUT);
            smartLayout.doClick(); // Have to fire a click action explicitily
        }
        else { // Else set the layout as column layout
            display.setNameLayout(COLUMN_LAYOUT);
            columnLayout.doClick();
        }
    }

    // Set up zoom menu
    private void setZoomMenu() {
        JMenuItem zoomout = new JMenuItem("Zoom out");
        zoomout.setFont(new Font("Dialog", 0, 11));

        zoomout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                control.reset();
            }
        });

        add(zoomout);
        add(new JSeparator());
    }

    // Set up search menu
    private void setSearchMenu() {
        final String name;
        if (cls != null) name = "Search " + cls;
        else name = "Search";
        JMenuItem search = new JMenuItem(name);
        search.setFont(new Font("Dialog", 0, 11));
        JMenuItem deSearch = new JMenuItem("DeSearch");
        deSearch.setFont(new Font("Dialog", 0, 11));

        search.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String pattern = null;
                try {
                    pattern = JOptionPane.showInputDialog(
                    (JPanel) display, "Search for ... in this range", name,  JOptionPane.INFORMATION_MESSAGE);
                }
                catch(ClassCastException e1) {
                    System.err.println("DisplayPopup.setSearchMenu(): " + e1);
                }
                if ((pattern != null) && (pattern.length() != 0)) control.doSearch(pattern);
            }
        });

        deSearch.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                control.deSearch();
            }
        });

        add(search);
        add(deSearch);
    }
}