/** A little ornament used in the vertival line in LocusDisplay
 */

package jadex.swing.map;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;

/** This Knob is used in the south end in the chromosome line in the LocusDisplay 
 * or SequenceMapDisplay.
 * @author wgm
 * @version 1.0
*/

public class DownKnob extends Knob {

	//If at the Bottom, it should be a circle. Otherwise, it should be an arrow
	private boolean isBottom = true; // default at the Bottom
	// This control is used for moving activities
        
	/** Default constructor. Link mouse activities to this knob in this constructor
	*/
        public DownKnob() {
		super();
	}

        /** An overloaded constructor. Set the DisplayControl for this Knob.
        * @param c the DisplayControl used for this Knob.
        */
	public DownKnob(DisplayControl c) {
		super(c);
	}

	/** Set the direction of this knob.
        * @param bottom true, can be moved down.
        */
	public void setDirection(boolean bottom) {
		if (isBottom == bottom) return;
		isBottom = bottom;
		if (isBottom) setToolTipText("No noving");
		else setToolTipText("Moving Down");
		repaint();
	}

        /** Get the direction of the Knob.
        * @return "Down" for moving down, "No" for no moving down.
        */
	public String getDirection() {
		String rtn = null;
		if (isBottom) rtn = new String("Down");
		else rtn = new String("No");
		return rtn;
	}
	
	/** Override the superclass' method. All paint stuff */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		Graphics2D g2 = (Graphics2D) g;
		Dimension2D d2 = (Dimension2D) getSize();
		float w = (float) d2.getWidth();
		float h = (float) d2.getHeight();

		g2.setPaint(Color.white); // Background
		g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

		if (mouseIn) g2.setPaint(Color.red); // Color for these small ornaments
		else g2.setPaint(Color.gray);

		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));

		if (isBottom)g2.fill(new Ellipse2D.Float(0.0f, 0.0f, w, h));
		else {
			g2.draw(new Line2D.Float(0.0f, h/2.0f, w/2.0f, h));
			g2.draw(new Line2D.Float(w/2.0f, h, w, h/2.0f));
			g2.draw(new Line2D.Float(w/2.0f, h, w/2.0f, 0.0f));
		}
	}

	/** Move down the range. */
	protected void moving() {
		if (isBottom) return;
		else if (control != null) control.movingDown();
	}
}