/** IntervalUI is a user interface to display individual interval.
 */

/** There is a bug in the first release: In Windows platform, a browser
 *   window appear when a popup should popup.
 */

package jadex.swing.map;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import jade.db.*;
import java.awt.image.*;
import java.awt.geom.*;
import jadex.swing.*;
import java.awt.font.*;
import java.applet.*;
import java.net.*;

/**
 * A customized JComponent to display a Interval.
 * @author wgm.
 * @version 1.1.
 */

public class IntervalUI extends JComponent implements MouseListener {

    private float start; // Start value of the range of this IntervalUI
    private float end; // End value of the range of this IntervalUI
    private float w, h; // Width and height of this IntervalUI
    private String name; // Name of this IntervalUI holded interval
    private MapRange range = null; // Maprange of this intervalUI
    private Color background = Color.white; // Background color
    private Color forIn = Color.red; // Foreground color when mouse in
    private Color forOut = Color.black; // Foreground color when mouse out
    private Color foreground = null;
    private FontMetrics fontM = null;
    private LineMetrics lineM = null;
    private Font deFont = null; // Font for the text

    private AceObject interval = null; // Interval is an aceobject

    private MapControl control = null; // MapControl is used to control this whole view display
    private JPopupMenu pop = null; // Popup menu for showing the zoom function

    // The URL address for detailed Interval information from the Wormbase.
    private final String url = "http://www.wormbase.org/perl/ace/elegans/misc/etree?class=Rearrangement&name=";

    /** Default constructor. */
    public IntervalUI() {
    }

    /** Constructor with start, end values and name of interval.
    * @param start the start value of the MapRange for this IntervalUI.
    * @param end the end value of the MapRange for this IntervalUI.
    * @param name the name of this IntervalUI.
    */
    public IntervalUI(float start, float end, String name) {

        setStart(start);
        setEnd(end);
        range = new MapRange(start, end);
        setName(name);
        initialize();
    }
    
    /** Constructor with interval (aceobject) as only argument. 
    * @param o a Interval as a AceObejct to be displayed in this UI.
    */
    public IntervalUI(AceObject o) {
        this.interval = o;
        try {
            range = getRange(o);
            setStart(range.getStartF());
            setEnd(range.getEndF());
        }
        catch (UnknownPositionException e) {
            System.out.println("Position unknown: " + e);
            return ;
        }
        setName(o.name());
        initialize();
    }

    private void initialize() {
        setPopup();
        //setToolTipText("Left mouse click to show the detailed information, right mouse for zooming");
    }
    /** Set aceobject as the interval for this intervalUI. 
    * @param o the AceObject to be displayed in this IntervalUI.
    */
    public void setAceObject(AceObject o) {
        this.interval = o;
    }

    /** Set the MapControl for this IntervalUI.
    * @param c the MapControl for coordinating all zooming activities in the whole MapView.
    */
    public void setMapControl(MapControl c) {
        this.control = c;
    }
    
    /** Set up a JPopup. */
    private void setPopup() {
        pop = new JPopupMenu();
        JMenuItem zoom = new JMenuItem("Zoom in this interval");
        zoom.setFont(new Font("Dialog", 0, 11));
        zoom.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (control != null) {
                    control.setRange(range);
                    setBackground(Color.orange);
                }
            }
        });
        pop.add(zoom);
    }

    /** Get the maprange of this intervalUI */
    private MapRange getRange(AceObject o) throws UnknownPositionException {
        float start, end;
        AceObject left = o.get("Left");
        AceObject right = o.get("Right");
        if ((left == null) || (right == null)) throw new UnknownPositionException();
        start = Float.parseFloat(left.right().name());
        end = Float.parseFloat(right.right().name());
        return new MapRange(start, end);
    }

    /** Set the start value. */
    public void setStart(float start) {
        this.start = start;
    }

    /** Get the start value. */
public float getStart() { return start; }

    /** Set the end value. */
    public void setEnd(float end) {
        this.end = end;
    }

    /** Get the end value. */
public float getEnd() { return end; }

    /** Set the name of this intervalUI. */
    public void setName(String name) {
        this.name = name;
        this.setToolTipText(name + " " + range.toStringF());
        foreground = forOut;
        this.addMouseListener(this);
        deFont = new Font("Monospace", 0, 8);
    }

    /** Get the name of this intervalUI. */
public String getName() { return name; }

    // All these methods for implementation of mouse listener methods
    // There is a problem: How to distinguish the difference between
    // Popup trigger and interval show trigger under different platforms
    // To correct this bug, put the interval show code into mouseClicked
    // method.

    /** For popupmenu showing up.
    */
    public void mousePressed(MouseEvent e) {
        if (e.isPopupTrigger()) { // This way only works in Linux
            pop.show(this, e.getX(), e.getY());
        }
        /*else if (interval != null) { // But not in Windows
        String s = url + name;
        if(!(OpenFrames.getInstance().show(s, name)))
        OpenFrames.getInstance().show(interval);
        }*/
    }


    /** For detailed information display.
    */
    public void mouseClicked(MouseEvent e) {
        if (interval != null) { // But not in Windows
            String s = url + name;
            if(!(OpenFrames.getInstance().show(s, name)))
            OpenFrames.getInstance().show(interval);
        }
    }

    /** For popupmenu showing up.
    */
    public void mouseReleased(MouseEvent e) {
        if (e.isPopupTrigger()) {
            pop.show(this, e.getX(), e.getY()); // Windows use MouseReleased to Pop up popup menu
        }
    }

    /** For color change. When mouse in, show in red. */
    public void mouseEntered(MouseEvent e) {
        foreground = forIn;
        repaint();
    }

    // For color chnage. When mouse out, show in black. */
    public void mouseExited(MouseEvent e) {
        foreground = forOut;
        repaint();
    }

    /** Check if two intervals intersect. 
    * @param iu another UI to be checked.
    * @return true for intersecting.
    */
    public boolean intersect(IntervalUI iu) {
        float start1 = iu.getStart();
        float end1 = iu.getEnd();
        if ((end <= start1) || (start >= end1))  return false;
        else return true;
    }

    // All paint stuffs are here
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        removeAll();

        Graphics2D g2d = (Graphics2D) g;
        Dimension d = getSize();
        w = (float) d.width;
        h = (float) d.height;

        g2d.setColor(background);
        g2d.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

        g2d.setColor(foreground);
        g2d.draw(new Line2D.Float(w/2.0f, 5.0f, w/2.0f, h - 5.0f));
        g2d.draw(new Line2D.Float(w/2.0f-5.0f, 5.0f, w/2.0f+5.0f, 5.0f));
        g2d.draw(new Line2D.Float(w/2.0f-5.0f, h - 5.0f, w/2.0f+5.0f, h - 5.0f));

        //Show the name of this interval
        g2d.setFont(deFont);
        //fontM = g2d.getFontMetrics();
        //int length = fontM.stringWidth(name);
        //g2d.drawString(name, (w-length)/2.0f, h/2.0f);
        drawVerticalString(name, g2d, w/2.0f + 2.0f, 7.0f);
    }

    // Draw the text label for this interval
    private void drawVerticalString(String s, Graphics2D g2d, float x, float y) {
        char[] sarray = s.toCharArray();
        float fy = y;
        for (int i = 0; i < sarray.length; i++) {
        char[] c = {sarray[i]};
            lineM = deFont.getLineMetrics(c, 0, 1, g2d.getFontRenderContext());
            float h = lineM.getHeight();
            fy += h;
            g2d.drawString(sarray[i]+"", x, fy);
        }
    }

    /** Set the background color. 
    * @param b the new background.
    */
    public void setBackground(Color b) {
        if (b.equals(background)) return;
        background = b;
        repaint();
    }

    /** Reset the background to black. */
    public void resetBackground() {
        setBackground(Color.white);
    }
}