/** A little ornament used in the vertival line in LocusDisplay
 */

package jadex.swing.map;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;

/** A little ornamenet used in the vertical line for chromosome in LocusDisplay or CloneDisplay.
 * @author wgm
 * @version 1.0
 */

public abstract class Knob extends JPanel implements MouseListener {

    // Fix the size of knob to (10, 10)
    private final Dimension size = new Dimension(10, 10);

    // The displaycontrol for moving.
    protected DisplayControl control = null;

    protected boolean mouseIn = false; // Detects if mouse is in

    /** Default constructor. Link mouse activities to this knob in this constructor. */
    public Knob() {
        addMouseListener(this);
    }

    /** Overloaded constructor.
    * @param c the displaycontrol used in this Knob.
     */
    public Knob(DisplayControl c) {
        this.control = c;
        addMouseListener(this);
    }

    /** Set up a DisplayControl for this Knob.
     * @param c the DisplayControl to be used in this Knob.
    */
    public void setDisplayControl(DisplayControl c) {
        this.control = c;
    }

    /** Get the size of this Knob.
    */
    public Dimension getPreferredSize() {
        return size;
    }

    /** Set the dircetion of this knob. 
    * @param is true for a direction, false for no moving.
    * @see DownKnob
    * @see UpKnob
    */
    public abstract void setDirection(boolean is);
    
    /** The name of direction: up, down or no.
    */
    public abstract String getDirection();

    // All mouse activities
    public void mouseEntered(MouseEvent e) {
        mouseIn = true;
        repaint();
    }

    public void mouseExited(MouseEvent e) {
        mouseIn = false;
        repaint();
    }

    public void mousePressed(MouseEvent e) {
        moving();
    }

    protected abstract void moving(); // Subclass should implement this

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }
}