/*
 * LineMouse.java: Lighten up label and line when mouse is in.
 *
 * Created on October 12, 2000, 12:41 PM
 */

package jadex.swing.map;

import java.awt.event.*;

/**
 * Lighten up label and line when mouse is in. This class is used together with 
 * MapPointDisplay for label and line lightening up effect.
 * @author  wgm
 * @version 1.0
 */


public class LineMouse extends MouseAdapter {
    private LocusUI ui = null; // The mouse detector.
    private MapPointDisplay display = null; // The parenet Component.
    private String name = null; // The name of the LocusUI.

    /** The sole construcotr.
    * @param ui the LocusUI which Mouse will interact.
    * @param display the paraent of the ui.
     */
    public LineMouse(LocusUI ui, MapPointDisplay display) {
        this.ui = ui;
        this.display = display;
    }

    /** When Mouse eneters, set up the name of the LocusUI.
    */
    public void mouseEntered(MouseEvent e) {
        name = ui.getName();
        display.setMouseInName(name);
        try {
            display.getNameLayout().setMouseInName(name);
        }
        catch(NullPointerException e1) {
            System.err.println("LineMouse.mouseEntered(): " + e1);
        }
    }
    
    /** When Mouse exits, remove the selected name of the LocusUI.
    */
    public void mouseExited(MouseEvent e) {
        name = null;
        display.setMouseInName(null);
        try {
            display.getNameLayout().setMouseInName(name);
        }
        catch(NullPointerException e1) {
            System.err.println("LineMouse.mouseEntered(): " + e1);
        }
    }
}