/*
 * Locus.java
 *
 * Created on July 13, 2000, 11:37 AM
 */

package jadex.swing.map;

import jade.db.*;

/**
 * An implementataion of MapPoint. This class holds information on a Locus from AceDB.
 * In fact, this is a wrapper for the AceObject for the Locus.
 * @author  wgm
 * @version 1.0
 */
public class Locus implements MapPoint {

    private String name = null; // The name of this Locus.
    private float position = Float.MAX_VALUE; // The position of this Locus.
    private float error = Float.MAX_VALUE; // The error of the position.
    private AceObject locus = null; // The AceObject holding all information for this Locus.


    /** Default constructor. */
    public Locus() {
    }

    /** An overloaded construtor.
    * @param name the name of the Locus.
    * @param pos the posistion of this Locus.
     */
    public Locus(String name, float pos) {
        setName(name);
        setPosition(pos);
    }

    /** An overloaded constructor.
    * @param name the name of the Locus.
    * @param pos the position of this Locus.
    * @param o the AceObejct holding all information for this Locus.
    */
    public Locus(String name, float pos, AceObject o) {
        setName(name);
        setPosition(pos);
        if (o != null) setAceObject(o);
    }
    
    /** An overloaded constructor.
    * @param name the name of this locus.
    * @param pos the position of this locus.
    * @param err the error of the position of this locus.
    * @param o the AceObejct.
    */
    public Locus(String name, float pos, float err, AceObject o) {
        setName(name);
        setPosition(pos);
        setError(err);
        setAceObject(o);
    }

    /** An overloaded constructor.
    * @param o the AceObject holding all information for this locus.
    */
    public Locus(AceObject o) {
        setAceObject(o);
    }

    /** Set up the AceObject for this Locus.
    * @param o the AceObject holding all information for this Locus.
    */
    public void setAceObject(AceObject o) {
        this.locus = o;
    }

    /** Get the AceObject.
    */
public AceObject getAceObject() { return locus; }

    /** Get the position of this Locus.
    * @return the position of this Locus.
    * @exception throw an exception if the position has not be set.
    */
    public float getPosition() throws UnknownPositionException {
        if (position == Float.MAX_VALUE) {
            AceObject o = locus.get("Position");
            if (o == null) throw new UnknownPositionException(locus.name + " position unknown");
            float rtn = Float.parseFloat(o.name());
            return rtn;
        }
        else return position;
    }

    /** Get the name of this Locus.
    * @return the name of this Locus.
    */
    public String getName() {
        if (name != null) return name;
        else return locus.name();
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setPosition(float position) {
        this.position = position;
    }

    public void setError(float err) {
        this.error = err;
    }

public float getError() { return this.error; }
}