/** This is a control class for LocusDisplay. All control activities will
 *  be placed here
*/

package jadex.swing.map;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.util.*;

/** This is a control class for LocusDisplay. All control activities will be placed here. 
 * This class can listen to "zoom" property change event.
 * @author wgm
 * @version 1.0
 */
public class LocusControl extends DisplayControl {
	
	private MapControl control; // The control for the whole MapView
	private LocusDisplay display; // The display this Control controls
	//private MapPoint[] sPoints; // Showing MapPoints in this MapRange
	//private LocusUI[] sLoci; // Loci will be shown in display
	
	// Searching pattern
	//private String pattern = null;
	//private String[] patterns = null; // for muli-search
	
	/** Default constructor. */
	public LocusControl() {}
	
	/** Overloaded constructor.
        * @param m the DisplayModel holding all data to be displayed.
        */
	public LocusControl(DisplayModel m) {
		setModel(m);
	}
	
	/** Overloaded constructor. 
        * @param d the LocusDisplay this control controls.
        */
	public LocusControl(LocusDisplay d) {
		setDisplay(d);
	}
	
	/** overloaded constructor.
        * @param m the DisplayModel holding all data.
        * @param d the LocusDisplay this control controls.
        */
	public LocusControl(DisplayModel m,LocusDisplay d) {
		setModel(m);
		setDisplay(d);
	}

        /** Set the LocusDisplay.
        * @param d the LocusDisplay this control controls.
        */
	public void setDisplay(LocusDisplay d) {
		this.display = d;
	}
	
        /** Get the display. 
        * @return the MapPointDisplay, which is the superclass of LocusDisplay.
        */
	public MapPointDisplay getDisplay() { return display; }
	
	/** Show the data in LocusDisplay: points, little bars, and default names
	* This is the central points controled by this LocusControl. All changes will be
	* through this method.
        */
	public void updateUI() {
		// If there are data can be shown and display can be updated
		if (display != null) {
			display.setLocusUI(sLoci);
			//System.out.println("sLoci.length: " + sLoci.length);
			display.setTotalLocus(tPoints);
			//System.out.println("tPoint.length: " + tPoints.length);
			display.repaint();
			if (display.getPopup() != null) display.getPopup().checkShowMenu();
		}
	}
	
        /** Reset UI whenever the MapRange changed.
        */
	protected void resetUI() {
		int size = showNumber > tPoints.length ? tPoints.length : showNumber;
		sPoints = new MapPoint[size];
		sLoci = new LocusUI[size];
		for (int i = 0; i < size; i++) {
			sPoints[i] = tPoints[i];
			sLoci[i] = new LocusUI(tPoints[i]);
		}                
		if (pattern != null) doSearch(pattern);
		else updateUI();
	}
	
	/** The number of shown LocusUI.*/
	public int getShowNumber() {
		if (sLoci != null) return sLoci.length;
		else return 0; 
	}
}