/** Layout for distribute the loci on the locusdisplay
 *  This class temporary can only be used in this package
 */

package jadex.swing.map;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

/** 
* Layout for distribute the loci on the locusdisplay. 
* @author wgm
* @version 1.0
*/

public class LocusLayout {
	
	// All these variables should be initialized on the constructor
	float yPadding; // vertival padding
	int NUMBER_PER_COLUMN; // the number of names for each column.
	boolean cShown; // a flag to control if connecting lines should be displayed.
	Color cColor; // color of the connecting lines.
	
	// Properties set by set methods
	LocusUI[] lUI = null; // Locus ui Data.
	String mouseInName = null; // the name of the Locus where mouse is in.
	
	// A comparator for sorting LocusUI based on their positions.
	java.util.Comparator comp = new java.util.Comparator() {
			public int compare(Object o1, Object o2) {
				LocusUI p1 = (LocusUI) o1;
				LocusUI p2 = (LocusUI) o2;
				String name1 = p1.getName();
				String name2 = p2.getName();
				Float pos1 = new Float(p1.getPosition());
				Float pos2 = new Float(p2.getPosition());
				int cv = pos2.compareTo(pos1);
				return ((cv != 0) ? cv : name2.compareTo(name1));
			}
			public boolean equals(Object o1, Object o2) {
				MapPoint p1 = (MapPoint) o1;
				MapPoint p2 = (MapPoint) o2;
				String name1 = p1.getName();
				String name2 = p2.getName();
				return name1.equalsIgnoreCase(name2);
			}	
	};
	
        /** The only constructor.
        * @param yPadding set the vertical padding.
        * @param npc the number per column for displaying.
        * @param connect the flag for connecting lines.
        * @param the color for the connecting lines.
        */
	public LocusLayout(float yPadding, int npc, boolean connect, Color c) {
		this.yPadding = yPadding;
		this.NUMBER_PER_COLUMN = npc;
		this.cShown = connect;
		this.cColor = c;
	}
	
	/** Set the showing loci. 
        * @param lUI the LocusUI to be displayed.
        */
	public void setLocusUI(LocusUI[] lUI) {
		this.lUI = lUI;
		doSorting();
	}
	// Template method (one of design patterns)
	private void doSorting() {
		sortOnColumn();
	}
	
	// Sort all LocusUI on each column by their positions.
	private void sortOnColumn() {
		
		int total = lUI.length;
		int column = (int)((float) total / NUMBER_PER_COLUMN + 0.95f);
		java.util.List l = Arrays.asList(lUI);
		
		int start = 0;
		for (int i = 1; i <= column; i ++) {
			int end = start + NUMBER_PER_COLUMN;
			if (end > total) end = total;
			java.util.List l1 = l.subList(start, end);
			Collections.sort(l1, comp);
			start = end;
		}
	}

	/** Set the name of the LocusUI where mouse is in.
        * @param name the name of the LocusUI where mouse is in.
        */
	public void setMouseInName(String name) {
		this.mouseInName = name;
	}
	
	/** Set if the connecting lines should be displayed.
         * @param c true for displaying the connecting lines, false for no displaying.
        */
	public void setConnect(boolean c) {
		this.cShown = c;
	}
	
	// All detailed layout activitise are here.
        /** The actual layout method.
        * @param x1 the initial x position
        * @param h the height of the display.
        * @param offfest the offset of the x coordinate.
        * @param rel the ratio between genetic distance and graphic coordinate.
        * @param g2 the Graphics context.
        */
	public void drawNames(float x1, float h, float offset, float rel, Graphics2D g2) {
		
		if (lUI == null) return;
		
		int total = lUI.length;
		float x = x1 + 25.0f;
		
		//g2.setColor(cColor);
		float y = yPadding - 2.0f;
		float diff = (h - 2 * yPadding) / (NUMBER_PER_COLUMN - 1);

		int count = 0;
		
		int column = (int)((float) total / NUMBER_PER_COLUMN + 0.95f);
		for (int j = 0; j < column; j++) {
			for (int i = 0; i < NUMBER_PER_COLUMN; i++) {
				if (count == total) return ;
				lUI[count].setBounds((int) x, (int) y, 40, 10);
				if (cShown) { 
					float pos = lUI[count].getPosition();
					float startY = (offset - pos) * rel + yPadding;
					if ((mouseInName != null) && (lUI[count].getName().equals(mouseInName)))
						g2.setColor(Color.red);
					else g2.setColor(cColor);
					g2.draw(new Line2D.Float(x1, startY, x, y + 5.0f));
				}
				y += diff;
				count ++;
			}
			x = x + 40.0f;
			y = yPadding - 2.0f;
		}
	}
	
        /** Get the name of this layout.
        * @return always columnLayout.
        */
	public String getName() { return new String("columnLayout"); }
}