package jadex.swing.map;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import jadex.swing.*;
import java.applet.*; // Used to display the locus in the browser.
import java.net.*;

import jade.db.*;

/** This customized JLabel is for displaying Locus or Clone information. 
* @author wgm
* @version 1.0
 */
public class LocusUI extends JLabel implements MouseListener {
    private AceObject locus = null; // The displayed Locus
    private AceAccessor accessor = null; // This AceAccessor is used to fetch AceObject from AceDB for mouse event.
    private boolean isClone = false; // Check if this AceObject is a clone. 
    private float position; // The position of the Locus.
    private float error = Float.MAX_VALUE;
    //private float w, h;
    private String name; // The name of this Locus.
    private boolean showName = false; // a flag to control if the name should be displayed.
    private Color background = Color.white;
    private Color foreIn = Color.red; // Foreground color when mouse is in.
    private Color foreOut = Color.black; // Foreground color when mouse is out.
    private Color foreground = null; // Forground color.
    private boolean debug = false;

    // The url for the detailed information for Loci in the Wormbase.
    private String url = "http://www.wormbase.org/perl/ace/elegans/gene/locus?name=";

    /** Default constructor. */
public LocusUI() {super();}

    /** A overloaded constructor.
    * @param name the name of the Locus.
    * @param position the position of the Locus.
    */
    public LocusUI(String name,float position) {
        super();
        initialize(name, position);
    }

    /** A overloaded constructor.
    * @param p the MapPoint to be displayed in this LocusUI.
    */
    public LocusUI(MapPoint p) {
        try {
            String name = p.getName();
            float pos = p.getPosition();
            error = p.getError();
            initialize(name, pos);
        }
    catch (UnknownPositionException e) { System.err.println(e); }
        if (p instanceof Locus) {
            Locus l = (Locus) p;
            AceObject o = l.getAceObject();
            if (o != null) setAceObject(l.getAceObject());
        }
    }

    // Initilization method.
    private void initialize(String name, float position) {
        addMouseListener(this);
        this.position = position;
        foreground = foreOut;
        super.setForeground(foreground);
        super.setFont(new Font("Dialog", 0, 9));
        setName(name);
    }

    /** Set the AceObject for this LocusUI. If there is an AceObject set,
    * the detailed information for Locus or Clone can be displayed.
    * @param locus the AceObject displayed in this Locus.
    */
    public void setAceObject(AceObject locus) {
        this.locus = locus;
    }

    /** Get the AceObject attached to this LocusUI.
    * @return the AceObject attached to this LocusUI.
    */
    public AceObject getAceObject() {
        return this.locus;
    }

    /** Set the foreground color when mouse is out.
    * @param c the color.
    */
    public void setForeOutColor(Color c) {
        if (this.foreOut == c) return;
        this.foreOut = c;
        setForeground(c);
    }

    /** Get the foreground color used when mouse is not in.
    */
public Color getForOutColor() { return foreOut; }

    /** Set the position of the Locus.
    * @param position the position of the Locus.
    */
    public void setPosition(float position) {
        this.position = position;
    }

    /** Get the position of the Locus.
    * @return the position of the Locus.
    */
    public float getPosition() {
        return position;
    }

    /** Set the name for this display.
    * @param name the Name for this display.
    */
    public void setName(String name) {
        this.name = name;
        super.setText(name);
        if (error == Float.MAX_VALUE) this.setToolTipText(name + " " + getPosition());
        else this.setToolTipText(name + " " + getPosition() + " " + error);
    }

    /** Get the name for this display.
    * @return the name attached for this display.
    */
public String getName() { return name; }

    /** Set URL for Locus or Clone information.
    * @param url the name of the url linking to the detailed information.
    */
public void setURL(String url) { this.url = url; }

    /** Get the name of the url linking to the detailed information.
    * @return the name of the url linking to the detailed information.
    */
public String getURL() { return this.url; }

    /** Linking to the detailed information by using this method.
    */
    public void mousePressed(MouseEvent e) {
        // Try to create a AceObject by this LocusUI itself.
        if (locus == null) {
            if (accessor != null) {
                // There are two MapPoint classes now: Locus and Clone.
                if (isClone) {
                    locus = accessor.fetch("Clone", getName());
                }
                else locus = accessor.fetch("Locus", getName());
            }
        }
        if (locus != null) {
            String s = url + name;
            if (!(OpenFrames.getInstance().show(s, name)))
            OpenFrames.getInstance().show(locus);
        }
        //System.out.println(getName() + " clicked.");
    }

    /** Do nothing. */
public void mouseReleased(MouseEvent e) {}
    /** Do nothing. */
public void mouseClicked(MouseEvent e) {}

    /** Change color when mouse enters. */
    public void mouseEntered(MouseEvent e) {
        foreground = foreIn;
        super.setForeground(foreIn);
        //validate();
    }
    
    /** Change color when mouse exits. */
    public void mouseExited(MouseEvent e) {
        foreground = foreOut;
        super.setForeground(foreOut);
        //validate();
    }

    /** Set the AceAccessor for fetching AceObject directly from the LocusUI.
     * @param acc the AceAccessor.
     */
    public void setAceAccessor(AceAccessor acc) {
        this.accessor = acc;
    }

    /** Get the AceAccessor.
     * @return the AceAccessor for this LocusUI.
     */
    public AceAccessor getAceAccessor() {
        return this.accessor;
    }

    /** Set this LocusUI as a CloneUI
     */
    public void setIsClone(boolean isClone) {
        this.isClone = isClone;
    }

    /** Check if this UI for CloneUI.
     */
    public boolean isClone() {
        return this.isClone;
    }
}