/*
 * MapCartoon.java
 *
 * Created on July 11, 2000, 4:43 PM
 */

package jadex.swing.map;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.beans.*;

import jade.db.*;
/**
 * This customized JPanel is used to show the displayed region on the chromosome.
 * It is also used as a zooming tool.
 * @author  wgm
 * @version 1.0
 */
public class MapCartoon extends JPanel implements MouseListener, MouseMotionListener, PropertyChangeListener {

    private final float WIDTH = 10.0f; // The width of the cartoon
    private float inX, inY, outX, outY; // Absolute coordinates
    private float relInX, relInY, relOutX, relOutY; // Relative coordinates to the JPanel
    private float alpha = 1.0f;
    private AlphaComposite composite = null;
    private int rule = AlphaComposite.SRC_IN;
    private boolean start = true;
    private boolean dragging = false;
    private boolean inScale = false;
    private Shape drawing = null;
    private Shape cartoon = null;
    private boolean top = false;
    private boolean bottom = false;
    private boolean popup = false;
    private int popY;
    private float w, h; //Width and legth of this panel
    private AceObject[] mainLoci = null; // Marker points for the map
    private float paddingY = 2.0f;

    private long TOTAL_LENGTH_LONG = -1;
    private float TOTAL_LENGTH_FLOAT = Float.POSITIVE_INFINITY;

    private MapControl control = null; // The central control

    private boolean debug = false;

    /** Holds value of property range. */
    private MapRange range = new MapRange(0.0f, 0.0f);
    private MapRange map = null; // Total scale
    /** Creates new MapCartoon */
    public MapCartoon() {
        this.addMouseListener(this);
        this.addMouseMotionListener(this);
        this.setToolTipText("Draw a rectangle around the bar for zooming.");
    }

    /** An overloaded constructor. This constructor can be used for clone or sequence maps.
    * @param length the total length of the chromosome.
    */
    public MapCartoon(long length) {
    this();
    TOTAL_LENGTH_LONG = length;
    }

    /** An overloaded constructor. This constructor is for the Genetic_map.
    * @param lengthF the total length of the chromosome in float unit.
    */
    public MapCartoon(float lengthF) {
    this();
    TOTAL_LENGTH_FLOAT = lengthF;
    }

    /** An overloaded constructor.
    * @param scale the MapRange of the chromosome.
    */
    public MapCartoon(MapRange scale) {
    this();
    setMap(scale);
    }

    /** Set the MapControl as the central control to coordinate all zooming activites.
    * @param c the MapControl for coordination.
    */
    public void setMapControl(MapControl c) { this.control = c; }

    /** An initilization method. Set the initial maprange.
    * @param r the MapRange to be displayed. If this is the first time, the total MapRange
    * is also set up.
    */
    public void setMap(MapRange r) {
    this.map = r;
    this.range = r;
    try {TOTAL_LENGTH_FLOAT = map.getExtendF(); }
    catch(UnknownPositionException e) {
    System.err.println("MapCartoon.setMap(): " + e);
    }
    }

    /** Get the total MaRange for the chromosome.
    * @return the total maprange.
    */
    public MapRange getMap() { return map; }

    /** Set the total length of the chromosome this cartoon represents.
    * @param length the total length of the chromosome.
    */
    public void setTotalLengthL(long length) {
    this.TOTAL_LENGTH_LONG = length;
    }

    /** Set the total length of the chromosome in float. This method is for the Genetic_maps.
    * @param lengthF the total length of the chromosome in float.
    */
    public void setTotalLengthF(float lengthF) {
    this.TOTAL_LENGTH_FLOAT = lengthF;
    }

    /** Set the marks.
    * @param loci the data for the markers.
    * @deprecated do not use this method. See {@link LocusPanel LocusPanel}.
    */
    public void setMainLoci(AceObject[] loci) {
    this.mainLoci = loci;
    }

    /** The most important method. */
    public void paintComponent(Graphics g) {

    super.paintComponent(g);
    Graphics2D g2 = (Graphics2D) g;

    Dimension2D d = (Dimension2D) getSize();
    w = (float) d.getWidth();
    h = (float) d.getHeight();

    //Created the buffered image.
    BufferedImage buffImg = new BufferedImage((int) w, (int) h, BufferedImage.TYPE_INT_ARGB);
    Graphics2D gbi = buffImg.createGraphics();

    //Clears the previously drawn image.
    g2.setPaint(Color.white);
    g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

    float rectX = (w - WIDTH)/2.0f;
    float rectY = paddingY;

    gbi.setColor(Color.black);
    cartoon = new RoundRectangle2D.Float(rectX, rectY, WIDTH, h-2.0f * paddingY, 5.0f, 5.0f);
    gbi.fill(cartoon);

    if (!start) {
    composite = AlphaComposite.getInstance(rule, alpha);
    gbi.setColor(Color.green);
    gbi.setComposite(composite);
    inX = relInX * w; outX = relOutX * w;
    inY = relInY * h; outY = relOutY * h;
    //System.out.println("inX: " + inX + " inY " + inY + " outX " + outX + " outY " + outY);
    float hD = (outY - inY) > 1.0f ? (outY - inY) : 1.0f;
    drawing = new Rectangle2D.Float(inX, inY, outX - inX, hD);
    gbi.fill(drawing);
    setRange();
    }

    g2.drawImage(buffImg, null, 0, 0);
    }

    //private int count = 0;

    /** Set the displayed MapRange.
    */
    private void setRange() {
    float ratio = TOTAL_LENGTH_FLOAT / (h - 2 * paddingY);
    float end = 0.0f;
    float start = 0.0f;
    try {
    end = map.getEndF() - (inY - paddingY) * ratio;
    start = map.getEndF() - (outY - paddingY) * ratio;
    }
    catch (UnknownPositionException e) {}
    range = new MapRange(start, end);
    if (control != null) control.setRange(range);
    }

    /** Set the displayed MapRange. Clients should call this method.
    * @param r the MapRange to be displayed.
    */
    public void setMapRange(MapRange r) {
    if (range.equalsF(r)) return;
    this.range = r;
    if (start) start = false;
    //setRange(r);
    try {
    float rel = (h - 2.0f * paddingY) / map.getExtendF();
    float offset = map.getEndF();

    float height = rel * r.getExtendF();
    float startY = (offset - r.getEndF()) * rel + paddingY;
    float endY = startY + height;
    relInY = startY / h;
    relOutY = endY / h;
    relInX = (w-WIDTH-5.0f)/(2.0f*w);
    relOutX = (w+WIDTH+5.0f)/(2.0f*w);
    repaint();
    }
    catch (UnknownPositionException e) { System.err.println(e); }
    }
    
    /** End the zooming.
    */
    public void mouseReleased(final java.awt.event.MouseEvent p1) {
    if (p1.isPopupTrigger() && !popup) {
    popY = p1.getY();
    popup = true;
    return ;
    }
    if (dragging) {
    rule = AlphaComposite.SRC_IN;
    alpha = 1.0f;
    repaint();
    dragging = false;
    inScale = false;
    top = false;
    bottom = false;
    popup = false;
    return;
    }
    else if ((drawing != null) && inScale) {
    alpha = 1.0f;
    rule = AlphaComposite.SRC_IN;
    repaint();
    return ;
    }
    }

    /** Do nothing here. */
    public void mouseClicked(final java.awt.event.MouseEvent p1) {
    }
    
    /** Do nothing here. */
    public void mouseEntered(final java.awt.event.MouseEvent p1) {
    }

    /** Start up the zooming. */
    public void mousePressed(final java.awt.event.MouseEvent p1) {
    if (p1.isPopupTrigger() && !popup) {
    popY = p1.getY();
    if (debug) System.out.println("Popup on");
    popup = true;
    }
    if ((drawing!=null) && drawing.contains(p1.getX(), p1.getY())) {
    inScale = true;
    alpha = 0.50f;
    rule = AlphaComposite.SRC;
    repaint();
    return;
    }
    start = false;
    inX = p1.getX();
    relInX = (float) inX / w;
    inY = p1.getY();
    relInY = (float) inY / h;
    }

    /** Do nothing. */
    public void mouseExited(final java.awt.event.MouseEvent p1) {
    }

    /** Zooming by creating rectangels.
    */
    public void mouseDragged(final java.awt.event.MouseEvent p1) {
    dragging = true;
    if (inScale) {
    if (Math.abs(p1.getY()-inY) < 2.0f) {top = true; bottom = false; return ;}
    if (Math.abs(p1.getY()-outY) < 2.0f) { top = false; bottom = true; return ; }
    if (popup) {
    inY = inY + (p1.getY() - popY);
    relInY = (float) inY / h;
    outY = outY + (p1.getY() - popY);
    relOutY = (float) outY / h;
    popY = p1.getY();
    }
    else if (top) {
    inY = p1.getY();
    relInY = (float) inY / h;
    }
    else if (bottom) {
    outY = p1.getY();
    relOutY = (float) outY / h;
    }
    }
    else {
    outX = p1.getX();
    relOutX = (float) outX / w;
    outY = p1.getY();
    relOutY = (float) outY / h;
    }
    alpha = 1.0f;
    rule = AlphaComposite.SRC_OUT;
    repaint();
    //setRange();
    return;
    }

    /** Do nothing. */
    public void mouseMoved(final java.awt.event.MouseEvent p1) {
    }

    /** Getter for property range.
     * @return Value of property range.
     */
    public MapRange getRange() {
        return range;
    }

    public static void main(String[] args) {
        JFrame frame = new JFrame("MapCartoon Test");
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        MapCartoon panel = new MapCartoon(new MapRange(-30.0f, 30.0f));
        PropertyChangeListener l = new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                if (e.getPropertyName().equals("range")) {
                    MapRange r = (MapRange) e.getNewValue();
                    System.out.println(r.toStringF());
                }
            }
        };
        panel.addPropertyChangeListener(l);
        frame.getContentPane().add(panel, "Center");
        frame.setSize(200, 400);
        //frame.setResizable(false);
        frame.setVisible(true);
        panel.setMapRange(new MapRange(-3.5f, 20.0f));
    }

    /** Listening to the Range property change.
    */
    public void propertyChange(final java.beans.PropertyChangeEvent p1) {

        if (p1.getPropertyName().equalsIgnoreCase("range")) {
            MapRange r = (MapRange) p1.getNewValue();
        if (r == range) {  return; } // The maprange coming from here, so do nothing
            setMapRange(r);
        }
    }
}