/*
 * MapControl.java: The total control of the mapview
 *
 * Created on September 19, 2000, 3:20 PM
 */

package jadex.swing.map;

import java.beans.*;

/**
 * This is a invisible mediator to coordinate all zooming tools in a MapView or SequenceMapView.
 * @author  wgm
 * @version 1.0
 */
public class MapControl extends Object {


    /** Holds value of property range. */
    private MapRange range;
    private MapRange map; // Total maprange of the chromosome
    /** Utility field used by bound properties. */
    private java.beans.PropertyChangeSupport propertyChangeSupport = new java.beans.PropertyChangeSupport (this);
    /** Creates new MapControl */
    public MapControl() {
    }

    /** An overloaded constructor.
    * @param r the MapRange.
    */
    public MapControl(MapRange r) {
        map = r;
        range = r;
    }

    /** Set the total MapRange to be displayed.
    * @param map the Total MapRange.
    */
    public void setMap(MapRange map) {
        this.map = map;
        if (range == null) range = map;
    }

    /** Get the total MapRange.
    * @return the total MapRange.
    */
public MapRange getMap() { return map; }
    /** Add a PropertyChangeListener to the listener list.
     * @param l The listener to add.
     */
    public void addPropertyChangeListener(java.beans.PropertyChangeListener l) {
        propertyChangeSupport.addPropertyChangeListener (l);
    }
    /** Removes a PropertyChangeListener from the listener list.
     * @param l The listener to remove.
     */
    public void removePropertyChangeListener(java.beans.PropertyChangeListener l) {
        propertyChangeSupport.removePropertyChangeListener (l);
    }
    /** Getter for property range.
     * @return Value of property range.
     */
    public MapRange getRange() {
        return range;
    }

    /** Reset to the full MapRange.
    */
    public void reset() {
        setRange(map);
    }

    /** Moving down the maprange. */
    public void movingUp() {
        MapRange r = range;
        try {
            float s = r.getStartF();
            float e = r.getEndF();
            float step = r.getExtendF() / 3.0f;
            MapRange rNew = new MapRange(s + step, e + step);
            setRange(rNew);
        }
    catch (UnknownPositionException e) { System.err.println(e); }
    }

    /** Moving up the maprange. */
    public void movingDown() {
        MapRange r = range;
        try {
            float s = r.getStartF();
            float e = r.getEndF();
            float step = r.getExtendF() / 3.0f;
            MapRange rNew = new MapRange(s - step, e - step);
            setRange(rNew);
        }
    catch (UnknownPositionException e) { System.err.println(e); }
    }

    /** Setter for property range.
     * @param range New value of property range.
     */
    public void setRange(MapRange range) {
        MapRange oldRange = this.range;
        this.range = range;
        if ((oldRange != null) && (oldRange.equalsF(range))) return;
        else {
            propertyChangeSupport.firePropertyChange ("range", oldRange, range);
        }
    }
}