/*
 * MapMenuBar.java
 *
 * Created on July 11, 2000, 2:13 PM
 */

package jadex.swing.map;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import jadex.swing.classView.*;
import jadeSocket.*;
import jade.db.*;
import jadex.swing.*;
import java.beans.*;

/**
 * A customized JMenuBar used in a MapView.
 * @author  wgm
 * @version 1.0
 */
public class MapMenuBar extends javax.swing.JMenuBar {

    private JInternalFrame frame = null; // A JIntervalFrame holding a MapView.
    private JFrame f = null; // A JFrame holding a MapView
    private Font dFont = null; // Default font.
    private MapView view = null; // A MapView this JMenuBar will interacts with.
    private AceAccessor accessor = null; // For accessing the AceDB.
    private ConnectPanel connectP = null; // For get the host name and port number for the AceDB.
    // Control if these panels displayed or not.
    private JCheckBox mk, locator, loci, it, scaleLeft, scaleRight;
    private JMenu viewMenu = null; 
    private JMenu helpMenu = null; 
    private JMenu file = null; // File menu.

    private boolean isApplet = false; // Is this menubar in an applet.

    // For get information from the connectPanel.
    private PropertyChangeListener connectProp = new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent e) {
            if (e.getPropertyName().equalsIgnoreCase("buttonClicked")) {
                String host = connectP.getHost();
                int port = connectP.getPort();
                if ((host != null) && (port != 0)) {
                    AceAccessor accessor = new JSAceAccessor(host, port);
                    AceObject[] o = accessor.find("genetic_map", "*");
                    JPanel panel = new JPanel();
                    panel.setLayout(new GridLayout(3, 0));
                    for (int i = 0; i < o.length; i ++) {
                        panel.add(new ObjectLabel(o[i]));
                    }
                    JFrame f = new JFrame("Genetic_map: " + o.length);
                    panel.setBorder(BorderFactory.createEtchedBorder());
                    f.getContentPane().add(panel, "Center");
                    f.setSize(300, 200);
                    f.setLocation(400, 400);
                    f.setVisible(true);
                }
            }
        }
    };

    /** Creates new MapMenuBar.
    * @param f the internalFrame holding a MapView.
    */
    public MapMenuBar(JInternalFrame f) {
        this.frame = f;
        initialize();
    }

    /** An overloaded constructor.
    * @param f the JFrame holding a MapView.
    */
    public MapMenuBar(JFrame f) {
        this.f = f;
        initialize();
    }

    /** An overloaded constructor.
    * @param view the MapView to be controlled by this Menubar.
    */
    public MapMenuBar(MapView view) {
        this.f = view.getViewFrame();
        this.view = view;
        isApplet = view.isApplet();
        initialize();
    }
    /** Setting up all menues.
     */
    private void initialize() {
        dFont = new Font("Dialog", 0, 11);
        if (!isApplet) setFileMenu();
        setViewMenu();
        //setToolsMenu();
        setHelpMenu();
    }

    /** Get the MapView this Menubar controlled.
    * @return the MapView this Menubar controlled.
    */
    public MapView getMapView() {
        return this.view;
    }

    private void setFileMenu() {
        file = new JMenu("File");
        file.setFont(dFont);
        JMenuItem connect = new JMenuItem("Connect");
        connect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                connectP = new ConnectPanel();
                connectP.hideUserAndPwd();
                final JFrame f = new JFrame("Connect...");
                connectP.addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent e) {
                        if (e.getPropertyName().equalsIgnoreCase("buttonClicked")) {
                            f.dispose();
                        }
                    }
                });
                connectP.addPropertyChangeListener(connectProp);
                f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                f.setSize(300, 200);
                f.getContentPane().add(connectP, "Center");
                f.setLocation(400, 400);
                f.setVisible(true);
            }
        });
        connect.setFont(dFont);
        JMenuItem open = new JMenuItem("Open");
        open.setFont(dFont);
        open.addActionListener(new OpenFileAction(f, getMapView()));
        JMenuItem quit = new JMenuItem("close");
        quit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (frame != null) {
                    frame.getContentPane().removeAll();
                    showFullMenu(false);
                    frame.validate();
                    frame.repaint();
                }
                if (f != null) {
                    f.getContentPane().removeAll();
                    showFullMenu(false);
                    f.validate();
                    f.repaint();
                    view.setTitle("Map View");
                }
            }
        });
        quit.setFont(dFont);

        file.add(connect);
        file.add(new JSeparator());
        file.add(open);
        file.add(new JSeparator());
        file.add(quit);
        this.add(file);
    }

    private void setViewMenu() {

        viewMenu = new JMenu("View");
        viewMenu.setFont(dFont);
        JMenu itemNumber = new JMenu("Item Number");
        itemNumber.setFont(dFont);
        JMenu items = new JMenu("Items");
        items.setFont(dFont);

        scaleLeft = new JCheckBox("Left Scale");
        scaleLeft.setActionCommand("sl");
        scaleLeft.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setShowingPanes(e);
            }
        });
        scaleLeft.setFont(dFont);
        scaleLeft.setSelected(true);

        scaleRight = new JCheckBox("Right Scale");
        scaleRight.setActionCommand("sr");
        scaleRight.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setShowingPanes(e);
            }
        });
        scaleRight.setFont(dFont);
        scaleRight.setSelected(true);

        loci = new JCheckBox("Loci");
        loci.setActionCommand("loci");
        loci.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setShowingPanes(e);
            }
        });
        loci.setFont(dFont);
        loci.setSelected(true);

        it = new JCheckBox("Interval (IT)");
        it.setActionCommand("it");
        it.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setShowingPanes(e);
            }
        });
        it.setFont(dFont);
        it.setSelected(true);
        //heckBox pg = new JCheckBox("Physical gene (PG)");
        //.setFont(dFont);

        //view.add(setting);
        //view.add(new JSeparator());
        viewMenu.add(items);

        items.add(scaleLeft);
        items.add(scaleRight);
        //items.add(cr);
        items.add(it);
        items.add(loci);
        //items.add(pg);
        this.add(viewMenu);
    }

    // These menu should be shown after contents loaded.
    /** Control if view and help menu should be displayed.
    * @param false, turn off view and help menus; while true, turn on view and help menus.
    */
    public void showFullMenu(boolean show) {
        viewMenu.setVisible(show);
        helpMenu.setVisible(show);
    }

    /*
    private void setToolsMenu() {
    JMenu tools = new JMenu("Tools");
    tools.setFont(dFont);
    JMenuItem functions = new JMenuItem("Functions");
    functions.setFont(dFont);
    tools.add(functions);
    this.add(tools);
    }
     */

    private void setShowingPanes(ActionEvent e) {
        if (view != null) {
            String command = e.getActionCommand();
            if (command.equalsIgnoreCase("sl")) {
                if (scaleLeft.getSelectedObjects() != null) view.showLeftScale(true);
                else view.showLeftScale(false);

            }
            else if (command.equalsIgnoreCase("sr")) {
                if (scaleRight.getSelectedObjects() != null) view.showRightScale(true);
                else view.showRightScale(false);
            }
            else if (command.equalsIgnoreCase("loci")) {
                if (loci.getSelectedObjects()!= null) {
                    view.showLocus(true);
                }
                else view.showLocus(false);
            }
            else if (command.equalsIgnoreCase("it")) {
                if (it.getSelectedObjects() != null) view.showInterval(true);
                else view.showInterval(false);
            }
        }
    }

    private void setHelpMenu() {
        helpMenu = new JMenu("Help");
        helpMenu.setFont(dFont);

        JMenuItem help = new JMenuItem("Help");
        help.setFont(dFont);
        help.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFrame f = new JFrame("Help");
                f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                f.setSize(400, 300);
                HelpPane h = new HelpPane(isApplet);
                h.setBorder(BorderFactory.createEtchedBorder());
                f.getContentPane().add(new JScrollPane(h), "Center");
                f.setVisible(true);
            }
        });
        helpMenu.add(help);
        this.add(helpMenu);
    }

    public static void main (String[] args) {
        JFrame frame = new JFrame("MenuBar Test");
        JInternalFrame f = new JInternalFrame(null, true, true, true, true);
        JMenuBar bar = new MapMenuBar(f);
        frame.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) { System.exit(0); }
        });
        frame.setJMenuBar(bar);
        frame.setSize(400, 300);
        frame.setVisible(true);
    }
}
