/** All mappoint display should implement this interface: All display functions
 */

package jadex.swing.map;

/** This is the specification of a MapPointDisplay. All mappoint display should 
 * implement this interface.
 * @author wgm
 * @version 1.0
*/

public interface MapPointDisplay {
    /** Set the MapRange to be displayed.
    * @param r the MapRange to be displayed.
    */
    public void setRange(MapRange r);
    
    /** Set the name layout by layout name.
    * @param layoutName the Name of the Layout to be used.
    */
    public void setNameLayout(String layoutName);
    
    /** Control if the connecting lines should be displayed or not.
    * @param shown true for showing the connecting lines.
    */
    public void setConnectShown(boolean shown);
    
    /** Get the status of the connecting lines.
    * @return true if the connecting lines are displayed.
    */
    public boolean isConnectShown();
    
    /** Record the name of the MapPoint where the mouse is in.
    * @param name the name of the MapPoint where the mouse is in.
    */
    public void setMouseInName(String name);
    
    /** Get the layout name.
    * @return the name of the layout.
    */
    public LocusLayout getNameLayout();
}