package jadex.swing.map;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.awt.image.*;
import java.awt.geom.*;

/** A customized JPanel for the ruler of MapRange in a MapView.
 * @author wgm
 * @version 1.0
 */

public class MapScale1 extends JPanel implements PropertyChangeListener {
    private MapRange range = null; // The MapRange to be displayed.
    private float extend, end;
    private int TOTAL_TICK = 10; // The major ticks.

    /** Default constructor. */
    public MapScale1() {
    }

    /** An overloaded constructor.
    * @param range the MapRange to be displayed.
    */
    public MapScale1(MapRange range) {
        setRange(range);
    }

    /** Set the displayed MapRange.
    * @param range the MapRange to be displayed.
    */
    public void setRange(MapRange range) {
        this.range = range;
        try {
            end = range.getEndF();
            extend = range.getExtendF();
            repaint();
        }
        catch (UnknownPositionException e) {
            System.out.println(range.toStringF());
            System.out.println("Scale cannot be initialized: " + e);
        }
    }

    /** Listening to the Range propety change. */
    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName().equalsIgnoreCase("range")) {
            MapRange newRange = (MapRange) e.getNewValue();
            setRange(newRange);
        }
    }

    /** Get the displayed MapRange. */
    public MapRange getRange() {
        return this.range;
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d = (Dimension2D) getSize();
        float w = (float) d.getWidth();
        float h = (float) d.getHeight();

        g2.setColor(Color.white);
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

        g2.setColor(Color.black);
        g2.draw(new Line2D.Float(w/6.0f, 2.0f, w/6.0f, h-2.0f));

        float hpos = 2.0f;
        float diff = (h - 4.0f) / TOTAL_TICK;
        for (int i = 0; i <= TOTAL_TICK; i ++) {
            g2.draw(new Line2D.Float(0.0f, hpos, w/3.0f, hpos));
            float sY = hpos;
            for (int j = 0; j < 10; j++) {
                sY += diff / 10;
                g2.draw(new Line2D.Float(w/9.0f, sY, 2.0f*w/9.0f, sY));
            }
            drawLabel(w/3.0f, hpos, g2, i);
            hpos += diff;
        }
    }

    private void drawLabel(float posx, float posy, Graphics2D g2, int number) {
        float diff = extend / TOTAL_TICK;
        int ll = (int)((end - number * diff) * 100);
        float label = ll / 100.0f;
        g2.setFont(new Font("Dialog", 0, 9));
        g2.drawString(label + "", posx, posy + 2.0f);
    }
}