/*
 * MiniMapCartoon.java
 *
 * Created on August 2, 2000, 8:41 AM
 */

package jadex.swing.map;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.event.*;

/**
 * A customized JPanel to display the zoomed region in the full chromosome.
 * @author  wgm
 * @version 1.0
 */
public class MiniMapCartoon extends JPanel implements PropertyChangeListener {

    private MapRange range = null; // currently displayed MapRange.
    private MapRange map = null; // full MapRange.
    private MapPoint[] marker = null; // The Markers used in the MapCartoon.
    private float rel; // Ratio between biology distance and graphic coordinates.

    /** Creates new MiniMapCartoon.
     * @param r the MapRange for initialization.
     */
    public MiniMapCartoon(MapRange r) {
        map = r;
        range = map;
    }

    /** Set the displayed MapRange.
    * @param r the displayed MapRange.
    */
    public void setRange(MapRange r) {
        this.range = r;
        repaint();
    }

    /** Set the main marker for this MapCartoon.
    * @param m the data to be used as the Markers.
    */
    public void setMarker(MapPoint[] m) {
        this.marker = m;
    }

    /** Get the markers.
    * @return the markers 
    */
public MapPoint[] getMarker() { return marker; }

/** Get the displayed MapRange.
* @return the displayed MapRange.
*/
public MapRange getRange() { return range; }

    public void paintComponent(Graphics g) {

        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d2 = (Dimension2D) getSize();
        float w = (float) d2.getWidth();
        float h = (float) d2.getHeight();

        g2.setColor(Color.yellow);
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

        g2.setColor(Color.gray);
        g2.fill(new RoundRectangle2D.Float(w/3.0f, 1.0f, w/3.0f, h - 2.0f, 5.0f, 5.0f));

        try {
            float rel = (h - 2.0f) / map.getExtendF();
            float offset = map.getEndF();

            g2.setColor(Color.black);
            if (marker != null) {
                for (int i = 0; i < marker.length; i++) {
                    float y = (offset - marker[i].getPosition()) * rel + 1.0f;
                    g2.draw(new Line2D.Float(w/6.0f, y, w/3.0f, y));
                }
            }

            float height = rel * range.getExtendF();
            if (height < 1.0f) height = 1.0f;
            float startY = (offset - range.getEndF()) * rel + 1.0f;
            g2.setColor(Color.green);
            g2.fill(new Rectangle2D.Float(w/3.0f, startY, w/3.0f, height));
        }
    catch (UnknownPositionException e) { System.err.println(e); }
    }

    /** Listen to the Range property change.
    */
    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName().equalsIgnoreCase("range")) {
            MapRange r = (MapRange) e.getNewValue();
            setRange(r);
        }
    }
}