/*
 * OpenFileAction.java
 *
 * Created on July 20, 2000, 3:24 PM
 */

package jadex.swing.map;

import java.awt.event.*;
import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.*;

import jadex.swing.classView.*;

/**
 * This ActionListener is used to open a genetic_map file in the MapView menubar.
 * @author  wgm
 * @version 1.0
 */
public class OpenFileAction implements ActionListener {

    private JFrame frame = null; // The frame the genetic_map will be displayed.
    private File file = null; // The file containing information for the genetic_map.
    // These arrays containing data 
    private Object[][] marker, locus, interval;
    private String title = null; // The title for the frame.
    // the array length.
    private int locusLength, intervalLength;
    private Vector markers, loci, intervals;
    private MapView view = null;

    private boolean debug = false;
    private boolean debug1 = false;

    /** Creates new OpenFileAction */
    public OpenFileAction() {
    }

    /** An overloaded constructor.
    * @param f the JFrame displaying the Genetic_map.
    * @param view the MapView.
    */
    public OpenFileAction(JFrame f, MapView view) {
        frame = f;
        this.view = view;
        //markers = new Vector();
        //loci = new Vector();
        //intervals = new Vector();
    }

    /** Actual method to open the file.
    */
    public void actionPerformed(ActionEvent p1) {
        JFileChooser chooser = new JFileChooser();
        chooser.setCurrentDirectory(new File("/home/wgm/java/forte4j/Development"));
        int returnVal = chooser.showOpenDialog(frame);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            file = chooser.getSelectedFile();
            if (debug) System.out.println(file.getName());
            try {
                processFile(file);
                processView();
            }
            catch(Exception e) {
                System.err.println("OpenFileAction.actionPerformed(): " + e);
                try {
                    PrintWriter writer = new PrintWriter(new FileWriter("except.out"));
                    e.printStackTrace(writer);
                    writer.close();
                }
            catch(IOException e1) {System.err.println(e1); }
            }
        }

    }

    private void processView() {
        view.setMarker(getMarkers());
        view.setInterval(getIntervals());
        view.setLocus(getLoci());
        view.setFrame();
    }
    
    /** Processing the file
    */
    public void processFile(File file) {
        Vector content = null;
        markers = new Vector();
        loci = new Vector();
        intervals = new Vector();
        try {
            BufferedReader reader = new BufferedReader(new FileReader(file));
            String line = null;

            while (true)  {
                line = reader.readLine();
                if (debug) System.out.println(line);
                if (line == null) break;
                if (line.length() == 0) break;
                if (line.indexOf("Map") != -1) {
                    title = line;
                    if (view != null)view.setTitle("Map View: " + title);
                    continue;
                }
                if (line.indexOf("marker_length") != -1) {
                    content = markers;
                    continue;
                }
                if (line.indexOf("loci_length") != -1) {
                    getLocusLength(line);
                    content = loci;
                    continue;
                }
                if (line.indexOf("interval_length") != -1) {
                    getIntervalLength(line);
                    content = intervals;
                    continue;
                }
                content.addElement(line);
            }
        }
    catch (IOException e) { System.out.println(e); }

        if (debug) System.out.println("Locus length: " + loci.size());
        marker = setMarker(markers);
        locus = setLoci(loci);
        if (debug) System.out.println("Interval length: " + intervals.size());
        setIntervals(intervals);
    }

    private void getLocusLength(String line) {
        int pos = new String("loci_length").length();
        String s = line.substring(pos).trim();
        locusLength = Integer.parseInt(s);
        if (debug) System.out.println(s);
    }

    private void getIntervalLength(String line) {
        int pos = new String("interval_length").length();
        String s = line.substring(pos).trim();
        intervalLength = Integer.parseInt(s);
        if (debug) System.out.println(s);
    }

    private Object[][] setMarker(Vector marker) {
        int size = marker.size();
        Object[][] os = new Object[size][2];
        String s = null;
        for (int i = 0; i < size; i ++) {
            s = (String) marker.elementAt(i);
            if (debug1) System.out.println(s);
            int pos = s.indexOf("  ");
            String name = s.substring(0, pos).trim();
            Float position = new Float(s.substring(pos).trim());
            os[i][0] = name;
            os[i][1] = position;
        }
        return os;
    }

    private Object[][] setLoci(Vector loci) {
        int size = loci.size();
        Object[][] os = new Object[size][3];
        String s = null;
        for (int i = 0; i < size; i ++) {
            s = (String) loci.elementAt(i);
            if (debug1) System.out.println(s);
            //int pos = s.indexOf("  ");
            //String name = s.substring(0, pos).trim();
            //Float position = new Float(s.substring(pos).trim());
            //os[i][0] = name;
            //os[i][1] = position;
            int pos1 = s.indexOf("  ");
            int pos2 = s.lastIndexOf("  ", s.length() - 1);
            String name = s.substring(0, pos1).trim();
            Float pos = new Float(s.substring(pos1, pos2).trim());
            Float error = new Float(s.substring(pos2).trim());
            os[i][0] = name;
            os[i][1] = pos;
            os[i][2] = error;
        }

        return os;
    }

    private void setIntervals(Vector intervals) {
        int size = intervals.size();
        interval = new Object[size][3];
        String s = null;
        for (int i = 0; i < size; i ++) {
            s = (String) intervals.elementAt(i);
            if (debug1) System.out.println(s);
            int pos1 = s.indexOf("  ");
            int pos2 = s.lastIndexOf("  ", s.length() - 1);
            String name = s.substring(0, pos1).trim();
            Float start = new Float(s.substring(pos1, pos2).trim());
            Float end = new Float(s.substring(pos2).trim());
            interval[i][0] = name;
            interval[i][1] = start;
            interval[i][2] = end;
        }
    }

public Object[][] getMarkers() { return marker; }

    public Object[][] getLoci() {
        return locus;
    }


    public Object[][] getIntervals() {
        return interval;
    }
}