/** A subclass of locuslayout to implement a smart locus labelling method
 */

package jadex.swing.map;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

/** A subclass of locuslayout to implement a smart locus labelling method.
 * @author wgm
 * @version 1.0
 */

public class SmartLocusLayout extends LocusLayout {

    /** The sole construcotr.
    * @param yPadding the vertical padding.
    * @param npc the number per column.
    * @param connect the flag to control if the connecting lines should be displayed.
    * @param c the color of the connecting lines
    */
    public SmartLocusLayout(float yPadding, int npc, boolean connect, Color c) {
        super(yPadding, npc, connect, c);
    }

    private void doSorting() {
        java.util.List l = Arrays.asList(lUI);
        Collections.sort(l, comp);
    }

    /** Actual layout methods.
    * @param x1 the initial x position
    * @param h the height of the display.
    * @param offset the initial position.
    * @param rel the ratio between the biological position and the graphics coordinate.
    * @param g2 the Graphics context.
    */
    public void drawNames(float x1, float h, float offset, float rel, Graphics2D g2) {
        if (lUI == null) return;

        int total = lUI.length;
        float x = x1 + 25.0f;

        java.util.List listPos = new ArrayList();

        for (int j = 0; j < total; j++) {

            float pos = lUI[j].getPosition();
            float y = (offset - pos) * rel + yPadding;
            Point p = findPos(listPos, x, y - 5.0f, h);
            lUI[j].setBounds((int)p.getX(), (int)p.getY(), 40, 10);
            listPos.add(new Rectangle((int)p.getX(), (int)p.getY(), 40, 10));
            if (cShown) {
                if ((mouseInName != null) && (lUI[j].getName().equals(mouseInName)))
                g2.setColor(Color.red);
                else g2.setColor(cColor);
                g2.draw(new Line2D.Float(x1, y, (float) p.getX(), (float) p.getY() + 5.0f));
            }
        }
    }

    private Point findPos(java.util.List l, float x, float y, float h) {
        Rectangle r = new Rectangle((int) x, (int) y, 40, 10);
        Rectangle cur = null;
        Iterator iterator = null;
        boolean found = false;
    int[] steps = {-10, 20, -40, 60};

        while (true) {
            for (int i = 0; i < steps.length; i ++) {
                int count = 0;
                for (iterator = l.iterator(); iterator.hasNext();) {
                    cur = (Rectangle) iterator.next();
                    if (cur.intersects(r)) {
                        break;
                    }
                    count ++;
                }
                if (count == l.size()) {
                    found = true;
                    break;
                }
                float pY = r.y + steps[i];
                if ((pY < 0) || (pY > h)) break;
                r.setLocation(r.x, r.y + steps[i]);
            }
            if (found) break;
            r.setLocation(r.x + 40, (int) y);
        }
        return new Point(r.x, r.y);
    }

    /** Get the specified name for this LocusLayout.
    * @return the name of this LocusLayout, smaartLayout.
    */
public String getName() { return new String("smartLayout"); }
}