/** A little ornament used in the vertival line in LocusDisplay
 */

package jadex.swing.map;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;

/** This Knob is used on the north end of the chromosome line in the MapView.
 * @author wgm
 * @version 1.0
 */

public class UpKnob extends Knob {

    //If at the top, it should be a circle. Otherwise, it should be an arrow
    private boolean isTop = true; // default at the top
    // This is a control to implement the moving behavior

    /** Default constructor. Link mouse activities to this knob in this constructor. */
    public UpKnob() {
        super();
    }

    /** An overloaded constructor.
    * @param c the DisplayControl used in this Knob.
    */
    public UpKnob(DisplayControl c) {
        super(c);
    }

    /** Set the dircetion of this knob.
    * @param top the direction of this Knob: true for moving up, false for no moving.
    */
    public void setDirection(boolean top) {
        if (isTop == top) return;
        isTop = top;
        if (isTop) setToolTipText("No noving");
        else setToolTipText("Moving up");
        repaint();
    }

    /** Get the direction as a string.
    * @return the direction, Up for moving up, No for no moving.
    */
    public String getDirection() {
        String rtn = null;
        if (isTop) rtn = new String("Up");
        else rtn = new String("No");
        return rtn;
    }

    /** All paint activities. */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d2 = (Dimension2D) getSize();
        float w = (float) d2.getWidth();
        float h = (float) d2.getHeight();

        g2.setPaint(Color.white); // Background
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

        if (mouseIn) g2.setPaint(Color.red); // Color for these small ornaments
        else g2.setPaint(Color.gray);

        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));

        if (isTop)g2.fill(new Ellipse2D.Float(0.0f, 0.0f, w, h));
        else {
            g2.draw(new Line2D.Float(0.0f, h/2.0f, w/2.0f, 0.0f));
            g2.draw(new Line2D.Float(w/2.0f, 0.0f, w, h/2.0f));
            g2.draw(new Line2D.Float(w/2.0f, 0.0f, w/2.0f, h));
        }
    }

    /** Move up the range. */
    protected void moving() {
        if (isTop) return;
        else if (control != null) control.movingUp();
    }
}