/** This is a JPanel used to zoom MapCartoon by JButtons
 */

package jadex.swing.map;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.applet.*;

import jadex.swing.*;

/** A customized JPanel holding all zooming buttons.
 * @author wgm
 * @version 1.0
*/

public class ZoomButtonControl extends JPanel implements ActionListener {

    // The range of control controlled by this class.
    private MapControl control = null;
    private MapRange map = null; // The largest range of the control.
    private float extend; // Total extend of the control.
    private float step;   // Step for zooming in or out

    /** Default constructor.
    */
    public ZoomButtonControl() {
        buildUI();
    }

    /** An overloaded constructor.
    * @param c the MapControl used for these zooming activities.
    */
    public ZoomButtonControl(MapControl c) {
        buildUI();
        setMapControl(c);
    }

    //Setting up all GUIs
    private void buildUI() {

        String sep = File.separator;
        String path = new String("jadex" + sep + "swing" + sep + "map" + sep);
        setLayout(new GridLayout(1, 4));

        ImageIcon icon1, icon2, icon3, icon4;

        try {
            icon1 = new ImageIcon(path + "Up16.gif");
            icon2 = new ImageIcon(path + "ZoomIn16.gif");
            icon3 = new ImageIcon(path + "ZoomOut16.gif");
            icon4 = new ImageIcon(path + "Down16.gif");
        }
        catch(Exception e) { // Maybe in an applet
            icon1 = new ImageIcon(getClass().getResource("Up16.gif"));
            icon2 = new ImageIcon(getClass().getResource("ZoomIn16.gif"));
            icon3 = new ImageIcon(getClass().getResource("ZoomOut16.gif"));
            icon4 = new ImageIcon(getClass().getResource("Down16.gif"));
        }

        JButton up = new JButton(icon1);
        up.setToolTipText("Click to move up the selected region in the chromosome");
        up.setPreferredSize(new Dimension(20, 20));
        up.setActionCommand("up");
        up.addActionListener(this);
        add(up);

        JButton zoomIn = new JButton(icon2);
        zoomIn.setToolTipText("Click to zoom in the selected region in the chromosome");
        zoomIn.setPreferredSize(new Dimension(20, 20));
        zoomIn.setActionCommand("zoomIn");
        zoomIn.addActionListener(this);
        add(zoomIn);

        JButton zoomOut = new JButton(icon3);
        zoomOut.setToolTipText("Click to zoom out the selected region in the chromosome");
        zoomOut.setPreferredSize(new Dimension(20, 20));
        zoomOut.setActionCommand("zoomOut");
        zoomOut.addActionListener(this);
        add(zoomOut);

        JButton down = new JButton(icon4);
        down.setToolTipText("Click to move down the selected region in the chromosome");
        down.setPreferredSize(new Dimension(20, 20));
        down.setActionCommand("down");
        down.addActionListener(this);
        add(down);
    }

    /** Set up the MapControl for all zooming buttons.
    * @param c the MapControl coordinating all zooming activities in a MapView.
    */
    public void setMapControl(MapControl c) {
        this.control = c;
        try {
            map = c.getMap();
            extend = map.getExtendF();
            step = extend / 40.0f; // Arbitrary setting up a step, should be changed in the future.
        }
    catch (UnknownPositionException e) { System.err.println(e); }
    }

    /** A test method. */
    public static void main(String[] args) {
        JFrame f = new JFrame("ButtonControl Panel");
        f.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) { System.exit(0); }
        });
        JPanel p = new ZoomButtonControl();
        f.getContentPane().add(p, "Center");
        f.setSize(100, 50);
        f.setVisible(true);
    }

    /** Processing all button actions. All actions have been centralized here. 
    * Probably this is not a good way, and should do more refactoring.
    */
    public void actionPerformed(ActionEvent p1) {
        String command = p1.getActionCommand();
        MapRange current = control.getRange();
        try {

            float  cStart = current.getStartF();
            float cEnd = current.getEndF();
            float step = current.getExtendF() / 3.0f;

            if (command.equalsIgnoreCase("up")) {
                if (cEnd == map.getEndF()) return;
                cEnd += step;
                cStart += step;
                //cStart = cEnd;
                //cEnd = cStart + current.getExtendF();
                if (cEnd > map.getEndF()) cEnd = map.getEndF();
            }
            else if (command.equalsIgnoreCase("down")) {
                if (cStart == map.getStartF()) return;
                cEnd -= step;
                //cEnd = cStart;
                cStart -= step;
                //cStart = cEnd + current.getExtendF();
                if (cStart < map.getStartF()) cStart = map.getStartF();
            }
            else if (command.equalsIgnoreCase("zoomIn")) {
                //if ((cEnd - cStart) < (extend / 200.0f)) return; // Zoomin is enough at this step.
                float step1 = (cEnd - cStart) / 20.0f;
                cStart += step1;
                cEnd -= step1;
                if (cStart >= cEnd) {
                    cEnd = cEnd + step1;  // Return to the original values
                    cStart = cStart - step1;
                }
                //System.out.println(cEnd + "   " + cStart);
            }
            else if (command.equalsIgnoreCase("zoomOut")) {
                cStart -= step;
                cEnd += step;
                if (cStart < map.getStartF()) cStart = map.getStartF();
                if (cEnd > map.getEndF()) cEnd = map.getEndF();
            }

            MapRange range = new MapRange(cStart, cEnd);
            control.setRange(range);
        }
    catch (UnknownPositionException e) { System.err.println(e); }
    }
}
