/*
 * ZoomPopupMenu.java
 *
 * Created on July 26, 2000, 8:59 AM
 */

package jadex.swing.map;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * A customized JPopupMenu for zooming. This JPopupMenu is used in IntervalContainer class.
 * @author  wgm
 * @version 1.0
 */
public class ZoomPopupMenu extends JPopupMenu {

    private MapZoom panel = null; // The controlled JPanel.
    private JFrame f = null; // The JFrame holding the panel.
    private Font deFont = new Font("Dialog", 0, 11); // Default font.
    private int STEP = 5; // Zooming magnitude
    private int count = 0; // number of zooming

    // Zoom in activity.
    private ActionListener up = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            if (panel != null) {
                panel.horizontalZoom(STEP);
                count ++;
                //f.validate();
                //f.repaint();
            }
        }
    };

    // Zooming out
    private ActionListener down = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            if (panel != null) {
                panel.horizontalZoom(-STEP);
                count --;
                //f.validate();
                //f.repaint();
            }
        }
    };

    // Reset to the original setting.
    private ActionListener reset = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            if ((panel != null) && count != 0) {
                panel.horizontalZoom(-STEP * count);
                count = 0;
                //f.validate();
                //f.repaint();
            }
        }
    };


    /** Creates new ZoomPopupMenu */
    public ZoomPopupMenu() {
    }

    /** A overloaded constructor.
    * @param p the zoomable display.
    * @param f the JFrame holding this display.
    */
    public ZoomPopupMenu(MapZoom p, JFrame f) {
        this.f = f;
        setPanel(p);
    }

    /** Set the zoomable.
    * @param p the zoomabel Panel.
    */
    public void setPanel(MapZoom p) {
        this.panel = p;
        initialize();
    }

    /** GUI initialization. */
    private void initialize() {
        setLabel("Horizontal Zoom");
        JMenu zoom = new JMenu("Horizontal Zoom...");
        zoom.setFont(deFont);
        JMenuItem increase = new JMenuItem("+");
        increase.setFont(deFont);
        increase.addActionListener(up);
        JMenuItem decrease = new JMenuItem("-");
        decrease.setFont(deFont);
        decrease.addActionListener(down);
        zoom.add(increase);
        zoom.add(decrease);
        add(zoom);
        JMenuItem reset1 = new JMenuItem("reset");
        reset1.setFont(deFont);
        reset1.addActionListener(reset);
        add(new JSeparator());
        add(reset1);
    }
}