/*
 * AceDBCloneFetcher.java
 *
 * Created on September 26, 2000, 11:30 AM
 */

package jadex.swing.sequence;

import java.io.*;
import java.util.*;

import jade.db.*;
import jadex.swing.map.*;

/**
 * This class is used to fetch SequenceMap information from AceDB directly for SequenceMap display.
 * @author  wgm
 * @version 1.0
 */
public class AceDBCloneFetcher extends CloneFetcher {

    private String name = null; // The name of sequence_map
    private JSAceAccessor accessor = null; // The AceAccessor talking to AceDB

    /** Default constructor */
    public AceDBCloneFetcher() {
    }

    /** Second constructor: AceDB host, port and Sequene_map object name.
    * @param host the AceDB host name.
    * @param port the port number of the AceDB.
    * @param name the name of the Sequence_map.
     */
    public AceDBCloneFetcher(String host, int port, String name) {
        this.name = name;
        accessor = new JSAceAccessor(host, port);
    }

    /** Third constructor: AceAccessor and Sequence_map name.
    * @param ace the AceAccessor used to access the information of AceDB.
    * @param name the name of the Sequence_map to be displayed.
    */
    public AceDBCloneFetcher(JSAceAccessor ace, String name) {
        this.name = name;
        this.accessor = ace;
    }

    /** Set the name of the Sequence_map to be displayed.
    * @param name the name of the sequence_map.
    */
    public void setName(String name) {
        this.name = name;
    }

    /** Get the name of the displayed Sequence_map.
    * @return the name of the displayed Sequence_map.
    */
public String getName() { return name; }

    /** Set the AceAccessor for this object.
    * @param ace the AceAccessor used for this object.
    */
    public void setAccessor(JSAceAccessor ace) {
        this.accessor = ace;
    }

    /** Get the AceAccessor used in this Sequence_map dislay.
    * @return the AceAccessor used in this Sequence_map.
    */
public AceAccessor getAccessor() { return this.accessor; }

    /** Get all information about clones in the Sequence_map in a String.
    * @return a String containing all clone information in the Sequence_map.
    */
    public String getCloneInfor() {

        String query = null; // AQL query
        String result = null; // Query result
        StringBuffer buf = new StringBuffer(); // The StringBuffer will hold all results from AceDB
        buf.append(name + "\n");
        // First to get contig information
        query = new String("aql -h select a->contig from a in object(\"map\", \"" + name + "\")");
        //System.out.println(query);
        accessor.write(query);
        result = accessor.read();
        String[] ctgs = getContigs(result);
        getContigInfor(ctgs, buf);
        buf.append("\n");

        getContigCloneInfor(ctgs, buf);
        buf.append("\n");

        getCloneInfor(buf);
        buf.append("\n");

        getChromInfor(buf);

        return buf.toString();
    }

    // A refactored method to get contig information
    private void getContigInfor(String[] ctgs, StringBuffer buf) {
        buf.append(name + super.link + "contig\n");
        // Get individual contig information
        String query = null;
        String result = null;
        for (int i = 0; i < ctgs.length; i++) {
            buf.append(ctgs[i] + "\n");
            query = new String("aql -h select m, m[Left], m[Right] from m in object(\"Contig\", \""
            + ctgs[i] + "\")->Map");
            //System.out.println(query);
            accessor.write(query);
            result = accessor.read();
            buf.append(result + "\n");
            query = new String("aql -h select \"Pmap\", m[0], m[1] from m in object(\"Contig\", \""
            + ctgs[i] + "\")->pMap");
            //System.out.println(query);
            accessor.write(query);
            result = accessor.read();
            buf.append(result + "\n");
        }
    }

    // A helper method to get all contig names in sequence_map
    private String[] getContigs(String str) {
        StringTokenizer tokenizer = new StringTokenizer(str, "\n");
        java.util.List list = new ArrayList();

        String token = null;
        while (tokenizer.hasMoreTokens()) {
            token = tokenizer.nextToken().trim();
            if (token.startsWith("Format")) continue;
            if (token.startsWith(new String("//"))) continue;
            list.add(token);
        }

        return (String[]) list.toArray(new String[0]);
    }

    // A refactored method to get contigcloneinfor
    private void getContigCloneInfor(String[] ctgs, StringBuffer buf) {
        //get contig clones' information
        buf.append(name + super.link + "contig" + super.link + "clone\n");
        String query = null;
        String result = null;

        for (int i = 0; i < ctgs.length; i++) {
            query = new String("aql -h @clones := select a->clone from a in object(\"contig\", \"" +
            ctgs[i] + "\") ; select c:1, m[0], m[1], m[2] from c in @clones, m in c:1->Pmap");
            accessor.write(query);
            result = accessor.read();
            buf.append(result + "\n");
        }
    }

    // A refactored method to get cloneinfor
    private void getCloneInfor(StringBuffer buf) {
        // get sequence_map clones' information
        buf.append(name + super.link + "clone");
        String query = new String("aql -h @clones := select a->clone from a in object(\"map\", \""
        + name + "\" ) ; select c:1, m[Left], m[Right] from c in @clones, m in c:1->map");
        accessor.write(query);
        String result = accessor.read();
        buf.append(result + "\n");
    }


    // A refactored method to get Chrominfor
    private void getChromInfor(StringBuffer buf) {
        buf.append(name + super.link + "infor\n");
        String query = new String("aql -h select \"extent\", a[0], a[1] from a in object(\"map\", \""
        + name + "\")->Extent");
        accessor.write(query);
        String result = accessor.read();
        buf.append(result + "\n");
    }

    public static void main(String[] args) {
        if (args.length != 3) {
            System.out.println("USage java AceDBCloneFetcher host port sequence_map");
            System.exit(0);
        }
        String host = args[0];
        int port = Integer.parseInt(args[1]);
        String obj = args[2];
        AceDBCloneFetcher fetcher = new AceDBCloneFetcher(host, port, obj);
        long start = System.currentTimeMillis();
        fetcher.fetchClone();
        long end = System.currentTimeMillis();
        System.out.println("Time used to processing the data: " + (end - start));
        Map map = fetcher.getClone();
        System.out.println("Clone nummber: " + map.size());
        if (map != null) {
            Set keys = map.keySet();
            String name = null;
            for (Iterator iterator = keys.iterator(); iterator.hasNext(); ) {
                name = (String) iterator.next();
                Clone clone = (Clone) map.get(name);
                System.out.println(clone.toString());
            }
        }
        Map ctg = fetcher.getContig();
        if (ctg != null) System.out.println("Contig number: " + ctg.size());
        MapRange r = fetcher.getMapScale();
        if (r != null) System.out.println("Map extent: " + r.toString());
    }
}