/*
 * Clone.java: An individual clone is a MapPoint, but it also contains a maprange
 *
 * Created on September 26, 2000, 11:02 AM
 */

package jadex.swing.sequence;

import jadex.swing.map.*;
import jade.db.*;
/**
 * This wrapper class contains information about a clone. An individual clone is a MapPoint,
 * but it also has a MapRange.
 * @author  wgm
 * @version 1.0
 */
public class Clone extends Object implements MapPoint {

    private AceObject obj = null; // If the data coming from AceDB, this clone is an aceobject
    // which is warpped in this class
    private String name = null; // The name of the clone
    private float pos; // The center point of the clone, i.e., average of start and end of maprange
    private MapRange range = null; // The maprange of the clone in GeneticMap
    private MapRange pmap = null; // The PMap of the clone in contig
    private Contig contig = null;  // The possible contig this clone may be in

    /** Creates new Clone */
    public Clone() {
    }

    /** Another constructor with name as a argument */
    public Clone(String n) {
        setName(n);
    }

    /** Third constructor with name and maprange as arguements */
    public Clone(String n,MapRange r) {
        setName(n);
        setMapRange(r);
    }

    /** Fourth constructor with name, mapRange, and contig */
    public Clone(String n, MapRange r, Contig ctg) {
        setContig(ctg);
        setName(n);
        setPMap(r); // If there is a contig, maprange should be PMap
    }

    /** Get the name of the clone.
    * @return the name of the clone.
     */
    public String getName() {
        return this.name;
    }

    /** Get the MapRange of the clone.
    * @return the MapRange of the Clone.
    */
    public MapRange getMapRange() {
        return this.range;
    }

    /** Get the position of the clone.
    * @return the position of the clone.
    * @exception throw an unknownPositionException when the position is unknown.
    */
    public float getPosition() throws UnknownPositionException {
        return this.pos;
    }

    /** Set the name of the clone.
    * @param name the name of the clone.
    */
    public void setName(String name) {
        this.name = name;
    }

    /** Set the MapRange of the clone.
    * @param r the MapRange of the clone.
    */
    public void setMapRange(MapRange r) {
        this.range = r;
        try {
            float start = r.getStartF();
            float end = r.getEndF();
            float pos = (start + end) / 2.0f;
            setPosition(pos);
        }
        catch(UnknownPositionException e) {
            System.err.println("Clone.setMapRange(): " + e);
        }
    }

    /** Set the PMap of the clone if any.
    * @param r the PMap of the clone. 
    */
    public void setPMap(MapRange r) {
        this.pmap = r;
        if (contig != null) {
            setMapRange(contig.getRangeFromPMap(pmap));
        }
    }

    /** Set the position of the clone.
    * @param position the position of the clone.
    */
    public void setPosition(float position) {
        this.pos = position;
    }

    /** Set the contig this clone belongs to.
    * @param ctg the contig of the clone.
    */
    public void setContig(Contig ctg) {
        this.contig = ctg;
    }

    /** Do nothing */
    public void setError(float err) {
    }

    /** No error data from AceDB */
    public float getError() {
        return 0.0f;
    }

    /** The the wrapped AceObject for this clone.
    * @param o the wrapped AceObject for this clone.
    */
    public void setAceObject(AceObject o) {
        this.obj = o;
    }

    /** Get the wrapped AceObject for this clone.
    * @return the wrapped AceObject.
    */
    public AceObject getAceObject() {
        return this.obj;
    }

    /** Comparae if two clones are equal. If their names are the same, they should 
    * be the same clones.
    * @param another the compared clone.
    */
    public boolean equals(Clone another) {
        String aName = another.getName();
        if (name.equalsIgnoreCase(aName)) return true;
        else return false;
    }

    /** A overrode method for pretty-printing.
    */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        if (name != null) buf.append("Name: Clone : " + name + "\n");
        if (range != null) buf.append("Range: " + range + "\n");
        buf.append("Pos: " + pos + "\n");
        if (contig != null) buf.append("Contig: " + contig.getName() + "\n");
        return buf.toString();
    }
}