/*
 * CloneControl.java
 *
 * Created on October 6, 2000, 3:41 PM
 */

package jadex.swing.sequence;

import jadex.swing.map.*;
import jade.db.*;

import java.util.*;
import java.awt.Color;

/**
 * A subclass of DisplayControl to control CloneDisplay.
 * @author  Guanming Wu
 * @version 1.0
 */
public class CloneControl extends DisplayControl {

    private CloneDisplay display = null; // The display controled by this class
    private CloneFetcher fetcher = null; // Fetcher for get data from file or acedb

    //private DisplayModel model = null; // Use the displaymodel for holding all clone data
    // Do I need to subclass displaymodel for this special needs?
    // I don't think it is necessary now.
    private java.util.List assignedClones = null; // Clones has been distributed on the left side of
    // the vertical line as list of list

    private boolean debug = false; // Turn debug mode on or off
    /** Default constructor */
    public CloneControl() {
    }

    /** Second constructor with CloneDisplay as the argument */
    public CloneControl(CloneDisplay view) {
        setCloneDisplay(view);
    }

    /** Third constrcutor with clonedisplay and clonefetcher as the arguement */
    public CloneControl(CloneDisplay view, CloneFetcher fetcher) {
        setCloneDisplay(view);
        setCloneFetcher(fetcher);
    }

    /** Fourth constructor with clonedisplay and displaymodel as the argumetns */
    public CloneControl(CloneDisplay view, DisplayModel model) {
        setCloneDisplay(view);
        setModel(model);
    }

    /** Set the clonedisplay.
    * @param view the CloneDisplay this object controls.
    */
    public void setCloneDisplay(CloneDisplay view) {
        display = view;
    }

    /** Set the clone fetcher, and get data from data source and initialize the clone model.
    * @param f the CloneFetcher used for fetching data from AceDB, a file or some other sources.
    */
    public void setCloneFetcher(CloneFetcher f) {
        fetcher = f;
        //fetcher.fetchClone();   // Let other class call this method first to get data
        Map map = fetcher.getClone();
        if (map != null) {
            DisplayModel model = new DisplayModel();
            setModel(model);
            Set keys = map.keySet();
            String name = null;
            for (Iterator iterator = keys.iterator(); iterator.hasNext(); ) {
                name = (String) iterator.next();
                Clone clone = (Clone) map.get(name);
                model.addPoint(clone);
                if (debug) System.out.println(clone.toString());
            }
        }
        setTotalMapRange(f.getMapScale());
    }

    /** Get the clones displayed in the CloneDisplay.
    * @return the clone list.
    */
    public List getClones() {
        return Arrays.asList(getModel().getData());
    }

    /** Get the distributed clones.
    * @return the distributed clones. This is a list of lists.
    */
    public java.util.List getAssignedClones() {
        if (assignedClones == null) assignClones(getClones());
        return assignedClones;
    }

    /** The algorithim to distribute the clones. 
    * @param list the clones to be distributed.
    */
    protected void assignClones(java.util.List list) {
        assignedClones = new ArrayList();
        List first = new ArrayList(); // The first list in the assignedClones
        assignedClones.add(first);
        Clone clone = null;
        for (Iterator iter = list.iterator(); iter.hasNext();) {
            clone = (Clone) iter.next();
            if (!add(assignedClones, clone)) {
                List more = new ArrayList();
                more.add(clone);
                assignedClones.add(more);
            }
        }
    }

    /** A helper method. */
    private boolean add(List list, Clone clone) {
        boolean rtn = false;
        MapRange range = clone.getMapRange();
        for (Iterator it1 = list.iterator(); it1.hasNext();) {
            List l = (List) it1.next();
            boolean inter = false;
            for (Iterator it2 = l.iterator(); it2.hasNext();) {
                Clone aCln = (Clone) it2.next();
                MapRange range1 = aCln.getMapRange();
                if (range.intersect(range1)) {
                    // Intersect with some clone
                    inter = true;
                    break;
                }
            }
            if (inter) continue; // see next list
            l.add(clone); // clone can be added into this list
            rtn = true;
            break;
        }
        return rtn;
    }

    /** Return CloneDisplay implementing MapPointDisplay */
    public MapPointDisplay getDisplay() {
        return display;
    }

    /** UI should be reset after total points or
     * showing points in range have been changed.
    */
    protected void resetUI() {
        int size = showNumber > tPoints.length ? tPoints.length : showNumber;
        sPoints = new MapPoint[size];
        sLoci = new LocusUI[size];
        Clone tmp = null;
        AceAccessor accessor = null;
        
        // If CloneFetcher is a AceDBCloneFetcher, try to set the LocusUI for Clone Fetching.
        if (fetcher instanceof AceDBCloneFetcher) {
            AceDBCloneFetcher acbFetcher = (AceDBCloneFetcher) fetcher;
            accessor = acbFetcher.getAccessor();
        }
        
        for (int i = 0; i < size; i++) {
            sPoints[i] = tPoints[i];
            sLoci[i] = new LocusUI(tPoints[i]);
            sLoci[i].setAceAccessor(accessor);
            sLoci[i].setIsClone(true);
            tmp = (Clone) tPoints[i];
            sLoci[i].setToolTipText(tmp.getName() + " " + tmp.getMapRange());
        }
        if (pattern != null) doSearch(pattern);
        else updateUI();
        //updateUI();
    }

    /** Show the data in LocusDisplay: points, little bars, and default names
    * This is the central points controled by this LocusControl. All changes will be
    * through this method.
    */
    public void updateUI() {
        // If there are data to be shown and display can be updated
        if (display != null) {
            display.setShowNames(sLoci);
            //System.out.println("sLoci.length: " + sLoci.length);
            //display.setTotalLocus(tPoints);
            //System.out.println("tPoint.length: " + tPoints.length);
            display.repaint();
            if (display.getPopup() != null) display.getPopup().checkShowMenu();
        }
    }
}