/*
 * CloneUI.java: The vertical bar for representing the clone on the left
 *
 * Created on November 1, 2000, 10:51 AM
 */

package jadex.swing.sequence;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

/**
 * The vertical bar for representing the clone on the left to the chromosome line
 * on the CloneDisplay.
 * @author  wgm
 * @version 1.0
 */
public class CloneUI extends JComponent implements MouseListener {
    
    private String name = null; // The name of the clone
    private Clone clone = null; // The clone this display is for
    private Color MOUSE_IN_COLOR = Color.red; // The color when mouse is in
    private Color MOUSE_OUT_COLOR = Color.orange; // The color when mouse is out
    private Color foreColor = MOUSE_OUT_COLOR; // The color of this UI
    
    /** Creates new CloneUI */
    public CloneUI() {
        initialize();
    }

    /** Another constructor with name as the argument */
    public CloneUI(String name) {
        setName(name);
        initialize();
    }
    
    /** One more constructor with clone as the argument */
    public CloneUI(Clone c) {
        setClone(c);
        initialize();
    }
    
    /** GUI initializtion */
    private void initialize() {
        addMouseListener(this);
    }
    
    /** Set the name of the clone. 
    * @param name the name of the clone.
    */
    public void setName(String name) { 
        this.name = name; 
    }
    
    /** Get the name of the clone.
    * @return the name of the clone displayed in this UI.
    */
    public String getName() { return this.name; }
    
    /** set the clone to be displayed.
    * @param c the clone to be displayed.
    */
    public void setClone(Clone c) { 
        this.clone = c;
        setName(clone.getName());
        setToolTipText(name + ": " + clone.getMapRange());
    }
    
    /** Get the displayed clone.
    * @return the displayed clone.
    */
    public Clone getClone() { return this.clone; }
    
    /** Set the color of this cloneUI. */
    public void setColor(Color c) { 
        this.foreColor = c; 
        repaint();
    }
       
    /** Get the color of this cloneUI. */
    public Color getColor() { return this.foreColor; }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d2 = (Dimension2D) getSize();
        double w = d2.getWidth();
        double h = d2.getHeight();
        g2.setPaint(foreColor);
        g2.fill(new Rectangle2D.Double(0.0, 0.0, w, h));
    }
    
    public void mouseReleased(final java.awt.event.MouseEvent p1) {
    }
    public void mouseEntered(final java.awt.event.MouseEvent p1) {
        setColor(MOUSE_IN_COLOR);
    }
    
    public void mouseClicked(final java.awt.event.MouseEvent p1) {
    }
    public void mousePressed(final java.awt.event.MouseEvent p1) {
    }
    public void mouseExited(final java.awt.event.MouseEvent p1) {
        setColor(MOUSE_OUT_COLOR);
    }
    
    public static void main(String[] args) {
        JFrame f = new JFrame("CloneUI Tester");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) { System.exit(0); }
        });
        JPanel p = new JPanel();
        p.setBackground(Color.white);
        p.setLayout(null);
        CloneUI ui = new CloneUI();
        ui.setColor(Color.blue);
        p.add(ui);
        ui.setBounds(50, 50, 1, 50);
        f.getContentPane().add(p, "Center");
        f.setSize(200, 200);
        f.setVisible(true);
    }
}