/*
 * Contig.java: This is a wrapper for contig
 *
 * Created on September 26, 2000, 2:51 PM
 */

package jadex.swing.sequence;

import jadex.swing.map.*;
import jade.db.AceObject;

/**
 * A wrapper class for Contig.
 * @author  wgm
 * @version 1.0
 */
public class Contig extends Object {

    private String name = null; // The name of the contig
    private MapRange extend = null; // The range of this contig in the Sequence_Map
    private MapRange pMap = null; // pMap: whatever it means. I need this to get the positions of clones
    private MapRange extendF = null; // The range of this contig in the Genetic_Map
    private String map = null; // The name of the map this contig in
    private AceObject obj = null; // The wrapped AceObject

    // These variables are used to do coordinate transfer: y = K * x + b.
    private float k = Float.MAX_VALUE;
    private float b = Float.MAX_VALUE;

    /** Creates new Contig */
    public Contig() {
    }

    /** Second constructor with name as the arguemnt */
    public Contig(String name) {
        setName(name);
    }

    /** Third constructor with name, map as the arguement */
    public Contig(String name, String map) {
        setName(name);
        setMap(map);
    }

    /** Fourth constructor with all properties as the arguments */
    public Contig(String name, String map, MapRange ext, MapRange pm) {
        this(name, map);
        setExtend(ext);
        setPMap(pm);
    }

    /** Transfer the PMap range to SequenceMap range.
    * @param p the PMap.
    * @return the MapRange in Sequence_map.
     */
    public MapRange getRangeFromPMap(MapRange p) {
        if (k == Float.MAX_VALUE) {
            calculateKB();
        }
        MapRange r = null;
        try {
            float start = k * p.getStart() + b;
            float end = k * p.getEnd() + b;
            r = new MapRange(start, end);
        }
        catch(UnknownPositionException e) {
            System.err.println("Contig.getRangeFromPMap(): " + e);
        }
        return r;
    }

    /** Try to get k, b values */
    private void calculateKB() {
        if ((extend == null) || (pMap == null)) // If either is null, cannot calculate
        throw new UnsupportedOperationException("Contig.calculate()");
        else {
            try {
                k = (extend.getStartF() - extend.getEndF()) / (pMap.getStartF() - pMap.getEndF());
                b = extend.getStartF() - pMap.getStartF() * k;
            }
            catch (UnknownPositionException e) {
                System.err.println("Contig.calculate: " + e);
            }
        }
    }

    /** Set the name of the contig.
    * @param name the name of the contig.
    */
public void setName(String name) { this.name = name; }

    /** Get the name of the contig.
    * @return the name of the contig.
    */
public String getName() { return this.name; }

    /** Set the name of the Sequence_map this contig is in.
    * @param map the name of the Sequence_map this contig is in.
    */
public void setMap(String map) { this.map = map; }

    /** Get the name of the Sequence_map this contig is in.
    * @return the name of the Sequence_map.
    */
public String getMap() { return this.map; }

    /** Set the MapRange if contig in Sequence_map. */
public void setExtend(MapRange r) { this.extend = r; }

public MapRange getExtend() { return this.extend; }

    /** Set the MapRange of contig in GeneticMap */
public void setExtendF(MapRange r) { this.extendF = r; }

public MapRange getExtendF() { return this.extendF; }

public void setPMap(MapRange r) { this.pMap = r; }

public MapRange getPMap() { return this.pMap; }

    /** Set the wrapped AceObject for this Contig.
    * @param o the wrapped AceObject.
    */
public void setAceObject(AceObject o) { this.obj = o; }

    /** Get the wrapped AceObject for this Contig.
    * @return the wrapped AceObject.
    */
public AceObject getAceObject() { return this.obj; }

    /** For pretty-printing purpose.
    */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        if (name != null) buf.append("Name: Contig : " + name + "\n");
        if (map != null) buf.append("Map: " + map + "\n");
        if (extend != null) buf.append("Extend: " + extend.toString() + "\n");
        if (extendF != null) buf.append("ExtendF: " + extendF.toString() + "\n");
        if (pMap != null) buf.append("PMap: " + pMap.toString() + "\n");
        if (obj != null) buf.append("Obj: " + obj + "\n");
        return buf.toString();
    }
}