/*
 * FileCloneFetcher.java
 *
 * Created on September 26, 2000, 11:29 AM
 */

package jadex.swing.sequence;

import java.io.*;
import java.util.*;

import jade.db.*;
import jadex.swing.map.MapRange;
/**
 * This subclass of CloneFetcher is used to get clone information from local file systems.
 * @author  wgm
 * @version 1.0
 */
public class FileCloneFetcher extends CloneFetcher {

    private String file = null; // The file containing the data for all clones
    /** Creates new FileCloneFetcher */
    public FileCloneFetcher() {
    }

    /** Second constructor.
    * @param file the name of file to be opened by this object.
     */
    public FileCloneFetcher(String file) {
        setFile(file);
    }

    /** Set the name of the file to be opened.
    * @param f the name of the file.
    */
    public void setFile(String f) {
        file = f;
    }

    /** Fetch all clone information into a String.
    * @return the string containing all clone information.
    */
    public String getCloneInfor() {
        StringBuffer buf = new StringBuffer();
        try {
            BufferedReader reader = null;
            try {
                reader = new BufferedReader(new FileReader(file));
            }
            catch(Exception e) { // Just for an appletInput
                InputStream in = getClass().getResourceAsStream(file);
                reader = new BufferedReader(new InputStreamReader(in));
            }
            String line = null;
            //int count = 1;
            while ((line = reader.readLine()) != null) {
                //System.out.println(count + " " + line);
                buf.append(line + "\n");
                //count ++;
            }
        }
        catch(IOException e) {
            System.err.println("FileCloneFetcher.getCloneInfor(): " + e);
        }
        return buf.toString();
    }

    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("Provide a file name");
            System.exit(0);
        }
        String file = args[0];
        FileCloneFetcher fetcher = new FileCloneFetcher(file);
        long start = System.currentTimeMillis();
        fetcher.fetchClone();
        long end = System.currentTimeMillis();
        System.out.println("Time used to processing the data: " + (end - start));
        Map map = fetcher.getClone();
        if (map != null) {
            Set keys = map.keySet();
            String name = null;
            for (Iterator iterator = keys.iterator(); iterator.hasNext(); ) {
                name = (String) iterator.next();
                Clone clone = (Clone) map.get(name);
                System.out.println(clone.toString());
            }
        }
        Map ctg = fetcher.getContig();
        if (ctg != null) System.out.println("Contig number: " + ctg.size());
        MapRange r = fetcher.getMapScale();
        if (r != null) System.out.println("Map extent: " + r.toString());
    }
}