/*
 * HorizontalMapScale.java: A horizonal mapscale to show bp position on the sequence map.
 * In the future, this class should be integrated into jadex.swing.map.MapScale1.
 *
 * Created on December 4, 2000, 9:55 AM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.beans.*;
import java.text.NumberFormat;

import javax.swing.*;

import jadex.swing.map.*;

/**
 * A horizonal mapscale to show bp position on the sequence map. In the future, 
 * this class should be integrated into jadex.swing.map.MapScale1.
 * @author  wgm
 * @version 1.0
 */
public class HorizontalMapScale extends JPanel implements PropertyChangeListener {

    private MapRange range = null; // The maprange to be displayed in this mapscale
                                   // This is a bean property.
    private long extend, start;
    private int TOTAL_TICK = 10;
    
    private Font labelFont = new Font("Dialog", 0, 9); // The font for number label
    
    // Note: The color model of this display will be determined by its parent JPanel

    // Layout parameters
    private float xPadding;
    
    /** Default constructor: create a horizontal mapscale */
    public HorizontalMapScale() {
        initialize();
    }

    /** Second constructor 
    * @param range the maprange of this mapscale 
    */
    public HorizontalMapScale(MapRange range) {
        initialize();
        setRange(range);
    }
    
    /** An inializing method */
    private void initialize() {
        xPadding = 10.0f;
        setBorder(BorderFactory.createEtchedBorder());
    }

    /** Set the property of range
     * @param range the maprange to be displayed in this scale
     */
    public void setRange(MapRange range) {
        this.range = range;
        try {
            start = range.getStart();
            extend = range.getExtend();
            repaint();
        }
        catch (UnknownPositionException e) {
            System.err.println("HorizontalMapScale.setMapRange(): " + e);
        }
    }

    /** ProertyChangeListener method
     * @param e the range change event
     */
    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName().equalsIgnoreCase("range")) {
            MapRange newRange = (MapRange) e.getNewValue();
            setRange(newRange);
        }
    }

    /** Get the currently displayed MapRange.
    * @return the currently displayed MapRange.
    */
    public MapRange getRange() {
        return this.range;
    }

    /** The most important method in this class. */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d = (Dimension2D) getSize();
        float w = (float) d.getWidth();
        float h = (float) d.getHeight();
        
        Insets insets = getInsets();

        g2.setPaint(getBackground());
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

        float startX = xPadding + insets.left;
        float endX = w - xPadding - insets.right;
        float y = h - h / 6.0f;
        
        g2.setPaint(getForeground());
        g2.draw(new Line2D.Float(startX, y, endX, y));

        float xPos = startX;
        float diff = (w - insets.left - insets.right - 2 * xPadding) / TOTAL_TICK;
        
        g2.setFont(labelFont);
        FontRenderContext fontContext = g2.getFontRenderContext();
        NumberFormat format = NumberFormat.getNumberInstance();
        for (int i = 0; i <= TOTAL_TICK; i ++) {
            g2.draw(new Line2D.Float(xPos, y, xPos, y - h / 3.0f));
            float sX = xPos;
            if (i < TOTAL_TICK) {
                for (int j = 0; j < 9; j++) {
                    sX += diff / 10;
                    g2.draw(new Line2D.Float(sX, y, sX, y - h / 12.0f));
                }
            }
            drawLabel(xPos, y - h / 3.0f, g2, i, fontContext, format);
            xPos += diff;
        }
    }

    /** A helper to paint the labels around the major marks */
    private void drawLabel(float posx, float posy, Graphics2D g2, int number, 
                                FontRenderContext fontContext, NumberFormat format) {
        long diff = extend / TOTAL_TICK;
        long label = start + number * diff;
        String labelS = format.format(label);
        Rectangle2D rect2 = labelFont.getStringBounds(labelS, fontContext);
        float x = posx - (float) rect2.getWidth() / 2.0f;
        g2.drawString(labelS, x, posy);
    }
    
    public static void main(String[] args) {
        JFrame f = new JFrame("MapScale Test");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        HorizontalMapScale p = new HorizontalMapScale();
        p.setPreferredSize(new Dimension(1204, 25));
        p.setRange(new MapRange(1, 1000000));
        JPanel blank = new JPanel();
        f.getContentPane().add(blank, "Center");
        f.getContentPane().add(p, "South");
        f.setSize(1204, 150);
        f.setVisible(true);
    }
}