/*
 * HorizontalMiniMapCartoon.java: A minimapcartoon to show the position of the displaying
 * region on the whole chromosome
 *
 * Created on December 10, 2000, 1:01 PM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.awt.geom.*;
import java.beans.*;
import javax.swing.*;

import jadex.swing.map.MapRange;
import jadex.swing.map.UnknownPositionException;

/**
 * A minimapcartoon to show the position of the displaying region on the whole 
 * chromosome.
 * @author  wgm
 * @version 1.0
 */
public class HorizontalMiniMapCartoon extends JPanel implements PropertyChangeListener {
    private MapRange range = null; // The current maprange
    private MapRange fullRange = null; // The full maprange for the chromosome
    
    // Color mode
    private Color fullColor = Color.black; // The color of the whole chromosome
    private Color displayColor = Color.green; // The color of the display region
    
    // dimension mode
    private float xPadding = 1.0f;
    private float yPadding = 1.0f;
    
    /** Creates new HorizontalMiniMapCartoon */
    public HorizontalMiniMapCartoon() {
        setBorder(BorderFactory.createEtchedBorder());
        setPreferredSize(new Dimension(50, 20));
    }

    /** Set the full maprange */
    public void setFullMapRange(MapRange r) {
        fullRange = r;
        range = fullRange;
        repaint();
    }

    /** Get the full maprange */
    public MapRange getFullMapRange() {
        return fullRange;
    }

    /** Set the current maprange */
    public void setMapRange(MapRange r) {
        range = controlRange(r);
        repaint();
    }

    /** Get the current maprange */
    public MapRange getMapRange() {
        return this.range;
    }

    /** A checker to ensure that the current maprange is within the fullmaprange */
    private MapRange controlRange(MapRange r) {
        MapRange r1 = null;
        // Clone the passed maprange to make sure the original data is still there
        try {
            long start = r.getStart();
            long end = r.getEnd();
            if (start < fullRange.getStart()) start = fullRange.getStart();
            if (end > fullRange.getEnd()) end = fullRange.getEnd();
            r1 = new MapRange(start, end);
        }
        catch(UnknownPositionException e) {
            System.err.println("HorizontalMiniMapCartoon.controlRange(): " + e);
        }
        return r;
    }

    public void propertyChange(final java.beans.PropertyChangeEvent p1) {
        if (p1.getPropertyName().equalsIgnoreCase("range")) {
            MapRange r = (MapRange) p1.getNewValue();
            if ((range != null) && r.equals(range)) return;
            else {
                range = r;
                repaint();
            }
        }
    }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        // Clear up the background
        Dimension2D d2 = (Dimension2D) getSize();
        float w = (float) d2.getWidth();
        float h = (float) d2.getHeight();
        g2.setPaint(getBackground());
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));
        // calculate all converting factors
        Insets insets = getInsets();
        float width = w - 2 * xPadding - insets.left - insets.right;
        float height = h / 3.0f;
        float rel = 0.0f;
        if (fullRange == null) return;
        try {
            long extend = fullRange.getExtend();
            rel = width / extend; // x / bp for converting
        }
        catch(UnknownPositionException e) {
            System.err.println("HorizonalMiniMapCartoon.paint(): " + e);
        }
        
        // Paint the whole chromosome
        g2.setPaint(fullColor);
        float x = insets.left + xPadding;
        float y = h / 3.0f;
        g2.fill(new Rectangle2D.Float(x, y, width, height));
        
        // Paint the current maprange
        g2.setPaint(displayColor);
        float x1 = 0.0f, x2 = 0.0f, w1 = 0.0f;
        try {
            x1 = rel * (range.getStart() - 1); // Offset is 1
            x2 = rel * (range.getEnd() - 1);
            w1 = x2 - x1;
            if (w1 < 1.0f) w1 = 1.0f;
        }
        catch(UnknownPositionException e) {
            System.err.println("HorizontalMiniMapCartoon.paintCurrentRange(): " + e);
        }
        g2.fill(new Rectangle2D.Float(x + x1, y, w1, height));
    }
}