/*
 * SequenceDisplayAndTreeAdapter.java: An adapter to connect the selections of SequenceDisplay
 * and SequenceTreeDisplay
 *
 * Created on December 5, 2000, 9:41 PM
 */

package jadex.swing.sequence;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.*;

import org.jdom.*;
import jadex.xml.XMLUtilities;
import jadex.swing.map.MapRange;

/**
 * An adapter to connect the selections of SequenceDisplay and SequenceTreeDisplay.
 * @author  wgm
 * @version 1.0
 */
public class SequenceDisplayAndTreeAdapter implements TreeSelectionListener, SequenceSelectionListener {

    private SequenceDisplay display = null; // One end: the Sequence display
    private JTree tree = null; // Another end: the JTree for sequence
    private TreePath rootPath = null; // The root of JTree from TreeDisplay
    
    private boolean isDone = false; // A flag to prevent the event cycle
    
    /** Constructor
    * @param display the SequenceDisplay to be conntected.
    * @param treeDisplay the SequenceTreeDisplay to be connected.
    */
    public SequenceDisplayAndTreeAdapter(SequenceDisplay display, SequenceTreeDisplay treeDisplay) {
        this.display = display;
        tree = treeDisplay.getTree();
        rootPath = tree.getPathForRow(0);
        
        // Create the two links
        display.getSelectionModel().addSelectionListener(this);
        tree.addTreeSelectionListener(this);
    }

    /** EventHandler for event fired by JTree in SequenceTreeDisplay
     */
    public void valueChanged(TreeSelectionEvent e) { 
        if (isDone) {
            isDone = false;
            return;
        }
        isDone = true;
        TreePath[] paths = tree.getSelectionPaths();
        if ((paths == null) || (paths.length == 0)) {
            display.clearSelection();
            return ;
        }
        SequenceTreeNode node = null;
        Element seq = null;
        java.util.List sel = new ArrayList();
        MapRange range = null;
        for (int i = 0; i < paths.length; i++) {
            node = (SequenceTreeNode) paths[i].getLastPathComponent();
            seq = node.getSequence();
            range = XMLUtilities.getMapRange(seq, seq.getNamespace()).combine(range);
            sel.add(seq);
        }      
        display.setSelectedSequences(sel);
        display.setMapRange(range);
    }

    /** EventHandler for event fired by SequenceSelectionModel from SequenceDisplay
     */
    public void valueChanged(SequenceSelectionEvent e) {
        if (isDone) {
            isDone = false;
            return;
        }
        isDone = true;
        Object source = e.getSource();
        if (source instanceof SequenceSelectionModel) {          
            SequenceSelectionModel sequenceSelectionModel = (SequenceSelectionModel) e.getSource();
            java.util.List list = sequenceSelectionModel.getSelection();
            DefaultTreeModel treeModel = (DefaultTreeModel) tree.getModel();
            int size = list.size();
            TreePath[] paths = new TreePath[size];
            for (int i = 0; i < size; i++) {
                Element seq = (Element) list.get(i);
                paths[i] = constructPath(treeModel.getPathToRoot(new SequenceTreeNode(seq)));
                //if (!tree.isPathSelected(paths[i])) tree.addSelectionPath(paths[i]);
                if (i == (size - 1)) tree.scrollPathToVisible(paths[i]);
            }         
            tree.setSelectionPaths(paths);  
        }
    }
 
    private TreePath constructPath(TreeNode nodes[]) {
        TreePath rtnPath = rootPath;
        for (int i = 1; i < nodes.length; i++) {
            rtnPath = rtnPath.pathByAddingChild(nodes[i]);
        }
        return rtnPath;
    }
    
    /** Clear up */
    public void clear() {
        display.getSelectionModel().removeSelectionListener(this);
        tree.removeTreeSelectionListener(this);
        // Marked for gc
        display = null;
        rootPath = null;
        tree = null;
    }
}