/*
 * SequenceElementTableView.java
 *
 * Created on December 13, 2000, 3:21 PM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

import org.jdom.*;
/**
 * This customized JPanel is used to display some more information for individual
 * sequence element.
 * @author  wgm
 * @version 1.0
 */
public class SequenceElementTableView extends JPanel implements SequenceElementView {

    private JTable table = null; // The table for this view
    
    /** Creates new SequenceElementTableView */
    public SequenceElementTableView() {
        initialize();
    }

    /** Second constructor
     * @param sequence the sequence to be displayed
     */
    public SequenceElementTableView(Element sequence) {
        initialize();
        setSequence(sequence);
    }

    /** Initialization stuff */
    private void initialize() {
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(new BorderLayout());
    }

    /** Get this SequenceView. This class is an implementation of SequenceView.
    * @return this SequenceView.
    */
    public javax.swing.JComponent getSequenceView() {
        return this;
    }
    
    /** Set the Sequence elmenet to be displayed.
    * @param sequence the sequence element to be displayed.
    */
    public void setSequence(org.jdom.Element sequence) {
        SequenceTableModel tableModel = new SequenceTableModel(sequence);
        table = new JTable();
        table.setRowHeight(225); // Randomly set a enough high row height
        SequenceCellRenderer renderer = new SequenceCellRenderer();
        try {
            table.setDefaultRenderer(Class.forName("org.jdom.Element"), renderer);
            table.setDefaultEditor(Class.forName("org.jdom.Element"), renderer);
        }
        catch(ClassNotFoundException e) {
            System.err.println("SequenceElementTableView.setSequence(): " + e);
        }
        table.setModel(tableModel);
        add(new JScrollPane(table), BorderLayout.CENTER); // It is important to add the table in a scrollpane to show header?
        
        // Add a JButton to the south to link to worbase for more ingformation
        //addJButtonForMore(sequence.getAttributeValue("name"));
    }
    
    /** A refactored method to add a button */
    private void addJButtonForMore(final String name) {
        JButton button = new JButton("For more information, click this...");
        button.setToolTipText("Link to Wormbase in a new browser window");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                IndividualSequenceView.showSequenceInDetail(name);
            }
        });
        JPanel p = new JPanel();
        p.add(button);
        add(p, BorderLayout.SOUTH);
    }
    
    /** Expose the table. 
    * @return the table for displaying individual Sequence view.
    */
    public JTable getTable() {
        return this.table;
    }

    /** A customized table model to show the informatin of sequence element */
    class SequenceTableModel extends AbstractTableModel {
        Element range = null;
        Element gene = null;
        Element remark = null;

        SequenceTableModel(Element sequence) {
            parseSequence(sequence);
        }

        /** Parse sequence element */
        private void parseSequence(Element sequence) {
            range = getCopy(sequence, "Range");
            
            gene = getCopy(sequence, "Gene");
            if (gene != null) remark = gene.getChild("Remark", gene.getNamespace());
            sequence = null; // Marked for gc
        }

        private Element getCopy(Element seq, String name) {
            Element elm = seq.getChild(name, seq.getNamespace());
            if (elm == null) return null;
            return elm.getCopy(name, seq.getNamespace());
        }
        /** There is only one row to be displayed in this table */
        public int getRowCount() {
            return 1;
        }

        public int getColumnCount() {
            if (gene == null) return 1;
            if (remark == null) return 2;
            return 3;
        }

        public java.lang.Object getValueAt(int p1,int p2) {
            Object rtn = null;
            switch (p2) {
                case 0 : {
                    rtn = range;
                    break;
                }
                case 1 : {
                    rtn = gene.getChild("Exons", gene.getNamespace());
                    break;
                }
                case 2 : {
                    rtn = remark;
                    break;
                }
            }
            return rtn;
        }

        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        public java.lang.String getColumnName(int p1) {
            String name = null;
            switch (p1) {
            case 0 : { name = "Position"; break; }
            case 1 : { name = "Exons"; break; }
            case 2 : { name = "Remark"; break; }
            }
            return name;
        }
        
        public boolean isCellEditable(int row, int column) {
            return true;
        }
    }

    /** A customized cell render for sequnece table */
    class SequenceCellRenderer implements TableCellRenderer, TableCellEditor {
        JComponent rangeView, exonView, remarkView;

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            JComponent comp = null;
            Element elm = (Element) value;
            String elmName = elm.getName();
            if (elmName.equals("Range")) {
                comp = getRangeView(elm);
            }
            else if (elmName.equals("Exons")) {
                comp = getExonView(elm);
            }
            else if (elmName.equals("Remark")) {
                comp = getRemarkView(elm);
            }
            else comp = new JLabel(value.toString());
            return comp;
        }

        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
            return getTableCellRendererComponent(table, value, isSelected, true, row, column);
        }

        /** Get the position table */
        private JComponent getRangeView(Element range) {
            if (rangeView == null) {
            String[] columnName = {"Start", "End", "Length"};
                long[] pos = new long[2];
                pos[0] = Long.parseLong(range.getAttributeValue("start"));
                pos[1] = Long.parseLong(range.getAttributeValue("end"));
                long len = pos[1] - pos[0];
                if (len > 0) len ++;
                else len --;
                Object[][] data = {{new Long(pos[0]), new Long(pos[1]), new Long(len)}};
                JTable rangeTable = new JTable(data, columnName);
                rangeView = new JScrollPane(rangeTable);
            }
            return rangeView;
        }

        /** Get the exon table */
        private JComponent getExonView(Element exons) {
            if (exonView == null) {
            String[] columnName = {"Exon#", "Start", "End", "Length"};
                java.util.List list = exons.getChildren("Exon", exons.getNamespace());
                int size = list.size();
                Object[][] data = new Object[size][columnName.length];
                Element exon = null;
                for (int i = 0; i < size; i++) {
                    exon = (Element) list.get(i);
                    data[i][0] = (i + 1) + "";
                    data[i][1] = exon.getAttributeValue("start");
                    data[i][2] = exon.getAttributeValue("end");
                    data[i][3] = exon.getAttributeValue("length");
                }
                JTable table = new JTable(data, columnName);
                exonView = new JScrollPane(table);
            }
            return exonView;
        }

        /** Get the remark view */
        private JComponent getRemarkView(Element remark) {
            if (remarkView == null) {
                JTextArea tf = new JTextArea();
                tf.setLineWrap(true);
                tf.setWrapStyleWord(true);
                tf.setText(remark.getTextTrim());
                remarkView = new JScrollPane(tf);
            }
            return remarkView;
        }
       
        public void cancelCellEditing() {
        }
        public boolean stopCellEditing() {
            return true;
        }
        public boolean shouldSelectCell(final java.util.EventObject p1) {
            return true;
        }
        public java.lang.Object getCellEditorValue() {
            return exonView;
        }
        public boolean isCellEditable(final java.util.EventObject p1) {
            return true;
        }
        public void addCellEditorListener(final javax.swing.event.CellEditorListener p1) {
        }
        public void removeCellEditorListener(final javax.swing.event.CellEditorListener p1) {
        }
    }
}