/*
 * SequenceLayoutModel.java: This class is used to transform bp coordinate on the chromosome
 * to the graphics coordinate, and the zoom and some layout processing.
 *
 * Created on November 25, 2000, 4:30 PM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import org.jdom.*;

import jadex.xml.XMLUtilities;
import jadex.swing.map.*;

/**
 * This class is used to transform bp coordinate on the chromosome to the graphics 
 * coordinate, and the zoom and some layout processing.
 * @author  wgm
 * @version 1.0
 */
public class SequenceLayoutModel extends Object {

    private SequenceDisplay display = null; // The display this layoutmodel works for

    // Layout settings
    protected float xPadding = 10.0f;
    protected float yPadding = 10.0f;
    protected float distance = 30.0f; // The distance between the sequence layers
    protected float height = 15.0f; // The height of the sequence renderer
    // These variabels are for coordinate switch
    private double rel; // The ratio between x on graphics and bp on chromsome (x / bp)
    private long offset; // The start bp position on cheomsome on the current graphics system
    
    private int currentH; // The current height of the display
    private int currentW; // The current width of the display
    private MapRange range = null; // The maprange currently displayed.
    protected final int ZOOM_STEP = 4; // The step of zooming

    // The current mouse position converted to bp
    private long mouseAt; 
    
    protected int totalLayers = 6; // The total layer number: This is a defautl value if it is not enough
    // the layer will be added one by one.
    protected int IS_UP = 0; // upper half of the box
    protected int IS_LOW = 1; // lower half of the box
    protected long totalBP; // The total bp length of the chromosome or sequenceS
    protected java.util.List boxes = null; // The total rectangles for the layers

    // This comparator is for sorting by left end of the sequence elements
    private Comparator leftSortComp = new Comparator() {
        public int compare(Object o1, Object o2) {
            Element seq1 = (Element) o1;
            Element seq2 = (Element) o2;
            MapRange range1 = XMLUtilities.getMapRange(seq1, seq1.getNamespace());
            MapRange range2 = XMLUtilities.getMapRange(seq2, seq2.getNamespace());
            long left1 = range1.getLeftEnd();
            long left2 = range2.getLeftEnd();
            int rtn = 0;
            if (left1 > left2) rtn = 1;
            else if (left1 < left2) rtn = -1;
            return rtn;
        }
    };

    // This comparator if for sortin by right end of the sequence elements
    private Comparator rightSortComp = new Comparator() {
        public int compare(Object o1, Object o2) {
            Element seq1 = (Element) o1;
            Element seq2 = (Element) o2;
            MapRange range1 = XMLUtilities.getMapRange(seq1, seq1.getNamespace());
            MapRange range2 = XMLUtilities.getMapRange(seq2, seq2.getNamespace());
            long right1 = range1.getRightEnd();
            long right2 = range2.getRightEnd();
            int rtn = 0;
            if (right1 > right2) rtn = 1;
            else if (right1 < right2) rtn = -1;
            return rtn;
        }
    };

    /** Creates new SequenceLayoutCalculator */
    public SequenceLayoutModel() {
    }

    /** Second constructor with display as the argument
     * @param display the container all sequence element should be drawed on
     */
    public SequenceLayoutModel(SequenceDisplay display) {
        this.display = display;
    }

    /** Third constructor with a layer as a argument
     * @param layer the total layer number
     * @param display the container all sequence element should be drawn on
     */

    public SequenceLayoutModel(int layer, SequenceDisplay display) {
        this.totalLayers = layer;
        this.display = display;
    }

    /** Do some initialization layout calculation */
    protected void relayout() {
        if (range == null) return; // If no range specified, do nothins and just return.

        Insets insets = display.getInsets();
        int w1 = display.getWidth();
        float w = w1 - 2 * xPadding - insets.left - insets.right;
        int h1 = display.getHeight();
        if ((h1 != currentH) || (w1 != currentW)) { // if there is a change of the height of diplal, reinitilize of the boxes
            currentW = w1;
            currentH = h1;
            float y = h1 - yPadding - insets.bottom;
            float x = xPadding + insets.left;
            if (boxes == null) { // First time, create a list
                boxes = new ArrayList();
            }
            else { // Not the first time
                totalLayers = boxes.size();
                boxes.clear();
            }
            for (int i = 0; i < totalLayers; i++) {
                float rectH = 2.0f * height;
                if (i == 0) rectH = height;
                y -= rectH;
                // Add a defautl number
                boxes.add(new Rectangle.Float(x, y, w, rectH));
                y = y - distance;
            }
        }

        try {
            long extend = range.getExtend();
            offset = range.getStart();
            rel = w / extend;
        }
        catch(UnknownPositionException e) {
            System.out.println("SequenceLayoutModel.relayout(): " + e);
        }
    }

    /** By setting total bp of chromosome, the rel and offset should be calculated.
    * @param total the total bp of the displayed chromosome.
    */
    public void setTotalBP(long total) {
        totalBP = total;
        setMapRange(new MapRange(1, total));
    }

    /** Set the displayed MapRange.
    * @param range the MapRange to be displayed.
    */
    public void setMapRange(MapRange range) {
        //System.out.println("New maprange: " + range.toString());
        range = checkMapRange(range);
        if (this.range != null) {
            if (range.equals(this.range)) return;
        }
        MapRange oldValue = this.range;
        this.range = range;
        display.repaint();
        display.firePropertyChange("range", oldValue, range);
    }

    /** Check Maprange to see its direction. If direction is negative, change 
     *to positive for display.
    * @param range the MapRange to be checked.
    * @return the range that has bas checked.
    */
    private MapRange checkMapRange(MapRange range) {
        try {
            if (range.getStart() <= range.getEnd()) return range; // Normal
            else {
                MapRange rtn = new MapRange(range.getEnd(), range.getStart());
                return rtn;
            }
        }
        catch(UnknownPositionException e) {
            System.err.println("SequenceLayoutModel.checkMapRange(): " + e);
            return range;
        }
    }
    
    /** Get the currently displayed MapRange.
    * @return the displayed MapRange.
    */
    public MapRange getMapRange() {
        return this.range;
    }
    /** Calculate the positon of sequence element based on its positions on the chromosome
     * @param sequence the calculated sequence element, should containing range information
     * @param layer which layer from the root this element is
     * @param isOdd check is the order number of the sequence is odd or even
     * @return the rectangle of the the sequence element in the graphics coordinate
     */
    public Rectangle getBounds(Element sequence, int layer, boolean isOdd) {
        if (sequence == null) return null;
        long[] pos = XMLUtilities.getPositions(sequence, display.getModel().getNamespace());
        float startX = fromBpToX(pos[0]);
        float endX = fromBpToX(pos[1]);
        if (!checkIn(startX, endX)) return null;
        float x;
        if (startX > endX) {
            x = startX;
            startX = endX;
            endX = x;
        }
        if (startX < -1.0f) startX = -1.0f; // Keep a little off to show that there are more
        if (endX > (display.getWidth() + 2)) endX = display.getWidth() + 2;
        
        Rectangle2D.Float rect1 = null;
        try {
            rect1 = (Rectangle2D.Float) boxes.get(layer);
        }
        catch (IndexOutOfBoundsException e) { // A new rectangle should be created
            rect1 = createNewLayer(layer);
        }
        float y = (float) rect1.getY();
        if (isOdd) y += height;
        return new Rectangle((int)startX, (int)y, (int)(endX - startX), (int)height);
    }

    /** Calculte a bounds for a sequence elemement containing a gene
     * @param sequence a sequence element with a gene element
     * @param layer the layer number, should be on the top
     * @isOdd check for lower or upper part
     * @ return a rectange including the bounds for the sequence and its gene
     */
    public Rectangle getSequenceAndGeneBounds(Element sequence, int layer, boolean isOdd) {
        Rectangle rect = getBounds(sequence, layer, isOdd);
        rect.y = rect.y - (int) distance - (int) height;
        rect.height = rect.height + (int) distance + (int) height;
        return rect;
    }

    /** Get a gene bounds from it parent sequence's bounds
     * @param rect the sequence, the gene's parent, bounds
     * @return the gene bounds
     */
    public Rectangle getGeneBounds(Rectangle rect) {
        Rectangle geneRect = new Rectangle();
        geneRect.x = rect.x;
        geneRect.y = rect.y - (int)distance - (int)height;
        geneRect.width = rect.width;
        geneRect.height = rect.height;
        return geneRect;
    }

    /** A helper to create a new box whenenver the default boxes are not enough */
    private Rectangle2D.Float createNewLayer(int layer) {
        Rectangle2D.Float rect0 = (Rectangle2D.Float) boxes.get(0);
        float x = rect0.x;
        float y = rect0.y - (distance * layer) - (height * 2 * layer);
        Rectangle2D.Float newRect = new Rectangle2D.Float(x, y, rect0.width, height * 2);
        boxes.add(newRect);
        return newRect;
    }

    /** Transform from the bp on chromsome to the graphics coordinate system */
    protected float fromBpToX(long bp) {
        return (float) (xPadding + rel * (bp - offset));
    }
    /** The opposite method of the above method */
    protected long fromXtoBp(float x) {
        return offset + (long) ((x - xPadding) / rel);
    }

    /** Check startX and endX is in the current visible area */
    private boolean checkIn(float startX, float endX) {
        Insets insets = display.getInsets();
        float x0 = xPadding + insets.left;
        if ((startX < x0) && (endX < x0)) return false;
        int w1 = display.getWidth();
        float xm = w1 - insets.right - xPadding;
        if ((startX > xm) && (endX > xm)) return false;
        return true;
    }

    /** Get the sequence element by a point coordinate on the graphics
     * @param x x-coordinate
     * @param y y-coordinate
    & @return Element corresponding to the point (x, y)
     */
    public Element getClosestElement(int x, int y) {
        // First check the point is in a box
        int box[] = getBoxFromY(x, y);
        if (box[0] == -1) return null;
        if (box[0] == 0) return display.getModel().getRoot(); // Root is always special
        // Get the absolute basepair position on the chromosome
        long bp = fromXtoBp(x);
        //System.out.println("SequeceLayoutModel.getClosestElement()::x: " + bp);
        return searchElement(box, bp);
    }

    /** Get a list of sequence elements in a rectangele
     * @param x1 the left end
     * @param x2 the right end
     * @param box the number of box to be checked
     * @return sequence elements in the rect
     */
    protected java.util.List getElementsBetween(Rectangle rect) {
        java.util.List list1 = null;
        int box = getBoxAboveY((float) rect.getMaxY());
        Element root = display.getModel().getRoot();
        if (box == 0) {
            list1 = new ArrayList();
            list1.add(root);
            return list1;
        }
        
        long bp1 = fromXtoBp(rect.x);
        if (bp1 <= 0) bp1 = 1;
        else if (bp1 > totalBP) bp1 = totalBP;
        long bp2 = fromXtoBp((float) rect.getMaxX());
        if (bp2 <= 0) bp2 = 1;
        else if (bp2 > totalBP) bp2 = totalBP;
        System.out.println("bp1 " + bp1 + " bp2 " + bp2);
        Element leftEnd = null;
        Element rightEnd = null;
        list1 = XMLUtilities.getSubSequences(root, root.getNamespace());
        for (int i = 1; i <= box; i++) {
            leftEnd = getElementLeftTo(bp1, list1);
            rightEnd = getElementRightTo(bp2, list1);
            list1 = getSubListElements(list1, leftEnd, rightEnd);
        }
        return list1;
    }
 
    /** Get a sublist betweem two element and combine all their subsubsequence into a new list */
    private java.util.List getSubListElements(java.util.List list, Element left, Element right) {
        int index1, index2;
        if (left == null) index1 = 0;
        else index1 = list.indexOf(left);
        if (right == null) index2 = list.size() - 1;
        else index2 = list.indexOf(right);
        
        java.util.List list1 = list.subList(index1, index2 + 1);
        java.util.List total = new ArrayList();
        java.util.List l = null;
        Element elm = null;
        for (Iterator it = list1.iterator(); it.hasNext(); ) {
            elm = (Element) it.next();
            l = XMLUtilities.getSubSequences(elm, elm.getNamespace());
            if (l != null) total.addAll(l);
        }
        return total;
    }
    
     /** This search is to finding the first element its left end not lower than bp
     * @param bp the bp position to be checked
     * @parent the parent element which subsequences will be searched
     * @return the first element which left end is higher than bp
     */
    private Element getElementRightTo(long bp, java.util.List list) {
        Collections.sort(list, leftSortComp);
        return searchRightMostElement(list, bp, 0, list.size());
    }

    /** A binary recursive search */
    private Element searchRightMostElement(java.util.List list, long bp, int start, int end) {
        if (start == end) return null; // If start and end collide, search has be dead.

        int index = (start + end) / 2;
        MapRange range1 = null;
        Element sequence1 = null;
        long left1;
        MapRange range2 = null;
        Element sequence2 = null;
        long left2;
        Element found = null;
        sequence1 = (Element) list.get(index);
        range1 = XMLUtilities.getMapRange(sequence1, sequence1.getNamespace());
        left1 = range.getLeftEnd();
        // The first element has be hit
        if (index == 0) {
            if (left1 >= bp) { // It is OK in this case, if the left end of the first element is equal to bp
                return sequence1;
            }
            else return null; // cannot find a sequence element
        }

        sequence2 = (Element) list.get(index - 1);
        range2 = XMLUtilities.getMapRange(sequence2, sequence2.getNamespace());
        left2 = range.getLeftEnd();

        if ((bp <= left1) && (bp > left2)) {
            return sequence1;
        }
        else {
            if (bp <= left1) { // move to the right side
                return searchRightMostElement(list, bp, index + 1, end);
            }
            else {
                return searchRightMostElement(list, bp, start, index);
            }
        }
    }

    /** This search is to finding the first element its right end not higher than bp
     * @param bp the bp position to be checked
     * @parent the parent element which subsequences will be searched
     * @return the first element which right end is not higher than bp
     */
    private Element getElementLeftTo(long bp, java.util.List list) {
        Collections.sort(list, rightSortComp);
        return searchLeftMostElement(list, bp, 0, list.size());
    }

    /** A binary recursive search */
    private Element searchLeftMostElement(java.util.List list, long bp, int start, int end) {
        if (start == end) return null; // If start and end collide, search has be dead.

        int index = (start + end) / 2;
        MapRange range1 = null;
        Element sequence1 = null;
        long right1;
        MapRange range2 = null;
        Element sequence2 = null;
        long right2;
        Element found = null;

        sequence1 = (Element) list.get(index);
        range1 = XMLUtilities.getMapRange(sequence1, sequence1.getNamespace());
        right1 = range.getRightEnd();
        // The first element has be hit
        if (index == (list.size() - 1)) {
            if (right1 <= bp) { // It is OK in this case, if the left end of the first element is equal to bp
                return sequence1;
            }
            else return null; // cannot find a sequence element
        }

        sequence2 = (Element) list.get(index + 1);
        range2 = XMLUtilities.getMapRange(sequence2, sequence2.getNamespace());
        right2 = range.getRightEnd();

        if ((bp >= right1) && (bp < right2)) {
            return sequence1;
        }
        else {
            if (bp >= right1) { // move to the right side
                return searchLeftMostElement(list, bp, start, index);
            }
            else {
                return searchLeftMostElement(list, bp, index + 1, end);
            }
        }
    }

    /**  A helper to get the box number for a point
     * @param y the y coordinate of the point
     * @return a two element array: int[0] the box number int[1] is 0 or 1, 0 for the upper half
     * and 1 for the lewer half. int[0] is -1 is no box can be found.
     */
    protected int[] getBoxFromY(float x, float y) {
        // Since there are less than ten boxes, do a linear search.
        int[] rtn = new int[2];
        rtn[0] = -1;
        for (Iterator i = boxes.iterator(); i.hasNext(); ) {
            Rectangle2D.Float rect = (Rectangle2D.Float) i.next();
            if (rect.contains(x, y)) {
                rtn[0] = boxes.indexOf(rect);
                if ((y - rect.getY()) < height) rtn[1] = IS_UP;
                else rtn[1] = IS_LOW;
                break;
            }
        }
        return rtn;
    }

    /** Get the box just above or containing a given y
     * @param y the y coordinate
     * @return the box number
     */
    protected int getBoxAboveY(float y) {
        int rtn = -1;
        for (Iterator it = boxes.iterator(); it.hasNext();) {
            Rectangle2D.Float rect = (Rectangle2D.Float) it.next();
            if (rect.y <= y) {
                rtn = boxes.indexOf(rect);
                break;
            }
        }
        return rtn;
    }
    
    /** Get the box just below or containing a given y
     * @param y the y coordinate
     * @return the box number
     */
    protected int getBoxBelowY(float y) {
        int rtn = -1; // If no box found, just return -1.
        for (int i = boxes.size() - 1; i >= 0; i--) {
            Rectangle2D.Float rect = (Rectangle2D.Float) boxes.get(i);
            if (rect.getMaxY() >= y) {
                rtn = i;
                break;
            }
        }
        return rtn;
    }

    /** Find an element based on box number and base pain positin on the chromosome.
     * @param box the box number
     * @param bp the base pair position on the chromosome
     * @return the element
     */
    protected Element searchElement(int[] box, long bp) {
        Element root = display.getModel().getRoot();
        //Namespace ns = display.getModel().getNamespace();
        Namespace ns = root.getNamespace();
        Element sequence = root;
        for (int i = 1; i <= box[0]; i++) {
            if (i == box[0]) sequence = doSearchElement(sequence, ns, bp, box[1]);
            else sequence = doSearchElement(sequence, ns, bp);
        }
        return sequence;
    }


    private Element doSearchElement(Element sequence, Namespace ns, long bp) {
        if (sequence == null) return null;
        java.util.List list = XMLUtilities.getSubSequences(sequence, ns);
        if ((list == null) || (list.size() == 0)) return null;
        int end = list.size();
        return checkElement(list, bp, ns, 0, end);
    }

    private Element doSearchElement(Element sequence, Namespace ns, long bp, int upOrLow) {
        if (sequence == null) return null;
        java.util.List list = XMLUtilities.getSubSequences(sequence, ns);
        if ((list == null) || (list.size() == 0)) return null;
        int end = list.size();
        Element elm = checkElement(list, bp, ns, 0, end);
        if (elm == null) return null;
        int index = list.indexOf(elm);
        if (((index % 2) == 0) && (upOrLow == IS_UP)) return elm;
        else if(((index % 2) != 0) && (upOrLow == IS_LOW)) return elm;
        else return checkNeighbor(list, index, bp, ns);
    }

    /** If a sequence has been found, but not at the layer of the box. Check the two
     * adjacent elements to see since there are some overlapping on the sequences
     */
    private Element checkNeighbor(java.util.List list, int index, long bp, Namespace ns) {
        // Check left neighbor
        if (index > 0) {
            Element left = (Element) list.get(index - 1);
            if (containsBp(left, bp, ns)) return left;
        }
        // Check right neighbor
        if (index < (list.size() - 1)) {
            Element right = (Element) list.get(index + 1);
            if (containsBp(right, bp, ns)) return right;
        }
        return null;
    }

    private boolean containsBp(Element seq, long bp, Namespace ns) {
        MapRange range = XMLUtilities.getMapRange(seq, ns);
        return range.contain(bp);
    }

    /** Do a binary search to get the element containing the specific bp position
     * @param list the sequence list, should be sorted.
     * @param bp the bp position
     * @param ns the namespace of the element
     * @param start the start index of the comparing range
     * @param end the end index of the comparing range
     * @return the element contain the bp at a specific layer
     */
    private Element checkElement(java.util.List list, long bp, Namespace ns, int start, int end) {
        if (end == start) {
            // Sometimes, the previous one will larger than the next one, or vice versa.
            return checkBefore(list, bp, ns, start);
            //return null; // If two indices collapse, no element can be found!
        }
        int index = (end + start) / 2;
        Element seq = (Element) list.get(index);
        //System.out.println("SequenceLayoutModel.checkElement(): index " + index + " Element checking " + seq.getAttributeValue("name"));
        long[] pos = XMLUtilities.getPositions(seq, ns);
        if (pos[0] < pos[1]) { // positive direction
            if (bp < pos[0]) return checkElement(list, bp, ns, start, index);
            else if (bp > pos[1]) return checkElement(list, bp, ns, index + 1, end);
            //System.out.println("SequenceLayoutModel.checkElement(): Seq " + XMLUtilities.getName(seq));
            return seq;
        }
        else { // Negative direction
            if (bp < pos[1]) return checkElement(list, bp, ns, start, index);
            else if (bp > pos[0]) return checkElement(list, bp, ns, index + 1, end);
            return seq;
        }
    }

    /** A method to check the previous element to see if there is overlapping. */
    private Element checkBefore(java.util.List list, long bp, Namespace ns, int start) {
        if (start < 1)  return null; // The first one already, do nothing.
        Element pre = (Element) list.get(start - 2);
        long[] pos = XMLUtilities.getPositions(pre, ns);
        if (pos[0] < pos[1]) {
            if ((bp >= pos[0]) && (bp <= pos[1])) return pre;
            else return null;
        }
        else {
            if ((bp >= pos[1]) && (bp <= pos[0])) return pre;
            else return null;
        }
    }
    
    /** Do zooming-in in the current range */
    public void zoomIn() {
        MapRange newValue = range.zoomIn(ZOOM_STEP);
        setMapRange(newValue);
    }

    /** Do zooming-out in the current range */
    public void zoomOut() {
        MapRange newValue = checkRange(range.zoomOut(ZOOM_STEP));
        setMapRange(newValue);
    }

    /** Moving the current range to the left */
    public void moveLeft() {
        MapRange newValue = checkRange(range.moveDown(ZOOM_STEP));
        setMapRange(newValue);
    }

    /** Moving the current range to the right */
    public void moveRight() {
        MapRange newValue = checkRange(range.moveUp(ZOOM_STEP));
        setMapRange(newValue);
    }

    /** Reset the maprange to the whole chromosome */
    public void reset() {
        setMapRange(new MapRange(1, totalBP));
    }

    /** Zoom in a specific rectangle
     */
    public void zoom(Rectangle rect) {
        int x1 = rect.x;
        int x2 = rect.x + rect.width;
        long start = fromXtoBp(x1);
        long end = fromXtoBp(x2);
        setMapRange(new MapRange(start, end));
    }

    /** Check the new range to see if it is overflowing */
    private MapRange checkRange(MapRange range) {
        try {
            long start = range.getStart();
            if (start < 1) range.setStart(1);
            long end = range.getEnd();
            if (end > totalBP) range.setEnd(totalBP);
        }
        catch(UnknownPositionException e) {
            System.err.println("SequenceLayoutModel.checkRange(): " + e);
        }
        return range;
    }
    
    /** Set current mouse position in bp unit */
    public void setMouseAt(long p) {
        if (mouseAt == p) return; // Do nothing for the same value
        if (p < 1) p = 1; // Sanity checking
        if (p > totalBP) p = totalBP;
        long tmp = mouseAt;
        mouseAt = p;
        display.firePropertyChange("mousePosition", new Long(tmp), new Long(mouseAt));
    }
    
    /** Get the current mouse position in bp unit */
    public long getMouseAt() {
        return mouseAt;
    }
    
    /** An convient method to output a sequence element list */
    protected void outputSequenceList(java.util.List list) {
        Element seq = null;
        MapRange range = null;
        String name = null;
        StringBuffer buffer = new StringBuffer();
        buffer.append("List: ");
        for (Iterator it = list.iterator(); it.hasNext(); ) {
            seq = (Element) it.next();
            name = seq.getAttributeValue("name");
            buffer.append(name);
            buffer.append(" ");
            
            range = XMLUtilities.getMapRange(seq, seq.getNamespace());
            try {
                buffer.append(range.getStart() + " ");
                buffer.append(range.getEnd() + " ");
            }
            catch(UnknownPositionException e) {
                System.err.println("SequenceLayoutModel.outputSequenceList(): " + e);
            }
        }
        System.out.println(buffer.toString());
    }
}