package jadex.swing.sequence;

/** SequenceModel.java: an model just like any model in swing for data storing and processing
 *
 * @version 1.0
 * @author
 */

import javax.swing.*;
import java.util.*;
import java.io.Serializable;

import jadex.xml.XMLUtilities;

import org.jdom.*; // The data structure of element of jdom will be used here instead of TreeNode

    /** 
    * This is the data model for SequenceDisplay. By using JDOM API, the data model
    * is rather simple compared to other ways.
    * @author wgm
    * @version 1.0.
    */

public class SequenceModel {
    
    private SequenceFetcher fetcher = null; // The data source of sequence

    /** Default constructor */
    public SequenceModel() {
    }

    /** Second constructor
     * @param fetcher data producer
     */
    public SequenceModel(SequenceFetcher fetcher) {
        setFetcher(fetcher);
    }

    /** Set the datasouce for this SequenceModel.
    * @param fetcher the data producer
     */
    public void setFetcher(SequenceFetcher fetcher) {
        this.fetcher = fetcher;
    }

    /** Get the data source.
    * @return the data source for this SequenceModel.
    */
    public SequenceFetcher getFetcher() {
        return fetcher;
    }

    /** Check if the element is a lead
     * @param node the checked element
     * @return result
     */
    public boolean isLeaf(Element node) {
        List list = node.getChildren();
        if ((list == null) || (list.size() == 0)) return true;
        else return false;
    }

    /** Get the root element.
    * @return root element.
    */
    public Element getRoot() {
        if (fetcher != null) return fetcher.getRoot();
        else return null;
    }

    /** Get the xml document from fetcher. */
    public Document getDocument() {
        if (fetcher != null) return fetcher.getDocument();
        else return null;
    }

    /** Get the namespace of this document */
    public Namespace getNamespace() {
        if (fetcher != null) return fetcher.getNamespace();
        else return null;
    }

    /** Search a specific sequence element by its name.
     * @param name the seached name of the sequence
     * @return the found sequence element. If ot found, return null
     */
    public Element search(String name) {
        Element root = getRoot();
        if (root == null) return null;  // If there is no data, just return
        // Check the root element
        String rootName = root.getAttributeValue("name");
        if (rootName.equalsIgnoreCase(name)) return root;

        Comparator comp = new Comparator() {
            public int compare(Object obj1, Object obj2) {
                Element seq1 = (Element) obj1;
                Element seq2 = (Element) obj2;
                String name1 = seq1.getAttributeValue("name");
                name1 = name1.toLowerCase();
                String name2 = seq2.getAttributeValue("name");
                name2 = name2.toLowerCase();
                return name1.compareTo(name2);
            }
        };

        return search(root, name, comp);
    }

    /** A recursive search routine
     * @param parent the parent element, which subsequences will be searched
     * @param seq the searched sequence element
     * @param comp the sorting comparator for searching
     * @return the found element, whose name is the same as seq's.
     */
    private Element search(Element parent, String name, Comparator comp) {
        Element found = null;
        List list = XMLUtilities.getSubSequences(parent, getNamespace());
        if ((list == null) || (list.size() == 0)) return null;
        Collections.sort(list, comp);
        int index = foundElement(list, name, 0, list.size());
        if (index >= 0) {
            found = (Element) list.get(index);
        }
        else {
            Element par = null;
            for (Iterator it = list.iterator(); it.hasNext();) {
                par = (Element) it.next();
                found = search(par, name, comp);
                if (found != null) break;
            }
        }
        return found;
    }

    private int foundElement(List list, String key, int start, int end) {
        if (start == end) return -1;
        int index = (start + end) / 2;
        Element seq = (Element) list.get(index);
        String name = seq.getAttributeValue("name");
        //System.out.println("name: " + name);
        int result = key.compareTo(name);
        if (result == 0) return index;
        if (result < 0) {
            return foundElement(list, key, start, index);
        }
        else {
            return foundElement(list, key, index + 1, end);
        }
    }

}