/*
 * SequenceRenderer.java: A renderer (just like cell render in JList or JTable) for display
 * individual sequence objects
 *
 * Created on November 21, 2000, 2:26 PM
 */

package jadex.swing.sequence;

import javax.swing.*;
import java.awt.Color;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;

import org.jdom.*;

/**
 * This customized JPanel is used for rendering the Sequence elemnet in the SequenceDisplay.
 * @author  wgm
 * @version 
 */
public class SequenceRenderer extends JPanel {

    // Color mode for this renderer
    private Color selectionForeground = Color.red;
    private Color selectionBackground = Color.orange;
    private Color background = Color.white;
    private Color foreground = Color.blue;
    private Color lineColor = Color.black;
    
    // The direction of the line: plus or minus
    private boolean isPlus; 
    
    // Layout initializeion
    private float xPadding = 1.0f;
    private float yPadding = 1.0f;
    
    private JLabel nameLabel = null; // The label used to display the text
    //private Font font = new Font("Dialog", 0, 10);
    
    /** Creates new SequenceRenderer */
    public SequenceRenderer() {
        setOpaque(true);
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        add(Box.createHorizontalGlue());
        
        nameLabel = new JLabel();
        Font font = nameLabel.getFont().deriveFont(10.0f);
        nameLabel.setFont(font);
        //nameLabel.setFont(font);
        nameLabel.setHorizontalAlignment(SwingConstants.CENTER);
        nameLabel.setVerticalAlignment(SwingConstants.CENTER);
        //add(left, "West");
        add(nameLabel);
        add(Box.createHorizontalGlue());
        //add(right, "East");
    }
    
    /** Get the SequenceRenderer.
    * @param name the name of the Sequence.
    * @param start the start position of the Sequence.
    * @param end the end position of the Sequence.
    * @param isSelected if this sequence element is selected.
    */
    public Component getSequenceRenderer(String name, long start, long end, boolean isSelected, boolean isFocused) {
        if (isSelected) {
            //setBorder(BorderFactory.createEtchedBorder());
	    setBackground(selectionBackground);
	    setForeground(selectionForeground);
        }
	else {
            //setBorder(BorderFactory.createEmptyBorder());
	    setBackground(background);
	    setForeground(foreground);
        }
        nameLabel.setText(name);
        if ((start - end) <= 0) isPlus = true;
        else isPlus = false;
	return this;
    }
    
    /** An overloaded for get SequenceRenderer.
    * @param sequence the Sequence element to be displayed.
    * @param isSelected if this sequence element is selected.
    * @param isFocused if this sequence is focused. This parameter is ignored here.
    */
    public Component getSequenceRenderer(Element sequence, boolean isSelected, boolean isFocused) {
        Attribute att = sequence.getAttribute("name");
        String name = att.getValue();
        Namespace ns = sequence.getNamespace();
        Element range = sequence.getChild("Range", ns);
        if (range == null) return null;
        String start = range.getAttribute("start").getValue();
        String end = range.getAttribute("end").getValue();
        return getSequenceRenderer(name, Long.parseLong(start), Long.parseLong(end), isSelected, isFocused);
    }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d2 = (Dimension2D) getSize();
        Insets insets = getInsets();
        float w = (float) d2.getWidth();
        float h = (float) d2.getHeight();
        g2.setPaint(getBackground());
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));
        
        // Do nothing if the rectange is too small
        if (nameLabel.getWidth() > 3 * w) {
            nameLabel.setText("");
            return;
        }
        
        int labelX1 = nameLabel.getX();
        int labelX2 = labelX1 + nameLabel.getWidth(); 
        
        g2.setPaint(lineColor);
        //g2.draw(new Line2D.Float(xPadding + insets.left, h / 2.0f, 
          //              w - xPadding - insets.right, h / 2.0f));   
        g2.draw(new Line2D.Float(xPadding + insets.left, h / 2.0f, labelX1 - 2.0f, h / 2.0f));   
        g2.draw(new Line2D.Float(labelX2 + 2.0f, h / 2.0f, w - xPadding - insets.right, h / 2.0f));   
               
        if (isPlus) {
            drawArrow(g2, new Point2D.Float(w - xPadding - insets.right - 6.0f, h / 2.0f - 3.0f),
                          new Point2D.Float(w - xPadding - insets.right, h / 2.0f),
                          new Point2D.Float(w - xPadding - insets.right - 6.0f, h / 2.0f + 3.0f));
        }
        else {
            drawArrow(g2, new Point2D.Float(xPadding + insets.left + 6.0f, h / 2.0f - 3.0f),
                          new Point2D.Float(xPadding + insets.left, h / 2.0f), 
                          new Point2D.Float(xPadding + insets.left + 6.0f, h / 2.0f + 3.0f));
        }
    }
    
    /** A helper method to draw an arror given the coordinates of three poins */
    private void drawArrow(Graphics2D g2, Point.Float point1, Point2D.Float point2, Point.Float point3) {
        g2.draw(new Line2D.Float(point1, point2));
        g2.draw(new Line2D.Float(point2, point3));
    }
}