/*
 * SequenceSelectionEvent.java: A sequence selection event object. When in use, first have to 
 * check whether this event is for selectin or for deselection.
 *
 * Created on November 22, 2000, 11:42 AM
 */

package jadex.swing.sequence;

import java.util.*;
import org.jdom.Element;
/**
 * A sequence selection event object. For use, first set whether this event is
 * for selectin or for deselection.
 * @author  wgm
 * @version 1.0
 */
public class SequenceSelectionEvent extends EventObject {

    private Element sequence = null; // The selected sequence
    private boolean isSelected; // Check if this event is for selection or for deselection
    private boolean isMultiSelected; // check is the event is multiselected
    private List list = null; // A list of de/selected sequences for multiselection
    
    /** Creates new SequenceSelectionEvent 
     * @param source An JComponent, should be a sequencedisplay
     * @param sequence the selected element
     * @param isSelected the flag for selecting or deselecting
     */
    public SequenceSelectionEvent(Object source, Element sequence, boolean isSelected) {
        super(source);
        setSelected(isSelected);
        setSequence(sequence);
        setMultiSelected(false);
    }
    
    /** Second constructor 
    * @param source An JComponent, should be a sequencedisplay
     * @param sequence the selected element
     * @param isMultiSelected the flag for multiselecting
    */
    public SequenceSelectionEvent(Object source, List list, boolean isSelected) {
        super(source);
        setSelected(isSelected);
        setSequences(list);
        setMultiSelected(true);
    }
    
    /** Set if this is used in selection.
    * @param is true for selection.
    */
    public void setSelected(boolean is) {
        isSelected = is;
    }
    
    /** Query if this event is used in selection.
    * @return true for selection, false for de-selection.
    */
    public boolean isSelected() {
        return isSelected;
    }
    
    /** Set if there are more than one selected sequences.
    * @param isMulti true for multi-selection.
    */
    public void setMultiSelected(boolean isMulti) {
        this.isMultiSelected = isMulti;
    }
    
    /** Query if this event is used in multi-selection.
    * @return true for multi-selection.
    */
    public boolean isMultiSelected() {
        return isMultiSelected;
    }

    /** Set the selected sequence.
    * @param sequence the selected sequence.
    */
    public void setSequence(Element sequence) {
        this.sequence = sequence;
    }
    
    /** Get the selected sequence.
    * @return the selected sequence.
    */
    public Element getSequence() {
        return this.sequence;
    } 
    
    /** For multiselecton, set the selected sequences.
    * @param the list of selected sequence elements.
    */
    public void setSequences(List list) {
        this.list = list;
    }
    
    /** For multiselection, get the selected sequences.
    * @return the list of selected sequence elements.
    */
    public List getSequences() {
        return this.list;
    }  
}