/*
 * SequenceSelectionModel.java: The class processes and stores selections of sequnce on the sequence
 * display. 
 *
 * Created on November 25, 2000, 4:19 PM
 */

package jadex.swing.sequence;

import java.io.Serializable;
import java.util.*;
import org.jdom.Element;

/**
 * This class processes and stores selections of sequnce on the sequence display. 
 * is supported.
 * @author  wgm
 * @version 1.0
 */
public class SequenceSelectionModel {
    private List selectionListeners = new ArrayList(); // All selection listener will be here
    private Element selectedElement = null; // The selected element or the first one on multi-selection
    private List selection =  new ArrayList(); // The selected elements during multi-selection. Hopefully, not be too big!
    
    /** Creates new SequenceSelectionModel */
    public SequenceSelectionModel() {
    }
    
    /** Set a selected Sequence element.
    * @param e the selected Sequence element.
    */
    public void setSelectedElement(Element e) {
        selectedElement = e;
        selection.add(e);
        fireSelectionValueChanged(e, true);
    }
    
    /** Get the selected element.
    * @return the selected sequence element.
    */
    public Element getSelectedElement() {
        return selectedElement;
    }
    
    /** Set a list of selected elements.
    * @param list the list of selected elements.
    */
    public void setSelectedElements(List list) {
        this.selection = list;
        fireSelectionValueChanged(list, true);
    }
    
    /** DeSelect a list of elements.
    * @param list the list of elements to be removed from the selections.
    */
    public void deSelectedElements(List list) {
        this.selection.removeAll(list);
        fireSelectionValueChanged(list, false);
    }
    
    /** Remove a selected sequence.
    * @param sequence the sequence element to be removed from the selections.
    */
    public void deSelectSequence(Element sequence) {
        if (selection.contains(sequence)) {
            selectedElement = sequence;
            selection.remove(sequence);
            fireSelectionValueChanged(sequence, false);
        }
    }
    /** Return the selected or deselected element.
    * @return the element to be selected or de-selected.
    */
    public Element getSelected() {
        return selectedElement;
    }
    
    /** Remove all selected sequence from selection */
    public void clearSelection() {
        if (selection.size() > 0) {
            deSelectedElements(selection);
        }
    }
    
    /** Register a selection listener */
    public void addSelectionListener(SequenceSelectionListener listener) {
        selectionListeners.add(listener);
    }
    
    /** Unregister a selection listener */
    public void removeSelectionListener(SequenceSelectionListener listener) {
        selectionListeners.remove(listener);
    }
    
    /** Notify all selection listeners a selection has changed 
     * @param seq selected or deselected sequence
     * @param isSelected true for selection and false for deselection
     */ 
    protected void fireSelectionValueChanged(Element seq, boolean isSelected) {
        if (selectionListeners.size() == 0) return; // do nothing if no listeners registered
        SequenceSelectionEvent e = new SequenceSelectionEvent(this, seq, isSelected);
        for (Iterator it = selectionListeners.iterator(); it.hasNext(); ) {
            SequenceSelectionListener l = (SequenceSelectionListener) it.next();
            l.valueChanged(e);
        }
    }
    
    /** Notify all selection listeners a selection has changed 
     * @param list selected or deselected sequence
     * @param isSelected true for selection and false for deselection
     */ 
    protected void fireSelectionValueChanged(List list, boolean isSelected) {
        if (selectionListeners.size() == 0) return; // do nothing if no listeners registered
        SequenceSelectionEvent e = new SequenceSelectionEvent(this, list, isSelected);
        for (Iterator it = selectionListeners.iterator(); it.hasNext(); ) {
            SequenceSelectionListener l = (SequenceSelectionListener) it.next();
            l.valueChanged(e);
        }
    }
    
    /** Get the selected list
     * @return a list containing all selected sequences
     */
    public List getSelection() {
        return selection;
    }
    
    /** This method will not fire a SequneceSelectionEvent, and is for fine control of display.
     * @param list the list of selected sequences.
    */
    public void setSelection(List list) {
        this.selection = list;
    }
}