/*
 * SequenceTreeDisplay.java: Display a sequence in a JTree based on an xml file
 *
 * Created on December 4, 2000, 2:29 PM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import org.jdom.*;

import jadex.swing.map.MapRange;
import jadex.swing.map.UnknownPositionException;
import jadex.xml.XMLUtilities;

/**
 * Display a sequence in a JTree based on an xml document or element.
 * @author  wgm
 * @versionb 1.0
 */
public class SequenceTreeDisplay extends JPanel {

    private JTree tree = null;
    private TreeModel treeModel = null;
    //private Element root = null;  // The root node of xml document
    private JTabbedPane contentPane = null; // The content panel
    private JScrollPane sp = null;
    //private JSplitPane split = null;
    
    private TreeSelectionListener selectionListener = null;

    /** Creates new SequenceTreeDisplay */
    public SequenceTreeDisplay() {
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(new BorderLayout());
        JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);

        sp = new JScrollPane();
        sp.setMinimumSize(new Dimension(200, 300));
        split.add(sp);
        
        contentPane = new JTabbedPane();
        contentPane.setBackground(Color.white);
        contentPane.setPreferredSize(new Dimension(400, 300));
        split.add(contentPane);

        add(split, "Center");
    }

    /** Set root element of the xml document to be displayed. */
    public void setRootElement(Element root) {
        //this.root = root;
        TreeNode rootNode = new SequenceTreeNode(root);
        treeModel = new DefaultTreeModel(rootNode);
        tree = new JTree(treeModel);
        //tree = new JTree();
        tree.setLargeModel(true);
        sp.setViewportView(tree);
        selectionListener = new SequenceTreeSelectionHandler();
        tree.addTreeSelectionListener(selectionListener);
    }

    /** Get the root element */
    //public Element getRootElemt() {
      //  return root;
    //}

    /** Get the JTree displaying these sequence elements.
    * @return the JTree displaying sequence elements.
    */
    public JTree getTree() {
        return this.tree;
    }
    
    /** Clear up. */
    public void clear() {
        tree.removeTreeSelectionListener(selectionListener);
        tree.removeAll();
        tree.setModel(null);
        tree.setSelectionModel(null);
        tree.getUI().uninstallUI(tree);
        sp.removeAll();
        treeModel = null;
        contentPane.removeAll();
        contentPane.getUI().uninstallUI(contentPane);
        //root = null;
    }
    
    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("Usage java SequenceTreeDisplay xmlFile");
            System.exit(0);
        }
        SequenceFetcher fetcher = new SequenceXMLFetcher(args[0]);
        SequenceModel model = new SequenceModel(fetcher);
        Element root = model.getRoot();
        SequenceTreeDisplay treeDisplay = new SequenceTreeDisplay();
        treeDisplay.setRootElement(root);

        JFrame f = new JFrame("Sequence Tree Display");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        f.getContentPane().add(treeDisplay, "Center");
        f.setSize(800, 300);
        f.setVisible(true);
    }

    // A tree selection listener
    class SequenceTreeSelectionHandler implements TreeSelectionListener {
        public void valueChanged(TreeSelectionEvent e) {
            contentPane.removeAll();
            TreePath[] paths = tree.getSelectionPaths();
            if ((paths == null) || (paths.length == 0)) return ;

            SequenceTreeNode node = null;
            Element seq = null;
            SequenceElementTableView panel = null;
            for (int i = 0; i < paths.length; i++) {
                node = (SequenceTreeNode) paths[i].getLastPathComponent();
                seq = node.getSequence();
                panel = new SequenceElementTableView();
                panel.setSequence(seq);
                contentPane.addTab(seq.getAttributeValue("name"), panel);
                if (i == (paths.length - 1)) contentPane.setSelectedComponent(panel);
            }
        }
    }
}