/*
 * SequenceViewApplet.java: The applet for displaying sequence view in the browser
 *
 * Created on November 14, 2000, 10:17 AM
 */

package jadex.swing.sequence;

import javax.swing.*;
import java.applet.*;
import java.util.jar.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

import org.jdom.*;

/**
 * The applet for displaying a SequenceView in the browser. This applet is embeded 
* in the browser window.
 * @author  wgm
 * @version 1.0
 */
public class SequenceViewApplet extends JApplet {

    // The default SAX parser used in the Applets.
    private final String saxdriver = "org.brownell.xml.aelfred2.SAXDriver";   
        
    // The ViewFactory for setting up all GUIs for the SequenceDisplay.
    private SequenceViewFactory viewFactory = null;
    /** Initialization method to set up all layout */
    public void init() {
        viewFactory = new SequenceViewFactory(this);
        IndividualSequenceView.setApplet(this);
        
        String name = null;
        String chromosome = null;
        try { 
            name = getParameter("sequence"); 
            chromosome = getParameter("chromosome");
        } // This line has to be changed!
        catch(Exception e) { 
            chromosome = "Chromosome_I"; 
        }
        if (chromosome == null) chromosome = "Chromosome_I";
        setSequence(chromosome, name);       
    }
    
    /** Just a check method for monitoring the memory usage.
    */
    public void start() {
        super.start();
        long free = Runtime.getRuntime().freeMemory();
        long total = Runtime.getRuntime().totalMemory();
        System.out.println("Memory: total " + total + " used " + (total - free) + " free " + free);
    }
    
    public void stop() {
        viewFactory.clear();
        viewFactory = null;
        getContentPane().removeAll();
        gc();
    }
    
    private void gc() {
        try {
            System.gc();
            Thread.currentThread().sleep(100);
            System.runFinalization();
            Thread.currentThread().sleep(100);
            System.gc();
            Thread.currentThread().sleep(100);
            System.runFinalization();
            Thread.currentThread().sleep(100);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
        
   /** Set the chromsome name and display it on the display 
     * @param name the chromsome name
    */
    public void setSequence(String chromosome, String name) {
        SequenceFetcher fetcher = new SequenceXMLFetcher(getInputStream(chromosome.toLowerCase()), saxdriver);
        viewFactory.setSequenceFetcher(fetcher);
        viewFactory.search(name);
    }
    
    private InputStream getInputStream(String name) {
        JarURLConnection jarConnection = null;
        try {
            URL jarUrl = getCodeBase();
            URL url = new URL("jar:" + jarUrl.toString() + name + ".jar!/");
            jarConnection = (JarURLConnection) url.openConnection();           
            JarFile file = jarConnection.getJarFile();
            JarEntry entry = file.getJarEntry(name + ".xml");
            return file.getInputStream(entry);
            
        }
        catch(IOException e) {           
            System.err.println("SequenceViewApplet.getInputStream(): " + e);
            return null;
        } 
        catch(NullPointerException e1) {
            try {
                JarFile file = new JarFile(name + ".jar");
                JarEntry entry = file.getJarEntry(name + ".xml");
                return file.getInputStream(entry);
            }
            catch(IOException e2) {
                System.err.println("SequenceViewApplet.getInputStream(): " + e2);
                return null;
            }
        }
        finally {jarConnection = null;}
    }
        
    public static void main(String args[]) {
        SequenceViewApplet applet = new SequenceViewApplet();
        applet.init();
        applet.start();
        JFrame f = new JFrame("Applet View");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        f.getContentPane().add(applet, "Center");
        f.setSize(1200, 800);
        f.setVisible(true);
    }
}