/*
 * SequenceMapXMLDump.java: Dump the whole chromosome sequence map from acedb in xml. To
 * increate the efficience of this class, aql instead of Jade API is used.
 *
 * Created on November 16, 2000, 3:11 PM
 */

package jadex.xml;

import jade.db.*;
import java.util.*;
import java.util.jar.*;
import java.io.*;

// For XML purpose
import org.jdom.*;
import org.jdom.output.*;

/**
 *
 * @author  Valued Sony Customer
 * @version
 */
public class SequenceMapXMLDump extends Object {
    // These final variables should be moved to somewhere else
    public static final String ROOT_NAME = "Chromosome"; // The root name of this xml
    public static final String NAME_SPACE = "http://www.wormbase.org"; // The namespace of this xml
    public static final String DTD_NAME = "chromosome.dtd"; // The dtd file name

    private JSAceAccessor accessor = null; // The AceAccessor to query the acedb
    
    private Document document = null; // The XML document
    private Element root = null; // The element refered to be changed
    
    private SequenceComparator comp = new SequenceComparator();
    
    class SequenceComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            Element seq1 = (Element) o1;
            Element seq2 = (Element) o2;
            long[] pos1 = XMLUtilities.getPositions(seq1, seq1.getNamespace());
            long[] pos2 = XMLUtilities.getPositions(seq2, seq2.getNamespace());
            long pos11 = (pos1[0] + pos1[1]) / 2;
            long pos21 = (pos2[0] + pos2[1]) / 2;
            int rtn = 0;
            if (pos11 > pos21) rtn = 1;
            else if (pos11 < pos21) rtn = -1;
            return rtn;
        }
    }

    /** Default constructor: Creates new SequenceMapXMLDump */
    public SequenceMapXMLDump() {
        initialize(true);
    }

    /** Second constructor: Based on the name of the chromosome */
    public SequenceMapXMLDump(String name) {
        initialize(true);
        setName(name);
    }

    /** Third constructor: provide the acedb server name and port number */
    public SequenceMapXMLDump(String host, int port, boolean withDTD) {
        initialize(withDTD);
        JSAceAccessor acc = new JSAceAccessor(host, port);
        setAccessor(acc);
    }

    /** Fourth constructor: Name and Acedb information as the argument */
    public SequenceMapXMLDump(String name, String host, int port, boolean withDTD) {
        initialize(withDTD);
        JSAceAccessor acc = new JSAceAccessor(host, port);
        setName(name);
        setAccessor(acc);
    }

    /** Several initializing activities should be here */
    private void initialize(boolean withDTD) {
        // Create the root element and add DTD and namespace
        root = new Element(ROOT_NAME, NAME_SPACE);
        if (withDTD) {
            DocType dtd = new DocType(ROOT_NAME, DTD_NAME);
            document = new Document(root, dtd);
        }
        else document = new Document(root);
    }

    /** Get and set the chromosome name */
    public void setName(String name) {
        root.removeChildren();
        root.removeAttribute("name");
        root.addAttribute("name", name);
    }

    /** Set and get AceAccessor: Query engine */
    public void setAccessor(JSAceAccessor acc) {
        this.accessor = acc;
    }

    public JSAceAccessor getAccessor() {
        return this.accessor;
    }

    /** A template method to dump the sequnece from acedb */
    public void dump(String name) {
        aqlSequenceDump(root, name);
    }

    /** Dump sequence by aql from acedb */
    public void aqlSequenceDump(Element element, String name) {
        String query = "aql -h select a, a[1], a[2] from a in object(\"Sequence\", \""
        + name + "\")->subsequence";
        String result = queryAceDB(query);
        int seqCount = parseSubResult(element, result);
        if (seqCount == 0) geneDump(element, name); // Check if we can find a gene structure here.
        subSequenceDump(element);
    }

    /** A helper to fetch sequence */
    private void subSequenceDump(Element element) {
        Element sub = element.getChild("Subsequence");
        if (sub != null) {
            java.util.List list = sub.getChildren();
            for (Iterator it = list.iterator(); it.hasNext(); ) {
                Element sequence = (Element) it.next();
                String name = sequence.getAttribute("name").getValue();
                aqlSequenceDump(sequence, name);
            }
        }
    }

    /** A help method to parse a subsequence query result */
    private int parseSubResult(Element element, String result) {
        BufferedReader reader = new BufferedReader(new StringReader(result));
        String line = null;
        java.util.List list = new ArrayList();
        long[] origion = XMLUtilities.getPositions(element, element.getNamespace());
        try {
            while ((line = reader.readLine())!= null) {
                if (line.startsWith("Format")) continue; // Format line
                if (line.startsWith("//")) continue; // active counting line
                StringTokenizer tokenizer = new StringTokenizer(line);
                String name = tokenizer.nextToken(); // First is the name of the sequence
                String start = tokenizer.nextToken(); // Second is the start point
                String end = tokenizer.nextToken(); // Third is the end point
                Element elm = new Element("Sequence");
                elm.addAttribute("name", name);
                Element range = createRange(start, end, origion);
                elm.addContent(range);
                list.add(elm);
            }
        }
        catch(IOException e) {
            System.err.println(e);
        }
        int count = list.size();
        if (count > 0) {
            Element sub = new Element("Subsequence");
            Collections.sort(list, comp);
            sub.setChildren(list);
            element.addContent(sub);
        }
        return count;
    }

    /** A help to create a abosolute range */
    private Element createRange(String start, String end, long[] origion) {
        long s = Long.parseLong(start);
        long e = Long.parseLong(end);
        long length = e - s;
        if (length < 0) length --;
        else length ++;
        
        // origins is a 2 element array: the first is the start pos, the second is the end pos
        if (origion != null) {
            if (origion[0] < origion[1]) {
                s = s + origion[0] - 1;
                e = e + origion[0] - 1;
            }
            else {
                s = origion[0] - s + 1;
                e = origion[0] - e + 1;
            }
        }
        
        Element range = new Element("Range");
        range.addAttribute("start", s + "");
        range.addAttribute("end", e + "");
        range.addAttribute("length", length + "");
        return range;
    }
    
    /** Dump a possible gene */
    public void geneDump(Element element, String name) {
        String query = "aql -h select a[0], a[1] from a in object(\"Sequence\", \"" +
        name + "\")->Source_exons";
        String result = queryAceDB(query);
        parseExonResult(element, name, result);
    }

    /** Gene exon parser */
    private void parseExonResult(Element element, String name, String result) {
        int count = 0;
        BufferedReader reader = new BufferedReader(new StringReader(result));
        String line = null;
        Element exons = new Element("Exons");
        try {
            while ((line = reader.readLine())!= null) {
                if (line.startsWith("Format")) continue; // Format line
                if (line.startsWith("//")) continue; // Acive object count
                StringTokenizer tokenizer = new StringTokenizer(line);
                String start = tokenizer.nextToken(); // First is the start position
                String end = tokenizer.nextToken(); // Second is the end position
                int length = Integer.parseInt(end) - Integer.parseInt(start); // Integer should be big enough for exons
                if (length < 0) length --;
                else length ++;
                Element exon = new Element("Exon");
                exon.addAttribute("start", start);
                exon.addAttribute("end", end);
                exon.addAttribute("length", length + "");
                exons.addContent(exon);
                count ++;
            }
        }
        catch (IOException e) {
            System.err.println("SequenceMapXMLDump.parseExonResult(): " + e);
        }
        if (count == 0) return ; // There are no exons, so this sequence is not a gene

        Element remark = remarkDump(name);
              
        Element gene = new Element("Gene");
        if (remark != null) gene.addContent(remark);
        gene.addContent(exons);
        element.addContent(gene);
    }

    /** A refactored method to dump DB_remark for gene */
    private Element remarkDump(String name) {
        // Try to get remark information for this gene if any
        String query = "aql -h select a[0] from a in object(\"Sequence\", \"" +
                            name + "\")->DB_Remark";
        String result1 = queryAceDB(query);
        BufferedReader reader = new BufferedReader(new StringReader(result1));
        StringBuffer remark = new StringBuffer();
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                if (line.startsWith("Format")) continue;
                if (line.startsWith("//")) continue;
                remark.append(line + "\n");
            }
        }
        catch (IOException e) {
            System.err.println("SequenceMapXMLDump.parseExonResult().1: " + e);
        }
        String remarkS = remark.toString().trim();
        Element remarkE = null;
        if (remarkS.length() > 0) {
            remarkE = new Element("Remark");
            remarkE.setText(remarkS);
        }
        return remarkE;
    }
    
    /** Query AceDB */
    private String queryAceDB(String query) {
        accessor.write(query);
        return accessor.read();
    }

    /** Get the document to display the contents */
    public Document getDocument() {
        return this.document;
    }

    /** A utility to print out document at any time */
    public void output(String name) {
        try {
            FileOutputStream output = new FileOutputStream(name.toLowerCase() + ".xml");
            output(output);
        }
        catch (IOException e) {
            System.err.println("SequenceMapXMLDump.output(): " + e);
        }
    }

    public void output(OutputStream output) {
        XMLOutputter out = new XMLOutputter("   ", true);
        try {
            out.output(document, output);
        }
        catch (IOException e) {
            System.err.println("SequenceMapXMLDump.output(): " + e);
        }
    }
    
    /** A utility to zip the xml files */
    public void zipIt(String name) {
        try {
            String n = name.toLowerCase();
            BufferedInputStream input = new BufferedInputStream(new FileInputStream(n + ".xml"));
            JarOutputStream jarout = new JarOutputStream(new FileOutputStream(n + ".jar"));
            jarout.putNextEntry(new JarEntry(n + ".xml"));
            PrintStream out = new PrintStream(jarout);
            byte[] buffer = new byte[1024];
            int read = -1;
            while((read = input.read(buffer, 0, buffer.length)) != -1) {
                out.write(buffer, 0, read);
                //out.flush();
            }
            input.close();
            jarout.close();
            out.close();
        }
        catch(IOException e) {
            System.err.println("SequenceMapXMLDump.zipIt(): " + e);
        }
    }
    
    /** This method to dump all chromosomes from acedb into six xml and six jar files */
    public void dumpAll() {
        String[] names = {"CHROMOSOME_I", "CHROMOSOME_II", "CHROMOSOME_III", "CHROMOSOME_IV",
            "CHROMOSOME_V", "CHROMOSOME_X"};
        String name = null;
        for (int i = 0; i < names.length; i++) {
            name = names[i].toLowerCase();
            createXMLFile(name);
        }
    }
    
    /** A template to create xml files */
    public void createXMLFile(String name) {
        setName(name);
        dump(name);
        output(name);
        zipIt(name);
    }
    
    /** A tester */
    public static void main(String[] args) {
        if (args.length == 3) {
            // Get all information from the arguments for dump
            String name = args[2].toLowerCase();
            SequenceMapXMLDump dump = new SequenceMapXMLDump(args[0], Integer.parseInt(args[1]), false);
            dump.createXMLFile(name);
        }
        else if (args.length == 2) {
            SequenceMapXMLDump dump = new SequenceMapXMLDump(args[0], Integer.parseInt(args[1]), false);
            dump.dumpAll();
        }
    }
}